@@ -1,5 +1,5 @@
 /* libaudit.c -- 
- * Copyright 2004-2006 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2007 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -34,22 +34,62 @@
 #include <errno.h>
 #include <sys/poll.h>
 #include <sys/utsname.h>
-#include <netinet/in.h> // inet6 addrlen
-#include <netdb.h>	// gethostbyname
-#include <arpa/inet.h>	// inet_ntop
 #include <fcntl.h>	/* O_NOFOLLOW needs gnu defined */
 
 #include "libaudit.h"
 #include "private.h"
 
+/* #defines for the audit failure query  */
+#define CONFIG_FILE "/etc/libaudit.conf"
+
+/* Local prototypes */
+struct nv_pair
+{
+        const char *name;
+        const char *value;
+};
+
+struct kw_pair
+{
+        const char *name;
+        int (*parser)(const char *, int);
+};
+
+struct nv_list
+{
+        const char *name;
+        int option;
+};
+
+struct libaudit_conf
+{
+        auditfail_t failure_action;
+};
+
+static const struct nv_list failure_actions[] =
+{
+  {"ignore",		FAIL_IGNORE },
+  {"log",		FAIL_LOG },
+  {"terminate",		FAIL_TERMINATE },
+  { NULL,		0 }
+};
 
 int audit_archadded = 0;
 int audit_syscalladded = 0;
 unsigned int audit_elf = 0U;
+static struct libaudit_conf config;
+
+static int audit_failure_parser(const char *val, int line);
 static int audit_name_to_uid(const char *name, uid_t *uid);
 static int audit_name_to_gid(const char *name, gid_t *gid);
 
+static const struct kw_pair keywords[] =
+{
+  {"failure_action",	audit_failure_parser },
+  { NULL,		NULL }
+};
 
+/* FIXME: Make this static again after deprecated functions no longer need it */
 int audit_priority(int xerrno)
 {
 	/* If they've compiled their own kernel and did not include
@@ -70,6 +110,218 @@ int audit_request_status(int fd)
 	return rc;
 }
 
+/*
+ * Set everything to its default value
+ */
+static void clear_config(void)
+{
+        config.failure_action = FAIL_IGNORE;
+}
+
+/* Get 1 line from file */
+static char *get_line(FILE *f, char *buf, size_t len)
+{
+	if (fgets(buf, len, f)) {
+		/* remove newline */
+		char *ptr = strchr(buf, 0x0a);
+		if (ptr)
+			*ptr = 0;
+		return buf;
+	}
+	return NULL;
+}
+
+static int nv_split(char *buf, struct nv_pair *nv)
+{
+	/* Get the name part */
+	char *ptr, *saved=NULL;
+
+	nv->name = NULL;
+	nv->value = NULL;
+	ptr = strtok_r(buf, " ", &saved);
+	if (ptr == NULL)
+		return 0; /* If there's nothing, go to next line */
+	if (ptr[0] == '#')
+		return 0; /* If there's a comment, go to next line */
+	nv->name = ptr;
+
+	/* Check for a '=' */
+	ptr = strtok_r(NULL, " ", &saved);
+	if (ptr == NULL)
+		return 1;
+	if (strcmp(ptr, "=") != 0)
+		return 2;
+
+	/* get the value */
+	ptr = strtok_r(NULL, " ", &saved);
+	if (ptr == NULL)
+		return 1;
+	nv->value = ptr;
+
+	/* Make sure there's nothing else */
+	ptr = strtok_r(NULL, " ", &saved);
+	if (ptr)
+		return 1;
+
+	/* Everything is OK */
+	return 0;
+}
+
+static const struct kw_pair *kw_lookup(const char *val)
+{
+        int i = 0;
+        while (keywords[i].name != NULL) {
+                if (strcasecmp(keywords[i].name, val) == 0)
+                        break;
+                i++;
+        }
+        return &keywords[i];
+}
+
+static int audit_failure_parser(const char *val, int line)
+{
+	int i;
+
+	audit_msg(LOG_DEBUG, "audit_failure_parser called with: %s", val);
+	for (i=0; failure_actions[i].name != NULL; i++) {
+		if (strcasecmp(val, failure_actions[i].name) == 0) {
+			config.failure_action = failure_actions[i].option;
+			return 0;
+		}
+	}
+	audit_msg(LOG_ERR, "Option %s not found - line %d", val, line);
+	return 1;
+}
+
+/*
+ *  Read the /etc/libaudit.conf file and all tunables.
+ */
+static int load_libaudit_config(const char *path)
+{
+	int fd, rc, lineno = 1;
+	struct stat st;
+	FILE *f;
+	char buf[128];
+
+	/* open the file */
+	rc = open(path, O_NOFOLLOW|O_RDONLY);
+	if (rc < 0) {
+		if (errno != ENOENT) {
+			audit_msg(LOG_ERR, "Error opening %s (%s)",
+				path, strerror(errno));
+			return 1;
+		}
+		audit_msg(LOG_WARNING,
+			"Config file %s doesn't exist, skipping", path);
+		return 0;
+	}
+	fd = rc;
+
+	/* check the file's permissions: owned by root, not world writable,
+	 * not symlink.
+	 */
+	audit_msg(LOG_DEBUG, "Config file %s opened for parsing", path);
+	if (fstat(fd, &st) < 0) {
+		audit_msg(LOG_ERR, "Error fstat'ing %s (%s)",
+			path, strerror(errno));
+		return 1;
+	}
+	if (st.st_uid != 0) {
+		audit_msg(LOG_ERR, "Error - %s isn't owned by root", path);
+		return 1;
+	}
+	if ((st.st_mode & S_IWOTH) == S_IWOTH) {
+		audit_msg(LOG_ERR, "Error - %s is world writable", path);
+		return 1;
+	}
+	if (!S_ISREG(st.st_mode)) {
+		audit_msg(LOG_ERR, "Error - %s is not a regular file", path);
+		return 1;
+	}
+
+	/* it's ok, read line by line */
+	f = fdopen(fd, "r");
+	if (f == NULL) {
+		audit_msg(LOG_ERR, "Error - fdopen failed (%s)",
+			strerror(errno));
+		return 1;
+	}
+
+	while (get_line(f, buf, sizeof(buf))) {
+		// convert line into name-value pair
+		const struct kw_pair *kw;
+		struct nv_pair nv;
+		rc = nv_split(buf, &nv);
+		switch (rc) {
+			case 0: // fine
+				break;
+			case 1: // not the right number of tokens.
+				audit_msg(LOG_ERR,
+				"Wrong number of arguments for line %d in %s",
+					lineno, path);
+				break;
+			case 2: // no '=' sign
+				audit_msg(LOG_ERR,
+					"Missing equal sign for line %d in %s",
+					lineno, path);
+				break;
+			default: // something else went wrong...
+				audit_msg(LOG_ERR,
+					"Unknown error for line %d in %s",
+					lineno, path);
+				break;
+		}
+		if (nv.name == NULL) {
+			lineno++;
+			continue;
+		}
+		if (nv.value == NULL)
+			return 1; 
+
+		/* identify keyword or error */
+		kw = kw_lookup(nv.name);
+		if (kw->name == NULL) {
+			audit_msg(LOG_ERR,
+				"Unknown keyword \"%s\" in line %d of %s",
+				nv.name, lineno, path);
+			fclose(f);
+			return 1;
+		}
+
+		/* dispatch to keyword's local parser */
+		rc = kw->parser(nv.value, lineno);
+		if (rc != 0) {
+			fclose(f);
+			return 1; // local parser puts message out
+		}
+
+		lineno++;
+	}
+
+	fclose(f);
+	return 0;
+}
+
+
+/*
+ * This function is called to get the value of the failure_action 
+ * tunable stored in /etc/libaudit.conf.  The function returns 1 if
+ * the tunable is not found or there is an error. If the tunable is found,
+ * 0 is returned the the tunable value is saved in the failmode parameter.
+ */
+int get_auditfail_action(auditfail_t *failmode)
+{
+	clear_config();
+
+	if (load_libaudit_config(CONFIG_FILE)) {
+		*failmode = config.failure_action;
+		return 1;
+	}
+
+	*failmode = config.failure_action;
+	return 0;
+}
+
 int audit_set_enabled(int fd, uint32_t enabled)
 {
 	int rc;
@@ -207,7 +459,7 @@ int audit_set_rate_limit(int fd, uint32_t limit)
 	rc = audit_send(fd, AUDIT_SET, &s, sizeof(s));
 	if (rc < 0)
 		audit_msg(audit_priority(errno),
-			"Error sending rate limit request (%s)",
+			"Error sending rate limit request (%s)", 
 			strerror(-rc));
 	return rc;
 }
@@ -228,12 +480,12 @@ int audit_set_backlog_limit(int fd, uint32_t limit)
 	return rc;
 }
 
-int audit_request_rules_list(int fd)
+int audit_request_rules_list_data(int fd)
 {
-	int rc = audit_send(fd, AUDIT_LIST, NULL, 0);
-	if (rc < 0)
+	int rc = audit_send(fd, AUDIT_LIST_RULES, NULL, 0);
+	if (rc < 0 && rc != -EINVAL)
 		audit_msg(audit_priority(errno),
-			"Error sending rule list request (%s)", 
+			"Error sending rule list data request (%s)", 
 			strerror(-rc));
 	return rc;
 }
@@ -248,106 +500,167 @@ int audit_request_signal_info(int fd)
 	return rc;
 }
 
-/* This function takes a watch and allocates a transport memblk
- * suitable for sending to the kernel. The caller must free it.
- * Returns NULL on failure. */
-static void *watch_to_transport(struct audit_watch *req, 
-		unsigned int size)
+int audit_update_watch_perms(struct audit_rule_data *rule, int perms)
 {
-	struct watch_transport r;
+	int i, done=0;
 
-	void *p;
-	void *memblk = malloc(size);
-	if (!memblk)
-		return NULL;
+	if (rule->field_count < 1)
+		return -1;
 
-	r.dev_major = req->dev_major;
-	r.dev_minor = req->dev_minor;
-	r.perms = req->perms; 
-	r.valid = 0;
-	r.pathlen = req->namelen;
-	r.fklen =  req->fklen;
-
-	p = mempcpy(memblk, &r, sizeof(r));
-	p = mempcpy(p, req->filterkey, req->fklen);
-	memcpy(p, req->name, req->namelen);
-	return memblk;
-}
-
-int audit_insert_watch(int fd, struct audit_watch *req)
-{
-	unsigned int size = sizeof(struct watch_transport) + req->namelen +
-			req->fklen;
-	void *memblk = watch_to_transport(req, size);
-
-	int rc = audit_send(fd, AUDIT_WATCH_INS, memblk, size);
-	if (rc < 0)
-		audit_msg(audit_priority(errno),
-			"Error sending watch insert request (%s)",
-			strerror(-rc));
-	free(memblk);
-	return rc;
-}
-
-int audit_remove_watch(int fd, struct audit_watch *req)
-{
-	unsigned int size = sizeof(struct watch_transport) + req->namelen +
-			req->fklen;
-	void *memblk = watch_to_transport(req, size);
-
-	int rc = audit_send(fd, AUDIT_WATCH_REM, memblk, size);
-	if (rc < 0)
-		audit_msg(audit_priority(errno),
-			"Error sending watch remove request (%s)", 
-			strerror(-rc));
-	free(memblk);
-	return rc;
-}
-
-int audit_request_watch_list(int fd)
-{
-	int rc = audit_send(fd, AUDIT_WATCH_LIST, NULL, 0);
-	if (rc < 0 && rc != -EINVAL) {
-		audit_msg(audit_priority(errno),
-			"Error sending watch list request (%s)",
-			strerror(-rc));
+	// First see if we have an entry we are updating
+	for (i=0; i< rule->field_count; i++) {
+		if (rule->fields[i] == AUDIT_PERM) {
+			rule->values[i] = perms;
+			done = 1;
+		}
 	}
-	return rc;
+	if (!done) {
+		// If not check to see if we have room to add a field
+		if (rule->field_count >= (AUDIT_MAX_FIELDS - 1))
+			return -2;
+	
+		// Add the perm
+		rule->fields[rule->field_count] = AUDIT_PERM;
+		rule->fieldflags[rule->field_count] = AUDIT_EQUAL;
+		rule->values[rule->field_count] = perms;
+		rule->field_count++;
+	}
+	return 0;
 }
 
-int audit_add_rule(int fd, struct audit_rule *rule, int flags, int action)
+int audit_add_watch(struct audit_rule_data **rulep, const char *path)
+{
+	return audit_add_watch_dir(AUDIT_WATCH, rulep, path);
+}
+
+int audit_add_dir(struct audit_rule_data **rulep, const char *path)
+{
+	return audit_add_watch_dir(AUDIT_DIR, rulep, path);
+}
+
+int audit_add_watch_dir(int type, struct audit_rule_data **rulep,
+			const char *path)
+{
+	size_t len = strlen(path);
+	struct audit_rule_data *rule = *rulep;
+
+	if (rule && rule->field_count) {
+		audit_msg(LOG_ERR, "Rule is not empty\n");
+		return -1;
+	}
+	if (type != AUDIT_WATCH && type != AUDIT_DIR) {
+		audit_msg(LOG_ERR, "Invalid type used\n");
+		return -1;
+	}
+
+	*rulep = realloc(rule, len + sizeof(*rule));
+	if (*rulep == NULL) {
+		free(rule);
+		audit_msg(LOG_ERR, "Cannot realloc memory!\n");
+		return -1;
+	}
+	rule = *rulep;
+	memset(rule, 0, len + sizeof(*rule));
+
+	rule->flags = AUDIT_FILTER_EXIT;
+	rule->action = AUDIT_ALWAYS;
+	audit_rule_syscallbyname_data(rule, "all");
+	rule->field_count = 2;
+	rule->fields[0] = type;
+	rule->values[0] = len;
+	rule->fieldflags[0] = AUDIT_EQUAL;
+	rule->buflen = len;
+	memcpy(&rule->buf[0], path, len);
+
+	// Default to all permissions
+	rule->fields[1] = AUDIT_PERM;
+	rule->fieldflags[1] = AUDIT_EQUAL;
+	rule->values[1] = AUDIT_PERM_READ | AUDIT_PERM_WRITE |
+				AUDIT_PERM_EXEC | AUDIT_PERM_ATTR;
+
+	return  0;
+}
+
+int audit_add_rule_data(int fd, struct audit_rule_data *rule,
+                        int flags, int action)
 {
 	int rc;
 
 	rule->flags  = flags;
 	rule->action = action;
-	rc = audit_send(fd, AUDIT_ADD, rule, sizeof(*rule));
+	rc = audit_send(fd, AUDIT_ADD_RULE, rule, 
+			sizeof(struct audit_rule_data) + rule->buflen);
 	if (rc < 0)
 		audit_msg(audit_priority(errno),
-			"Error sending add rule request (%s)", 
-			strerror(-rc));
+			"Error sending add rule data request (%s)",
+				errno == EEXIST ? 
+				"Rule exists" : strerror(-rc));
 	return rc;
 }
 
-int audit_delete_rule(int fd, struct audit_rule *rule, int flags, int action)
+int audit_delete_rule_data(int fd, struct audit_rule_data *rule,
+                           int flags, int action)
 {
 	int rc;
 
 	rule->flags  = flags;
 	rule->action = action;
-	rc = audit_send(fd, AUDIT_DEL, rule, sizeof(*rule));
+	rc = audit_send(fd, AUDIT_DEL_RULE, rule, 
+			sizeof(struct audit_rule_data) + rule->buflen);
 	if (rc < 0) {
 		if (rc == -ENOENT)
-			audit_msg(audit_priority(errno),
+			audit_msg(LOG_WARNING,
 			"Error sending delete rule request (No rule matches)");
 		else
-			audit_msg(LOG_WARNING,
-				"Error sending delete rule request (%s)",
+			audit_msg(audit_priority(errno),
+				"Error sending delete rule data request (%s)",
 				strerror(-rc));
 	}
 	return rc;
 }
 
+/*
+ * This function is part of the directory auditing code
+ */
+int audit_trim_subtrees(int fd)
+{
+	int rc = audit_send(fd, AUDIT_TRIM, NULL, 0);
+	if (rc < 0) 
+		audit_msg(audit_priority(errno),
+			"Error sending trim subtrees command (%s)",
+			strerror(-rc));
+	return rc;
+}
+
+/*
+ * This function is part of the directory auditing code
+ */
+int audit_make_equivalent(int fd, const char *mount_point,
+			 const char *subtree)
+{
+	int rc;
+	size_t len1 = strlen(mount_point);
+	size_t len2 = strlen(subtree);
+	struct audit_rule_data *rule = malloc(sizeof(struct audit_rule_data) +
+						len1 + len2);
+	
+	memset(rule, 0, sizeof(*rule) + len1 + len2);
+
+	rule->field_count = 2;
+	rule->buflen = len1 + len2;
+	rule->values[0] = len1;
+	memcpy(&rule->buf[0], mount_point, len1);
+	rule->values[1] = len2;
+	memcpy(&rule->buf[1], subtree, len2);
+
+	rc = audit_send(fd, AUDIT_MAKE_EQUIV, rule, 
+			sizeof(struct audit_rule_data) + rule->buflen);
+	if (rc < 0) 
+		audit_msg(audit_priority(errno),
+			"Error sending make_equivalent command (%s)",
+			strerror(-rc));
+	return rc;
+}
 
 /*
  * This function will retreive the loginuid or -1 if there
@@ -413,163 +726,7 @@ int audit_setloginuid(uid_t uid)
 	return rc;
 }
 
-// FIXME: make internal
-int audit_send_message(int fd, int type, const char *message)
-{
-	if (fd >= 0) {
-		int rc = audit_send(fd, type, message, strlen(message)+1);
-		if (rc < 0)
-			audit_msg(LOG_WARNING, 
-				"Error sending user message request (%s)", 
-				strerror(-rc));
-		return rc;
-	}
-	else { /* If for some reason fd < 0, send to syslog */
-		// This could be made configurable. Either syslog it or exit.
-		syslog(LOG_ERR, "\
-Can't send to audit system: %s pid=%d uid=%d loginuid=%d message=%s",
-				audit_msg_type_to_name(type),
-				getpid(), getuid(), audit_getloginuid(),
-				message);
-		return 0;
-	}
-}
-
-/*
- * This function will send a user space message to the kernel.
- * It returns the sequence number which is > 0 on success  
- * or <= 0 on error. (pam uses this) This is the main audit sending
- * function now.
- */
-int audit_send_user_message(int fd, int type, const char *message)
-{
-	int retry_cnt = 0;
-	int rc;
-retry:
-	rc = audit_send(fd, type, message, strlen(message)+1);
-	if (rc == -ECONNREFUSED) {
-		/* This is here to let people that build their own kernel
-		   and disable the audit system get in. ECONNREFUSED is
-		   issued by the kernel when there is "no on listening". */
-		return 0;
-	} else if (rc == -EPERM && getuid() != 0) {
-		/* If we get this, then the kernel supports auditing
-		 * but we don't have enough privilege to write to the
-		 * socket. Therefore, we have already been authenticated
-		 * and we are a common user. Just act as though auditing
-		 * is not enabled. Any other error we take seriously.
-		 * This is here basically to satisfy Xscreensaver. */
-		return 0;
-	} else if (rc == -EINVAL) {
-		/* If we get this, the kernel doesn't understand the
-		 * netlink message type. This is most likely due to
-		 * being an old kernel. Use the old message type. */
-		if (type >= AUDIT_FIRST_USER_MSG && 
-				type <= AUDIT_LAST_USER_MSG && !retry_cnt) {
-
-			/* do retry */
-			type = AUDIT_USER;
-			retry_cnt++;
-			goto retry;
-		} 
-	}
-	return rc;
-}
-
-int audit_send_message_if_enabled(int fd, const char *message)
-{	// Deprecated - do not change
-	if (audit_is_enabled(fd) > 0)
-		return audit_send(fd, AUDIT_USER, message, strlen(message)+1);
-	else
-		return 0;
-}
-
-/*
- * This function is used to send avc denial messages from user
- * space back into the kernel. (dbus, nscd, shadow uses this)
- * FIXME: deprecate
- */
-int audit_log_avc(int fd, int type, const char *fmt, va_list ap)
-{
-	int rc, retry_cnt = 0;
-	char buffer[1024];
-
-	vsnprintf(buffer, sizeof(buffer), fmt, ap);
-retry:
-	rc = audit_send_message(fd, type, buffer);
-	if (rc >= 0)
-		return rc;
-	if (rc == -ECONNREFUSED) {
-		/* This is here to let people that build their own kernel
-		   and disable the audit system get in. ECONNREFUSED is
-		   issued by the kernel when there is "no on listening". */
-		return 0;
-	} else if (rc == -EPERM && getuid() != 0) {
-		/* If we get this, then the kernel supports auditing
-		 * but we don't have enough privilege to write to the
-		 * socket. Therefore, we have already been authenticated
-		 * and we are a common user. Just act as though auditing
-		 * is not enabled. Any other error we take seriously. */
-		syslog(LOG_ERR, "\
-Can't send to audit system: %s pid=%d uid=%d loginuid=%d message=%s",
-				audit_msg_type_to_name(type),
-				getpid(), getuid(), audit_getloginuid(),
-				buffer);
-		return 0;
-	} else if (rc == -EINVAL) {
-		/* If we get this, the kernel doesn't understand the
-		 * netlink message type. This is most likely due to
-		 * being an old kernel. Use the old message type. */
-		if ( !retry_cnt ) {
-			/* do retry */
-			type = AUDIT_USER;
-			retry_cnt++;
-			goto retry;
-		} 
-	}
-	return rc;
-}
-
-int audit_vlog(int fd, const char *fmt, va_list ap)
-{	// Deprecated - do not change
-	char buffer[1024];
-
-	vsnprintf(buffer, sizeof(buffer), fmt, ap);
-	return audit_send_message(fd, AUDIT_USER, buffer);
-}
-
-// FIXME: deprecate
-int audit_log(int fd, int type, const char *fmt, ...)
-{
-	int rc;
-	va_list ap;
-
-	va_start(ap, fmt);
-	rc = audit_log_avc(fd, type, fmt, ap);
-	va_end(ap);
-
-	return rc;
-}
-
-/*
- * This function will log a user space message to the kernel
- * It returns the sequence number which is > 0 on success and
- * 0 or less on error. 
- * FIXME: deprecate
- */
-int audit_log_if_enabled(int fd, int type, const char *fmt, ...)
-{
-	int rc;
-	va_list ap;
-
-	va_start(ap, fmt);
-	rc = audit_log_avc(fd, type, fmt, ap);
-	va_end(ap);
-
-	return rc;
-}
-
-int audit_rule_syscall(struct audit_rule *rule, int scall)
+int audit_rule_syscall_data(struct audit_rule_data *rule, int scall)
 {
 	int word = AUDIT_WORD(scall);
 	int bit  = AUDIT_BIT(scall);
@@ -580,8 +737,8 @@ int audit_rule_syscall(struct audit_rule *rule, int scall)
 	return 0;
 }
 
-int audit_rule_syscallbyname(struct audit_rule *rule,
-                             const char *scall)
+int audit_rule_syscallbyname_data(struct audit_rule_data *rule,
+                                  const char *scall)
 {
 	int nr, i;
 	int machine;
@@ -603,44 +760,73 @@ int audit_rule_syscallbyname(struct audit_rule *rule,
 			nr = strtol(scall, NULL, 0);
 	}
 	if (nr >= 0) 
-		return audit_rule_syscall(rule, nr);
+		return audit_rule_syscall_data(rule, nr);
 	return -1;
 }
 
-int audit_rule_field(struct audit_rule *rule, int field,
-                     unsigned int value)
+int audit_rule_fieldpair_data(struct audit_rule_data **rulep, const char *pair,
+                              int flags)
 {
-	if (rule->field_count >= AUDIT_MAX_FIELDS) 
-		return -1;
-	rule->fields[rule->field_count] = field;
-	rule->values[rule->field_count] = value;
-	++rule->field_count;
-	return 0;
-}
-
-int audit_rule_fieldpair(struct audit_rule *rule, const char *pair, int flags)
-{
-	char       buf[128];
 	const char *f = pair;
-	const char *v = strchr(pair, '=');
+	char       *v;
+	int        op;
 	int        field;
-	int        negated = 0;
 	int        vlen;
-    
-	if (!f || !v || f == v)
+	int        offset;
+	struct audit_rule_data *rule = *rulep;
+
+	if (f == NULL)
 		return -1;
 
-	if (v > pair && v[-1] == '!')
-        	negated = 1;
+	/* look for 2-char operators first
+	   then look for 1-char operators afterwards
+	   when found, null out the bytes under the operators to split
+	   and set value pointer just past operator bytes
+	*/
+	if ( (v = strstr(pair, "!=")) ) {
+		*v++ = '\0';
+		*v++ = '\0';
+		op = AUDIT_NOT_EQUAL;
+	} else if ( (v = strstr(pair, ">=")) ) {
+		*v++ = '\0';
+		*v++ = '\0';
+		op = AUDIT_GREATER_THAN_OR_EQUAL;
+	} else if ( (v = strstr(pair, "<=")) ) {
+		*v++ = '\0';
+		*v++ = '\0';
+		op = AUDIT_LESS_THAN_OR_EQUAL;
+	} else if ( (v = strstr(pair, "&=")) ) {
+		*v++ = '\0';
+		*v++ = '\0';
+		op = AUDIT_BIT_TEST;
+	} else if ( (v = strstr(pair, "=")) ) {
+		*v++ = '\0';
+		op = AUDIT_EQUAL;
+	} else if ( (v = strstr(pair, ">")) ) {
+		*v++ = '\0';
+		op = AUDIT_GREATER_THAN;
+	} else if ( (v = strstr(pair, "<")) ) {
+		*v++ = '\0';
+		op = AUDIT_LESS_THAN;
+	} else if ( (v = strstr(pair, "&")) ) {
+		*v++ = '\0';
+		op = AUDIT_BIT_MASK;
+	}
 
-	snprintf(buf, sizeof(buf), "%*.*s", (int)(v-f-negated), 
-			(int)(v-f-negated), f);
-	audit_msg(LOG_DEBUG,"buf=%s\n", buf);
-	if ((field = audit_name_to_field(buf)) < 0) 
+	if (v == NULL || f == v)
+		return -1;
+
+	audit_msg(LOG_DEBUG,"pair=%s\n", f);
+	if ((field = audit_name_to_field(f)) < 0) 
 		return -2;
-	v++;
-	audit_msg(LOG_DEBUG,"f%d%s%s\n", field, negated ? "!=" : "=", v);
-	rule->fields[rule->field_count] = field | (negated ? AUDIT_NEGATE : 0);
+
+	/* Exclude filter can be used only with MSGTYPE field */
+	if (flags == AUDIT_FILTER_EXCLUDE && field != AUDIT_MSGTYPE)
+		return -12; 
+
+	audit_msg(LOG_DEBUG,"f%d%s%s\n", field, audit_operator_to_symbol(op),v);
+	rule->fields[rule->field_count] = field;
+	rule->fieldflags[rule->field_count] = op;
 	switch (field)
 	{
 		case AUDIT_UID:
@@ -681,9 +867,63 @@ int audit_rule_fieldpair(struct audit_rule *rule, const char *pair, int flags)
 				}
 			}
 			break;
+		case AUDIT_MSGTYPE:
+			if (flags != AUDIT_FILTER_EXCLUDE)
+				return -9;
+
+			if (isdigit((char)*(v)))
+				rule->values[rule->field_count] =
+					strtol(v, NULL, 0);
+			else
+				if (audit_name_to_msg_type(v) > 0)
+					rule->values[rule->field_count] =
+						audit_name_to_msg_type(v);
+				else
+					return -8;
+			break;
+		/* These next few are strings */
+		case AUDIT_OBJ_USER:
+		case AUDIT_OBJ_ROLE:
+		case AUDIT_OBJ_TYPE:
+		case AUDIT_OBJ_LEV_LOW:
+		case AUDIT_OBJ_LEV_HIGH:
+		case AUDIT_WATCH:
+		case AUDIT_DIR:
+			/* Watch & object filtering is invalid on anything
+			 * but exit */
+			if (flags != AUDIT_FILTER_EXIT)
+				return -7;
+			/* fallthrough */
+		case AUDIT_SUBJ_USER:
+		case AUDIT_SUBJ_ROLE:
+		case AUDIT_SUBJ_TYPE:
+		case AUDIT_SUBJ_SEN:
+		case AUDIT_SUBJ_CLR:
+		case AUDIT_FILTERKEY:
+			vlen = strlen(v);
+			if (field == AUDIT_FILTERKEY &&
+					vlen > AUDIT_MAX_KEY_LEN)
+				return -11;
+			else if (vlen > PATH_MAX)
+				return -11;
+			rule->values[rule->field_count] = vlen;
+			offset = rule->buflen;
+			rule->buflen += vlen;
+			*rulep = realloc(rule, sizeof(*rule) + rule->buflen);
+			if (*rulep == NULL) {
+				free(rule);
+				audit_msg(LOG_ERR, "Cannot realloc memory!\n");
+				return -3;
+			} else {
+				rule = *rulep;
+			}
+			strncpy(&rule->buf[offset], v, vlen);
+			break;
 		case AUDIT_ARCH:
 			if (audit_syscalladded) 
 				return -3;
+			if (!(op == AUDIT_NEGATE || op == AUDIT_EQUAL))
+				return -13;
 			if (isdigit((char)*(v))) {
 				int machine;
 
@@ -769,19 +1009,57 @@ int audit_rule_fieldpair(struct audit_rule *rule, const char *pair, int flags)
 			rule->values[rule->field_count] = audit_elf; 
 			audit_archadded = 1;
 			break;
+		case AUDIT_PERM:
+			if (flags != AUDIT_FILTER_EXIT)
+				return -7;
+			else if (op != AUDIT_EQUAL)
+				return -13;
+			else {
+				unsigned int i, len, val = 0;
+
+				len = strlen(v);
+				if (len > 4)
+					return -11;
+
+				for (i = 0; i < len; i++) {
+					switch (tolower(v[i])) {
+						case 'r':
+							val |= AUDIT_PERM_READ;
+							break;
+						case 'w':
+							val |= AUDIT_PERM_WRITE;
+							break;
+						case 'x':
+							val |= AUDIT_PERM_EXEC;
+							break;
+						case 'a':
+							val |= AUDIT_PERM_ATTR;
+							break;
+						default:
+							return -14;
+					}
+				}
+				rule->values[rule->field_count] = val;
+				audit_syscalladded = 1;// perm selects syscalls
+			}
+			break;
 		case AUDIT_DEVMAJOR...AUDIT_SUCCESS:
 			if (flags == AUDIT_FILTER_ENTRY)
 				return -7;
 			/* fallthrough */
 		default:
+			if (field == AUDIT_INODE) {
+				if (!(op == AUDIT_NEGATE || op == AUDIT_EQUAL))
+					return -13;
+			}
 			rule->values[rule->field_count] = strtol(v, NULL, 0);
 			break;
 	}
-	++rule->field_count;
+	rule->field_count++;
 	return 0;
 }
 
-void audit_rule_free(struct audit_rule *rule)
+void audit_rule_free_data(struct audit_rule_data *rule)
 {
 	free(rule);
 }
