@@ -1,5 +1,5 @@
 /* libaudit.h -- 
- * Copyright 2004-2006 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2007 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -35,7 +35,6 @@ extern "C" {
 #include <linux/audit.h>
 #include <stdarg.h>
 #include <syslog.h>
-#include <linux/fs.h>	/* For watch */
 
 
 /* Audit message types:
@@ -43,7 +42,7 @@ extern "C" {
  * 1100 - 1199 user space trusted application messages
  * 1200 - 1299 messages internal to the audit daemon
  * 1300 - 1399 audit event messages
- * 1400 - 1499 SE Linux use
+ * 1400 - 1499 kernel SE Linux use
  * 1500 - 1599 AppArmor events
  * 1600 - 1699 kernel crypto events
  * 1700 - 1799 kernel anomaly records
@@ -80,6 +79,8 @@ extern "C" {
 #define AUDIT_CHGRP_ID		1119    /* User space group ID changed */
 #define AUDIT_TEST		1120	/* Used for test success messages */
 #define AUDIT_TRUSTED_APP	1121	/* Trusted app msg - freestyle text */
+#define AUDIT_USER_SELINUX_ERR	1122	/* SE Linux user space error */
+#define AUDIT_USER_CMD		1123	/* User shell command and args */
 
 #define AUDIT_FIRST_DAEMON	1200
 #define AUDIT_LAST_DAEMON	1299
@@ -88,15 +89,26 @@ extern "C" {
 
 #define AUDIT_FIRST_EVENT	1300
 #define AUDIT_LAST_EVENT	1399
-#define AUDIT_CWD		1307	/* Current working directory */
-#define AUDIT_FS_INODE		1308	/* File system inode */
-#ifndef AUDIT_EXECVE
-#define AUDIT_EXECVE		1309
+#ifndef AUDIT_KERNEL_OTHER
+#define AUDIT_CWD               1307    /* Current working directory */
+#define AUDIT_EXECVE            1309    /* execve arguments */
+#define AUDIT_IPC_SET_PERM      1311    /* IPC new permissions record type */
+#define AUDIT_MQ_OPEN           1312    /* POSIX MQ open record type */
+#define AUDIT_MQ_SENDRECV       1313    /* POSIX MQ send/receive record type */
+#define AUDIT_MQ_NOTIFY         1314    /* POSIX MQ notify record type */
+#define AUDIT_MQ_GETSETATTR     1315    /* POSIX MQ get/set attribute record type */
+#define AUDIT_KERNEL_OTHER      1316    /* For use by 3rd party modules */
+#endif
+#ifndef AUDIT_FD_PAIR
+#define AUDIT_FD_PAIR		1317	/* audit record for pipe/socketpair */
+#endif
+#ifndef AUDIT_OBJ_PID
+#define AUDIT_OBJ_PID		1318	/* Ptrace target */
 #endif
 #define AUDIT_LAST_EVENT	1399
 
-#define AUDIT_FIRST_SELINUX		1400
-#define AUDIT_LAST_SELINUX		1499
+#define AUDIT_FIRST_SELINUX	1400
+#define AUDIT_LAST_SELINUX	1499
 
 /* For new kernel messages */
 #ifndef AUDIT_MAC_POLICY_LOAD
@@ -104,7 +116,19 @@ extern "C" {
 #define AUDIT_MAC_STATUS	1404    /* Changed enforcing,permissive,off */
 #define AUDIT_MAC_CONFIG_CHANGE	1405    /* Changes to booleans */
 #endif
-
+#ifndef AUDIT_MAC_UNLBL_ALLOW
+#define AUDIT_MAC_UNLBL_ALLOW	1406    /* NetLabel: allow unlabeled traffic */
+#define AUDIT_MAC_CIPSOV4_ADD	1407    /* NetLabel: add CIPSOv4 DOI entry */
+#define AUDIT_MAC_CIPSOV4_DEL	1408    /* NetLabel: del CIPSOv4 DOI entry */
+#define AUDIT_MAC_MAP_ADD	1409    /* NetLabel: add LSM domain mapping */
+#define AUDIT_MAC_MAP_DEL	1410    /* NetLabel: del LSM domain mapping */
+#endif
+#ifndef AUDIT_MAC_IPSEC_ADDSA
+#define AUDIT_MAC_IPSEC_ADDSA	1411    /* Add a XFRM state */
+#define AUDIT_MAC_IPSEC_DELSA	1412    /* Delete a XFRM state */
+#define AUDIT_MAC_IPSEC_ADDSPD	1413    /* Add a XFRM policy */
+#define AUDIT_MAC_IPSEC_DELSPD	1414    /* Delete a XFRM policy */
+#endif
 #define AUDIT_FIRST_APPARMOR		1500
 #define AUDIT_LAST_APPARMOR		1599
 #ifndef AUDIT_SD
@@ -119,7 +143,9 @@ extern "C" {
 #define AUDIT_LAST_KERN_ANOM_MSG	1799
 #define AUDIT_ANOM_PROMISCUOUS		1700 // Device changed promiscuous mode
 #endif
-
+#ifndef AUDIT_ANOM_ABEND
+#define AUDIT_ANOM_ABEND		1701 /* Process ended abnormally */
+#endif
 #define AUDIT_FIRST_ANOM_MSG		2100
 #define AUDIT_LAST_ANOM_MSG		2199
 #define AUDIT_ANOM_LOGIN_FAILURES	2100 // Failed login limit reached
@@ -131,14 +157,15 @@ extern "C" {
 #define AUDIT_ANOM_MAX_MAC		2106 // Max MAC failures reached
 #define AUDIT_ANOM_AMTU_FAIL		2107 // AMTU failure
 #define AUDIT_ANOM_RBAC_FAIL		2108 // RBAC self test failure
-#define AUDIT_ANOM_RBAC_INTEGRITY_FAIL	2108 // RBAC file integrity failure
-#define AUDIT_ANOM_CRYPTO_FAIL		2109 // Crypto system test failure
-#define AUDIT_ANOM_ACCESS_FS		2110 // Access of file or dir
-#define AUDIT_ANOM_EXEC			2111 // Execution of file
-#define AUDIT_ANOM_MK_EXEC		2112 // Make an executable
-#define AUDIT_ANOM_ADD_ACCT		2113 // Adding an acct
-#define AUDIT_ANOM_DEL_ACCT		2114 // Deleting an acct
-#define AUDIT_ANOM_MOD_ACCT		2115 // Changing an acct
+#define AUDIT_ANOM_RBAC_INTEGRITY_FAIL	2109 // RBAC file integrity failure
+#define AUDIT_ANOM_CRYPTO_FAIL		2110 // Crypto system test failure
+#define AUDIT_ANOM_ACCESS_FS		2111 // Access of file or dir
+#define AUDIT_ANOM_EXEC			2112 // Execution of file
+#define AUDIT_ANOM_MK_EXEC		2113 // Make an executable
+#define AUDIT_ANOM_ADD_ACCT		2114 // Adding an acct
+#define AUDIT_ANOM_DEL_ACCT		2115 // Deleting an acct
+#define AUDIT_ANOM_MOD_ACCT		2116 // Changing an acct
+#define AUDIT_ANOM_ROOT_TRANS		2117 // User became root
 
 #define AUDIT_FIRST_ANOM_RESP		2200
 #define AUDIT_LAST_ANOM_RESP		2299
@@ -165,6 +192,9 @@ extern "C" {
 #define AUDIT_LABEL_LEVEL_CHANGE	2304 /* Object's level was changed */
 #define AUDIT_USER_LABELED_EXPORT	2305 /* Object exported with label */
 #define AUDIT_USER_UNLABELED_EXPORT	2306 /* Object exported without label */
+#define AUDIT_DEV_ALLOC			2307 /* Device was allocated */
+#define AUDIT_DEV_DEALLOC		2308 /* Device was deallocated */
+#define AUDIT_FS_RELABEL		2309 /* Filesystem relabeled */
 
 #define AUDIT_FIRST_CRYPTO_MSG		2400
 #define AUDIT_LAST_CRYPTO_MSG		2499
@@ -174,6 +204,79 @@ extern "C" {
 #define AUDIT_LAST_USER_MSG2   2999
 #endif
 
+/* These are from the watching subtrees patch */
+#ifndef AUDIT_TRIM
+#define AUDIT_TRIM              1014    /* Trim junk from watched tree */
+#define AUDIT_MAKE_EQUIV        1015    /* Append to watched tree */
+#endif
+
+/* This is for the new operator patch */
+#ifndef AUDIT_BIT_MASK
+#define AUDIT_BIT_MASK			0x08000000
+#endif
+#ifndef AUDIT_LESS_THAN 
+#define AUDIT_LESS_THAN			0x10000000
+#define AUDIT_GREATER_THAN		0x20000000
+#define AUDIT_NOT_EQUAL			0x30000000
+#define AUDIT_EQUAL			0x40000000
+#define AUDIT_LESS_THAN_OR_EQUAL	(AUDIT_LESS_THAN|AUDIT_EQUAL)
+#define AUDIT_GREATER_THAN_OR_EQUAL	(AUDIT_GREATER_THAN|AUDIT_EQUAL)
+#define AUDIT_OPERATORS			(AUDIT_EQUAL|AUDIT_NOT_EQUAL)
+#endif
+#ifndef AUDIT_BIT_TEST
+#define AUDIT_BIT_TEST		(AUDIT_BIT_MASK|AUDIT_EQUAL)
+#undef  AUDIT_OPERATORS
+#define AUDIT_OPERATORS		(AUDIT_EQUAL|AUDIT_NOT_EQUAL|AUDIT_BIT_MASK)
+#endif
+
+/* This is for the new rule field definitions */
+#ifndef AUDIT_MSGTYPE
+#define AUDIT_MSGTYPE 12
+#endif
+
+/* This is from the audit by role patch */
+#ifndef AUDIT_SUBJ_USER
+#define AUDIT_SUBJ_USER     13
+#define AUDIT_SUBJ_ROLE     14
+#define AUDIT_SUBJ_TYPE     15
+#define AUDIT_SUBJ_SEN      16
+#define AUDIT_SUBJ_CLR      17
+#define AUDIT_OBJ_USER      19
+#define AUDIT_OBJ_ROLE      20
+#define AUDIT_OBJ_TYPE      21
+#define AUDIT_OBJ_LEV_LOW   22
+#define AUDIT_OBJ_LEV_HIGH  23
+#endif
+
+#ifndef AUDIT_PPID
+#define AUDIT_PPID          18
+#endif
+
+/* This is from the file system auditing patch */
+#ifndef AUDIT_WATCH
+#define AUDIT_WATCH         105	/* This is a field in syscall rule */
+#endif
+
+/* This is from the directory auditing patch */
+#ifndef AUDIT_DIR
+#define AUDIT_DIR       107
+#endif
+
+/* Defines for syscall classes - watch permissions */
+#ifndef AUDIT_PERM
+#define AUDIT_PERM		106	/* This is a field in syscall rule */
+#define AUDIT_PERM_EXEC		1
+#define AUDIT_PERM_WRITE	2
+#define AUDIT_PERM_READ		4
+#define AUDIT_PERM_ATTR		8
+#endif
+
+/* This is from filterkey patch */
+#ifndef AUDIT_FILTERKEY
+#define AUDIT_FILTERKEY     210
+#define AUDIT_MAX_KEY_LEN   32
+#endif
+
 /* This is new list defines from audit.h */
 #ifndef AUDIT_FILTER_USER
 #define AUDIT_FILTER_USER       0x00    /* Apply rule to user-generated messages */
@@ -184,12 +287,12 @@ extern "C" {
 #ifndef AUDIT_FILTER_ENTRY
 #define AUDIT_FILTER_ENTRY      0x02    /* Apply rule at syscall entry */
 #endif
-#ifndef AUDIT_FILTER_WATCH
-#define AUDIT_FILTER_WATCH      0x03    /* Apply rule to file system watches */
-#endif
 #ifndef AUDIT_FILTER_EXIT
 #define AUDIT_FILTER_EXIT       0x04    /* Apply rule at syscall exit */
 #endif
+#ifndef AUDIT_FILTER_EXCLUDE
+#define AUDIT_FILTER_EXCLUDE    0x05    /* Remove event type before sending */
+#endif
 #define AUDIT_FILTER_MASK	0x07	/* Mask to get actual filter */
 #ifndef AUDIT_FILTER_PREPEND
 #define AUDIT_FILTER_PREPEND    0x10    /* Prepend to front of list */
@@ -208,41 +311,51 @@ typedef enum {
         MACH_ALPHA
 } machine_t;
 
-#ifndef AUDIT_FILTERKEY_MAX
-/* From the watch patch  - 32 byte max key size */
-#define AUDIT_FILTERKEY_MAX    32
+/* These are the valid audit failure tunable enum values */
+typedef enum {
+	FAIL_IGNORE=0,
+	FAIL_LOG,
+	FAIL_TERMINATE
+} auditfail_t;
 
-/* Structure to transport watch data to and from the kernel */
-struct watch_transport {
-	uint32_t dev_major;
-	uint32_t dev_minor;
-	uint32_t perms;
-	uint32_t valid;
-	uint32_t pathlen;
-	uint32_t fklen;
-	char buf[0];
+/*
+ * audit_rule_data supports filter rules with both integer and string
+ * fields. It corresponds with AUDIT_ADD_RULE, AUDIT_DEL_RULE and
+ * AUDIT_LIST_RULES requests.
+ */
+#ifndef AUDIT_ADD_RULE
+#define AUDIT_ADD_RULE		1011    /* Add syscall filtering rule */
+#define AUDIT_DEL_RULE		1012    /* Delete syscall filtering rule */
+#define AUDIT_LIST_RULES	1013    /* List syscall filtering rules */
+
+struct audit_rule_data {
+	uint32_t	flags;  /* AUDIT_PER_{TASK,CALL}, AUDIT_PREPEND */
+	uint32_t	action; /* AUDIT_NEVER, AUDIT_POSSIBLE, AUDIT_ALWAYS */
+	uint32_t	field_count;
+	uint32_t	mask[AUDIT_BITMASK_SIZE];
+	uint32_t	fields[AUDIT_MAX_FIELDS];
+	uint32_t	values[AUDIT_MAX_FIELDS];
+	uint32_t	fieldflags[AUDIT_MAX_FIELDS];
+	uint32_t	buflen; /* total length of string fields */
+	char		buf[0]; /* string fields buffer */
 };
 #endif
 
-struct audit_watch {
-	uint32_t dev_major;
-	uint32_t dev_minor;
-	uint32_t perms;
-	uint32_t valid;
-	uint32_t namelen;
-	uint32_t fklen;
-	char     *name;
-	char     *filterkey;
-};
-
+//////////////////////////////////////////////////////
+// This is an external ABI. Any changes in here will
+// likely affect pam_loginuid. There might be other
+// apps that use this low level interface, but I don't
+// know of any.
+//
 /* data structure for who signaled the audit daemon */
 struct audit_sig_info {
         uid_t           uid;
         pid_t           pid;
+	char		ctx[0];
 };
 
 /* defines for audit subsystem */
-#define MAX_AUDIT_MESSAGE_LENGTH    8460 /* PATH_MAX*2+11+256+1 */
+#define MAX_AUDIT_MESSAGE_LENGTH    8970 // PATH_MAX*2+CONTEXT_SIZE*2+11+256+1
 struct audit_message {
 	struct nlmsghdr nlh;
 	char   data[MAX_AUDIT_MESSAGE_LENGTH];
@@ -262,10 +375,10 @@ struct audit_reply {
 	union {
 	struct audit_status     *status;
 	struct audit_rule       *rule;
+	struct audit_rule_data  *ruledata;
 	struct audit_login      *login;
 	const char              *message;
 	struct nlmsgerr         *error;
-	struct watch_transport  *watch;
 	struct audit_sig_info   *signal_info;
 	struct daemon_conf      *conf;
 	};
@@ -275,21 +388,30 @@ struct auditd_reply_list {
 	struct audit_reply reply;
 	struct auditd_reply_list *next;
 };
+//
+// End of ABI control
+//////////////////////////////////////////////////////
 
+//////////////////////////////////////////////////////
+// audit dispatcher interface
+//
 /* audit_dispatcher_header: This header is versioned. If anything gets
  * added to it, it must go at the end and the version number bumped.
  * This MUST BE fixed size for compatibility. If you are going to add
  * new member then add them into _structure_ part.
  */
 struct audit_dispatcher_header {
-	uint32_t        ver;    /* The version of this protocol */
-	uint32_t        hlen;   /* Header length */
-	uint32_t        type;   /* Message type */
-	uint32_t        size;   /* Size of data following the header */
+	uint32_t	ver;	/* The version of this protocol */
+	uint32_t	hlen;	/* Header length */
+	uint32_t	type;	/* Message type */
+	uint32_t	size;	/* Size of data following the header */
 };
 
 #define AUDISP_PROTOCOL_VER 0
 
+///////////////////////////////////////////////////
+// Libaudit API
+//
 /* Messages */
 typedef enum { MSG_STDERR, MSG_SYSLOG, MSG_QUIET } message_t;
 typedef enum { DBG_NO, DBG_YES } debug_message_t;
@@ -303,7 +425,7 @@ extern int  audit_get_reply(int fd, struct audit_reply *rep, reply_t block,
 		int peek);
 extern uid_t audit_getloginuid(void);
 extern int  audit_setloginuid(uid_t uid);
-extern int audit_detect_machine(void);
+extern int  audit_detect_machine(void);
 
 /* Translation functions */
 extern int        audit_name_to_field(const char *field);
@@ -320,10 +442,12 @@ extern int        audit_name_to_machine(const char *machine);
 extern const char *audit_machine_to_name(int machine);
 extern unsigned int audit_machine_to_elf(int machine);
 extern int          audit_elf_to_machine(unsigned int elf);
+extern const char *audit_operator_to_symbol(int op);
 
 /* AUDIT_GET */
 extern int audit_request_status(int fd);
 extern int audit_is_enabled(int fd);
+extern int get_auditfail_action(auditfail_t *failmode);
 
 /* AUDIT_SET */
 typedef enum { WAIT_NO, WAIT_YES } rep_wait_t;
@@ -334,40 +458,28 @@ extern int  audit_set_rate_limit(int fd, uint32_t limit);
 extern int  audit_set_backlog_limit(int fd, uint32_t limit);
 
 /* AUDIT_LIST */
-extern int  audit_request_rules_list(int fd);
-extern int  audit_request_watch_list(int fd);
+extern int  audit_request_rules_list_data(int fd);
 
 /* SIGNAL_INFO */
 extern int audit_request_signal_info(int fd);
 
 /* AUDIT_WATCH */
-extern int audit_insert_watch(int fd, struct audit_watch *req);
-extern int audit_remove_watch(int fd, struct audit_watch *req);
+extern int audit_update_watch_perms(struct audit_rule_data *rule, int perms);
+extern int audit_add_watch(struct audit_rule_data **rulep, const char *path);
+extern int audit_add_dir(struct audit_rule_data **rulep, const char *path);
+extern int audit_add_watch_dir(int type, struct audit_rule_data **rulep,
+				const char *path);
+extern int audit_trim_subtrees(int fd);
+extern int audit_make_equivalent(int fd, const char *mount_point,
+				const char *subtree);
 
 /* AUDIT_ADD */
-extern int  audit_add_rule(int fd, struct audit_rule *rule,
-                           int flags, int action);
+extern int  audit_add_rule_data(int fd, struct audit_rule_data *rule,
+                                int flags, int action);
 
 /* AUDIT_DEL */
-extern int  audit_delete_rule(int fd, struct audit_rule *rule,
-                              int flags, int action);
-
-/* AUDIT_USER */
-//deprecated - do not use!
-extern int audit_vlog(int fd, const char *fmt, va_list ap); //+dbus
-extern int audit_send_message_if_enabled(int fd, const char *message); //+pam
-
-// New USER message interfaces
-extern int audit_log_if_enabled(int fd, int type, const char *fmt, ...)//passwd
-#ifdef __GNUC__
-	__attribute__ ((format (printf, 3, 4)));
-#else
-	;
-#endif
-extern int audit_log(int fd, int type, const char *fmt, ...);
-extern int audit_send_message(int fd, int type, const char *message);
-extern int audit_log_avc(int fd, int type, const char *fmt, va_list ap);//dbus
-extern int audit_send_user_message(int fd, int type, const char *message);//pam
+extern int  audit_delete_rule_data(int fd, struct audit_rule_data *rule,
+                                   int flags, int action);
 
 /* The following are for standard formatting of messages */
 extern int audit_log_user_message(int audit_fd, int type, const char *message,
@@ -381,16 +493,25 @@ extern int audit_log_acct_message(int audit_fd, int type, const char *pgname,
 extern int audit_log_user_avc_message(int audit_fd, int type, 
 	const char *message, const char *hostname, const char *addr, 
 	const char *tty, uid_t uid);
+extern int audit_log_semanage_message(int audit_fd, int type,
+	const char *pgname, const char *op, const char *name, unsigned int id,
+        const char *new_seuser, const char *new_role, const char *new_range,
+	const char *old_seuser, const char *old_role, const char *old_range,
+	const char *host, const char *addr,
+        const char *tty, int result);
+extern int audit_log_user_command(int audit_fd, int type, const char *command,
+        const char *tty, int result);
 
 /* Rule-building helper functions */
-extern int  audit_rule_syscall(struct audit_rule *rule, int scall);
-extern int  audit_rule_syscallbyname(struct audit_rule *rule,
-                                     const char *scall);
-extern int  audit_rule_field(struct audit_rule *rule, int field,
-                             unsigned int value);
-extern int  audit_rule_fieldpair(struct audit_rule *rule, const char *pair, 
-		int flags);
-extern void audit_rule_free(struct audit_rule *rule);
+extern int  audit_rule_syscall_data(struct audit_rule_data *rule, int scall);
+extern int  audit_rule_syscallbyname_data(struct audit_rule_data *rule,
+                                          const char *scall);
+/* Note that the following function takes a **, where audit_rule_fieldpair()
+ * takes just a *.  That structure may need to be reallocated as a result of
+ * adding new fields */
+extern int  audit_rule_fieldpair_data(struct audit_rule_data **rulep,
+                                      const char *pair, int flags);
+extern void audit_rule_free_data(struct audit_rule_data *rule);
 
 #ifdef __cplusplus
 }
