@@ -28,12 +28,17 @@
  */
 //_S(AUDIT_GET,                      "GET"                           )
 //_S(AUDIT_SET,                      "SET"                           )
-//_S(AUDIT_LIST,                     "LIST"                          )
+_S(AUDIT_LIST,                       "LIST"                          )
 //_S(AUDIT_ADD,                      "ADD"                           )
 //_S(AUDIT_DEL,                      "DEL"                           )
 _S(AUDIT_USER,                       "USER"                          )
 _S(AUDIT_LOGIN,                      "LOGIN"                         )
 //_S(AUDIT_SIGNAL_INFO,              "SIGNAL_INFO"                   )
+//_S(AUDIT_ADD_RULE,                 "ADD_RULE"                      )
+//_S(AUDIT_DEL_RULE,                 "DEL_RULE"                      )
+_S(AUDIT_LIST_RULES,                 "LIST_RULES"                    )
+//_S(AUDIT_TRIM,                     "TRIM"                          )
+//_S(AUDIT_MAKE_EQUIV,               "MAKE_EQUIV"                    )
 _S(AUDIT_USER_AUTH,                  "USER_AUTH"                     )
 _S(AUDIT_USER_ACCT,                  "USER_ACCT"                     )
 _S(AUDIT_USER_MGMT,                  "USER_MGMT"                     )
@@ -56,6 +61,8 @@ _S(AUDIT_DAC_CHECK,                  "DAC_CHECK"                     )
 _S(AUDIT_CHGRP_ID,                   "CHGRP_ID"                      )
 _S(AUDIT_TEST,                       "TEST"                          )
 _S(AUDIT_TRUSTED_APP,                "TRUSTED_APP"                   )
+_S(AUDIT_USER_SELINUX_ERR,           "USER_SELINUX_ERR"              )
+_S(AUDIT_USER_CMD,                   "USER_CMD"                      )
 _S(AUDIT_DAEMON_START,               "DAEMON_START"                  )
 _S(AUDIT_DAEMON_END,                 "DAEMON_END"                    )
 _S(AUDIT_DAEMON_ABORT,               "DAEMON_ABORT"                  )
@@ -63,22 +70,40 @@ _S(AUDIT_DAEMON_CONFIG,              "DAEMON_CONFIG"                 )
 //_S(AUDIT_DAEMON_RECONFIG,          "DAEMON_RECONFIG"               )
 _S(AUDIT_DAEMON_ROTATE,              "DAEMON_ROTATE"                 )
 _S(AUDIT_SYSCALL,                    "SYSCALL"                       )
-_S(AUDIT_FS_WATCH,                   "FS_WATCH"                      )
+//_S(AUDIT_FS_WATCH,                 "FS_WATCH"                      )
 _S(AUDIT_PATH,                       "PATH"                          )
 _S(AUDIT_IPC,                        "IPC"                           )
 _S(AUDIT_SOCKETCALL,                 "SOCKETCALL"                    )
 _S(AUDIT_CONFIG_CHANGE,              "CONFIG_CHANGE"                 )
 _S(AUDIT_SOCKADDR,                   "SOCKADDR"                      )
 _S(AUDIT_CWD,                        "CWD"                           )
-_S(AUDIT_FS_INODE,                   "FS_INODE"                      )
+//_S(AUDIT_FS_INODE,                 "FS_INODE"                      )
 _S(AUDIT_EXECVE,                     "EXECVE"                        )
+_S(AUDIT_IPC_SET_PERM,               "IPC_SET_PERM"                  )
+_S(AUDIT_MQ_OPEN,                    "MQ_OPEN"                       )
+_S(AUDIT_MQ_SENDRECV,                "MQ_SENDRECV"                   )
+_S(AUDIT_MQ_NOTIFY,                  "MQ_NOTIFY"                     )
+_S(AUDIT_MQ_GETSETATTR,              "MQ_GETSETATTR"                 )
+_S(AUDIT_KERNEL_OTHER,               "KERNEL_OTHER"                  )
+_S(AUDIT_FD_PAIR,                    "FD_PAIR"                       )
+_S(AUDIT_OBJ_PID,                    "OBJ_PID"                       )
 _S(AUDIT_AVC,                        "AVC"                           )
 _S(AUDIT_SELINUX_ERR,                "SELINUX_ERR"                   )
 _S(AUDIT_AVC_PATH,                   "AVC_PATH"                      )
 _S(AUDIT_MAC_POLICY_LOAD,            "MAC_POLICY_LOAD"               )
 _S(AUDIT_MAC_STATUS,                 "MAC_STATUS"                    )
 _S(AUDIT_MAC_CONFIG_CHANGE,          "MAC_CONFIG_CHANGE"             )
+_S(AUDIT_MAC_UNLBL_ALLOW,            "MAC_UNLBL_ALLOW"               )       
+_S(AUDIT_MAC_CIPSOV4_ADD,            "MAC_CIPSOV4_ADD"               )
+_S(AUDIT_MAC_CIPSOV4_DEL,            "MAC_CIPSOV4_DEL"               )
+_S(AUDIT_MAC_MAP_ADD,                "MAC_MAP_ADD"                   )
+_S(AUDIT_MAC_MAP_DEL,                "MAC_MAP_DEL"                   )
+_S(AUDIT_MAC_IPSEC_ADDSA,            "MAC_IPSEC_ADDSA"               )
+_S(AUDIT_MAC_IPSEC_DELSA,            "MAC_IPSEC_DELSA"               )
+_S(AUDIT_MAC_IPSEC_ADDSPD,           "MAC_IPSEC_ADDSPD"              )
+_S(AUDIT_MAC_IPSEC_DELSPD,           "MAC_IPSEC_DELSPD"              )
 _S(AUDIT_ANOM_PROMISCUOUS,           "ANOM_PROMISCUOUS"              )
+_S(AUDIT_ANOM_ABEND,                 "ANOM_ABEND"                    )
 #ifdef WITH_APPARMOR
 _S(AUDIT_SD,                         "APPARMOR"                      )
 #endif
@@ -120,3 +145,7 @@ _S(AUDIT_LABEL_OVERRIDE,             "LABEL_OVERRIDE"                )
 _S(AUDIT_LABEL_LEVEL_CHANGE,         "LABEL_LEVEL_CHANGE"            )
 _S(AUDIT_USER_LABELED_EXPORT,        "USER_LABELED_EXPORT"           )
 _S(AUDIT_USER_UNLABELED_EXPORT,      "USER_UNLABELED_EXPORT"         )
+_S(AUDIT_DEV_ALLOC,                  "DEV_ALLOC"                     )
+_S(AUDIT_DEV_DEALLOC,                "DEV_DEALLOC"                   )
+_S(AUDIT_FS_RELABEL,                 "FS_RELABEL"                    )
+
