@@ -238,6 +238,7 @@ _S(237, "tkill")
 _S(238, "futex")
 _S(239, "sched_setaffinity")
 _S(240, "sched_getaffinity")
+_S(241, "tgkill")
 _S(243, "io_setup")
 _S(244, "io_destroy")
 _S(245, "io_getevents")
@@ -277,3 +278,31 @@ _S(283, "ioprio_get")
 _S(284, "inotify_init")
 _S(285, "inotify_add_watch")
 _S(286, "inotify_rm_watch")
+//_S(287, "")
+_S(288, "openat")
+_S(289, "mkdirat")
+_S(290, "mknodat")
+_S(291, "fchownat")
+_S(292, "futimesat")
+_S(293, "fstatat")
+_S(294, "unlinkat")
+_S(295, "renameat")
+_S(296, "linkat")
+_S(297, "symlinkat")
+_S(298, "readlinkat")
+_S(299, "fchmodat")
+_S(300, "faccessat")
+_S(301, "pselect6")
+_S(302, "ppoll")
+_S(303, "unshare")
+_S(304, "set_robust_list")
+_S(305, "get_robust_list")
+_S(306, "splice")
+_S(307, "sync_file_range")
+_S(308, "tee")
+_S(309, "vmsplice")
+//_S(310, "")
+_S(311, "getcpu")
+_S(312, "epoll_pwait")
+_S(313, "utimes")
+
