@@ -1,6 +1,6 @@
 /*
 * ausearch-llist.c - Minimal linked list library
-* Copyright (c) 2005-06 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -58,19 +58,21 @@ lnode *list_next(llist *l)
 	return l->cur;
 }
 
-void list_append(llist *l, lnode *node)
+void list_append(llist *l, struct audit_rule_data *r, size_t sz)
 {
 	lnode* newnode;
 
 	newnode = malloc(sizeof(lnode));
 
-	if (node->w)
-		newnode->w = node->w;
-	else
-		newnode->w = 0;
+	if (r) {
+		void *rr = malloc(sz);
+		if (rr) 
+			memcpy(rr, r, sz);
+		newnode->r = rr;
+	} else
+		newnode->r = NULL;
 
-	newnode->size = node->size;
-	newnode->item = l->cnt; 
+	newnode->size = sz;
 	newnode->next = 0;
 
 	// if we are at top, fix this up
@@ -90,9 +92,9 @@ void list_clear(llist* l)
 	register lnode* current;
 
 	current = l->head;
-	while (current)	{
+	while (current) {
 		nextnode=current->next;
-		free(current->w);
+		free(current->r);
 		free(current);
 		current=nextnode;
 	}
