@@ -1,5 +1,5 @@
 /* auditctl.c -- 
- * Copyright 2004, 2005 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2007 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -35,7 +35,6 @@
 #include <errno.h>
 #include <libgen.h>	/* For basename */
 #include "libaudit.h"
-#include "auditctl-llist.h"
 #include "private.h"
 
 /* This define controls how many rule options we will allow when
@@ -50,27 +49,23 @@
  */
 #define LINE_SIZE 1600
 
-/* These are all related to the fs watch code */
-#define WATCH_MAY_EXEC		1
-#define WATCH_MAY_WRITE		2
-#define WATCH_MAY_READ		4
-#define WATCH_MAY_APPEND	8
-
 
 /* Global functions */
 static int handle_request(int status);
 static void get_reply(void);
 static int audit_print_reply(struct audit_reply *rep);
-static int delete_all_rules(void);
-static int delete_all_watches(void);
+extern int delete_all_rules(int fd);
 
 /* Global vars */
 static int fd = -1;
 static int list_requested = 0;
 static int add = AUDIT_FILTER_UNSET, del = AUDIT_FILTER_UNSET, action = 0;
-static int ins = 0, rem = 0, ignore = 0;
+static int ignore = 0;
+static int exclude = 0, msgtype_cnt = 0;
+enum { OLD, NEW };
+int which;
 static struct audit_rule  rule;
-static struct audit_watch watch;
+static struct audit_rule_data *rule_new = NULL;
 
 extern int audit_archadded;
 extern int audit_syscalladded;
@@ -89,11 +84,13 @@ static int reset_vars(void)
 	add = AUDIT_FILTER_UNSET;
 	del = AUDIT_FILTER_UNSET;
 	action = 0;
-	ins = 0;
-	rem = 0;
+	exclude = 0;
+	msgtype_cnt = 0;
+	which = OLD;
 
 	memset(&rule, 0, sizeof(rule));
-	memset(&watch, 0, sizeof(watch));
+	free(rule_new);
+	rule_new = NULL;
 	if ((fd = audit_open()) < 0) {
 		fprintf(stderr, "Cannot open netlink audit socket\n");
 		return 1;
@@ -101,38 +98,66 @@ static int reset_vars(void)
 	return 0;
 }
 
+static void upgrade_rule(void)
+{
+	int i;
+
+	rule_new=malloc(sizeof(struct audit_rule_data));
+	memset(rule_new, 0, sizeof(struct audit_rule_data));
+	memcpy(rule_new, &rule, sizeof(rule));
+	which = NEW;
+	if (rule_new->field_count == 0)
+		return;
+
+	// now go through fields and move legacy ops to fieldflags
+	for (i=0; i<rule_new->field_count; i++) {
+		uint32_t ops = rule_new->fields[i] & 
+				(AUDIT_NEGATE|AUDIT_OPERATORS);
+		rule_new->fields[i] &= ~(AUDIT_NEGATE|AUDIT_OPERATORS);
+		if (ops & AUDIT_NEGATE)
+			rule_new->fieldflags[i] = AUDIT_NOT_EQUAL;
+		else
+			rule_new->fieldflags[i] = AUDIT_EQUAL;
+	}
+}
+
 static void usage(void)
 {
-    printf("usage: auditctl [options]\n"
-     "      -a <l,a>     Append rule to end of <l>ist with <a>ction\n"
-     "      -A <l,a>     Add rule at beginning of <l>ist with <a>ction\n"
-     "      -b <backlog> Set max number of outstanding audit buffers allowed\n"
-     "                   Default=64\n"
-     "      -d <l,a>     Delete rule from <l>ist with <a>ction\n"
-     "                   l=task,entry,exit,user,watch a=never,possible,always\n"
-     "      -D           Delete all rules and watches\n"
-     "      -e [0|1]     Set enabled flag\n"
-     "      -f [0..2]    Set failure flag\n"
-     "                   0=silent 1=printk 2=panic\n"
-     "      -F f=v       Build rule: field name, value\n"
-     "      -h           Help\n"
-     "      -i           Ignore errors when reading rules from file\n"
-     "      -k <key>     Set filterkey on watch\n"
-     "      -l           List rules\n"
-     "      -m text      Send a user-space message\n"
-     "      -p [r|w|x|a] Set permissions filter on watch\n"
-     "                   r=read, w=write, x=execute, a=append\n"
-     "      -r <rate>    Set limit in messages/sec (0=none)\n"
-     "      -R <file>    read rules from file\n"
-     "      -s           Report status\n"
-     "      -S syscall   Build rule: syscall name or number\n"
-     "      -t <syscall> Translate syscall number to syscall name\n"
-     "      -v           Version\n"
-     "      -w <path>    Insert watch at <path>\n"
-     "      -W <path>    Remove watch at <path>\n"
+    printf(
+    "usage: auditctl [options]\n"
+     "    -a <l,a>            Append rule to end of <l>ist with <a>ction\n"
+     "    -A <l,a>            Add rule at beginning of <l>ist with <a>ction\n"
+     "    -b <backlog>        Set max number of outstanding audit buffers\n"
+     "                        allowed Default=64\n"
+     "    -d <l,a>            Delete rule from <l>ist with <a>ction\n"
+     "                        l=task,entry,exit,user,watch,exclude\n"
+     "                        a=never,possible,always\n"
+     "    -D                  Delete all rules and watches\n"
+     "    -e [0..2]           Set enabled flag\n"
+     "    -f [0..2]           Set failure flag\n"
+     "                        0=silent 1=printk 2=panic\n"
+     "    -F f=v              Build rule: field name, operator(=,!=,<,>,<=,\n"
+     "                        >=,^,&) value\n"
+     "    -h                  Help\n"
+     "    -i                  Ignore errors when reading rules from file\n"
+     "    -k <key>            Set filter key on audit rule\n"
+     "    -l                  List rules\n"
+     "    -m text             Send a user-space message\n"
+     "    -p [r|w|x|a]        Set permissions filter on watch\n"
+     "                        r=read, w=write, x=execute, a=attribute\n"
+     "    -q <mount,subtree>  make subtree part of mount point's dir watches\n"
+     "    -r <rate>           Set limit in messages/sec (0=none)\n"
+     "    -R <file>           read rules from file\n"
+     "    -s                  Report status\n"
+     "    -S syscall          Build rule: syscall name or number\n"
+     "    -t                  Trim directory watches\n"
+     "    -v                  Version\n"
+     "    -w <path>           Insert watch at <path>\n"
+     "    -W <path>           Remove watch at <path>\n"
      );
 }
 
+/* Returns 0 ok, 1 deprecated action, 2 error */
 static int audit_rule_setup(const char *opt, int *flags, int *act)
 {
 	if (strstr(opt, "task")) 
@@ -143,18 +168,19 @@ static int audit_rule_setup(const char *opt, int *flags, int *act)
 		*flags = AUDIT_FILTER_EXIT;
 	else if (strstr(opt, "user"))
 		*flags = AUDIT_FILTER_USER;
-	else if (strstr(opt, "watch"))
-		*flags = AUDIT_FILTER_WATCH;
-	else
-		return 1;
+	else if (strstr(opt, "exclude")) {
+		*flags = AUDIT_FILTER_EXCLUDE;
+		exclude = 1;
+	} else
+		return 2;
 	if (strstr(opt, "never"))
 		*act = AUDIT_NEVER;
 	else if (strstr(opt, "possible"))
-		*act = AUDIT_POSSIBLE;
+		return 1;
 	else if (strstr(opt, "always"))
 		*act = AUDIT_ALWAYS;
 	else
-		return 1;
+		return 2;
 	return 0;
 }
 
@@ -171,6 +197,10 @@ static int check_path(const char *path)
 		fprintf(stderr, "The path passed for the watch is too big\n");
 		return 1;
 	}
+	if (path[0] != '/') {
+		fprintf(stderr, "The path must start with '/'\n");
+		return 1;
+	}
 	ptr = strdup(path);
 	base = basename(ptr);
 	nlen = strlen(base);
@@ -197,66 +227,41 @@ static int check_path(const char *path)
  * down to <name> (of terminating file or directory). 
  * Returns a 1 on success & -1 on failure.
  */
-static int audit_setup_watch_name(struct audit_watch *req, const char *opt, 
-	int *act)
+static int audit_setup_watch_name(struct audit_rule_data **rulep, char *path)
 {
-	if (!req->name) {
-		if (check_path(opt))
-			return -1;
-		req->name = strdup(opt);
-		if (!req->name) {
-			fprintf(stderr, "Out of memory\n");
-			return -1;
-		}
+	int type = AUDIT_WATCH;
+	size_t len;
+	struct stat buf;
 
-		// Trim trailing '/' should they exist
-		req->namelen = strlen(req->name);
-		if (req->namelen > 2 && req->name[req->namelen-1] == '/') {
-			while (req->name[req->namelen-1] == '/' &&
-					 req->namelen > 1) {
-				req->name[req->namelen-1] = 0;
-				req->namelen--;
-			}
-		}
+	if (check_path(path))
+		return -1;
 
-		*act = 1;
-		return 1;
+	// Trim trailing '/' should they exist
+	len = strlen(path);
+	if (len > 2 && path[len-1] == '/') {
+		while (path[len-1] == '/' && len > 1) {
+			path[len-1] = 0;
+			len--;
+		}
 	}
-	fprintf(stderr, "There's already a watch given\n");
-	return -1;
-}
-
-/*
- * Setup a filterkey for the watch.  
- * Returns a 1 on success & -1 on failure.
- */
-static int audit_setup_filterkey(struct audit_watch *req, const char *opt)
-{
-	if (!req->filterkey) {
-		req->filterkey = strdup(opt);
-		if (!req->filterkey) {
-			fprintf(stderr, "Out of memory\n");
-			return -1;
-		}
-		req->fklen = strlen(opt);
-		if (req->fklen > AUDIT_FILTERKEY_MAX) {
-			fprintf(stderr, "The filterkey is too big\n");
-			free(req->filterkey);
-			return -1;
-		}
-		return 1;
+	if (stat(path, &buf) == 0) {
+		if (S_ISDIR(buf.st_mode))
+			type = AUDIT_DIR;
 	}
-	fprintf(stderr, "There's already a filterkey given\n");
-	return -1;
+	/* FIXME: might want to check to see that rule is empty */
+	if (audit_add_watch_dir(type, rulep, path)) 
+		return -1;
+
+	return 1;
 }
 
 /*
  * Setup a watch permissions.
  * Returns a 1 on success & -1 on failure.
  */
-static int audit_setup_perms(struct audit_watch *req, const char *opt)
+static int audit_setup_perms(struct audit_rule_data *rule, const char *opt)
 {
-	unsigned int i, len;
+	unsigned int i, len, val = 0;
 
 	len = strlen(opt);
 	if (len > 4)
@@ -265,72 +270,77 @@ static int audit_setup_perms(struct audit_watch *req, const char *opt)
 	for (i = 0; i < len; i++) {
 		switch (tolower(opt[i])) {
 			case 'r':
-				if (!(req->perms & WATCH_MAY_READ))
-					req->perms |= WATCH_MAY_READ;
-				else
-					goto audit_watch_setup_exit;
+				val |= AUDIT_PERM_READ;
 				break;
 			case 'w':
-				if (!(req->perms & WATCH_MAY_WRITE))
-					req->perms |= WATCH_MAY_WRITE;
-				else
-					goto audit_watch_setup_exit;
+				val |= AUDIT_PERM_WRITE;
 				break;
 			case 'x':
-				if (!(req->perms & WATCH_MAY_EXEC))
-					req->perms |= WATCH_MAY_EXEC;
-				else
-					goto audit_watch_setup_exit;
+				val |= AUDIT_PERM_EXEC;
 				break;
 			case 'a':
-				if (!(req->perms & WATCH_MAY_APPEND))
-					req->perms |= WATCH_MAY_APPEND;
-				else
-					goto audit_watch_setup_exit;
+				val |= AUDIT_PERM_ATTR;
 				break;
 			default:
-				fprintf(stderr, 
-					"Permission %c isn't supported\n", 
+				fprintf(stderr,
+					"Permission %c isn't supported\n",
 					opt[i]);
 				return -1;
 		}
 	}
-	return 1;
 
-audit_watch_setup_exit:
-	fprintf(stderr, "Permission %c was already given\n", opt[i]);
+	if (audit_update_watch_perms(rule, val) == 0)
+		return 1;
 	return -1;
 }
 
-void audit_request_both_lists(int fd)
+static int equiv_parse(char *optarg, char **mp, char **sub)
 {
-	int rc;
-	if (audit_request_rules_list(fd) > 0) {
-		list_requested = 1;
-		get_reply();
-	}
- 
-	if ((rc = audit_request_watch_list(fd)) > 0) {
-		list_requested = 2;
-		get_reply();
-	} else if (rc == -EINVAL)
-		printf("File system watches not supported\n");
+	char *ptr = strchr(optarg, ',');
+	if (ptr == NULL)
+		return -1;	// no comma
+	*ptr = 0;
+	ptr++;
+	if (*ptr == 0)
+		return -1;	// ends with comma
+	*mp = optarg;
+	*sub = ptr;
+	if (strchr(*sub, ','))
+		return -1;	// too many commas
+	return 0;
 }
 
+void audit_request_rule_list(int fd)
+{
+	int rc;
+
+	/* Try out the new message type */
+	if ((rc = audit_request_rules_list_data(fd)) > 0) {
+		list_requested = 1;
+		get_reply();
+	} else if (rc == -EINVAL) { /* Not supported...drop back to old one */
+		if (audit_request_rules_list(fd) > 0) {
+			list_requested = 1;
+			get_reply();
+		}
+	}
+}
+// FIXME: Change these to enums
 /*
- * returns: -2 success - no reply, -1 error - noreply, 0 success - reply,
- *  > 0 success - rule
+ * returns: -3 depreacted, -2 success - no reply, -1 error - noreply,
+ * 0 success - reply, > 0 success - rule
  */
 static int setopt(int count, char *vars[])
 {
     int c;
-    int retval = 0;
+    int retval = 0, rc;
 
     optind = 0;
     opterr = 0;
     while ((retval >= 0) && (c = getopt(count, vars,
-			"hislDve:f:r:b:a:A:d:S:F:m:t:R:w:W:k:p:")) != EOF) {
+			"hislDvte:f:r:b:a:A:d:S:F:m:R:w:W:k:p:q:")) != EOF) {
 	int flags = AUDIT_FILTER_UNSET;
+	rc = 10;	// Init to something impossible to see if unused.
         switch (c) {
         case 'h':
 		usage();
@@ -348,13 +358,14 @@ static int setopt(int count, char *vars[])
 		break;
         case 'e':
 		if (optarg && ((strcmp(optarg, "0") == 0) ||
-				(strcmp(optarg, "1") == 0))) {
+				(strcmp(optarg, "1") == 0) ||
+				(strcmp(optarg, "2") == 0))) {
 			if (audit_set_enabled(fd, strtoul(optarg,NULL,0)) > 0)
 				audit_request_status(fd);
 			else
 				retval = -1;
 		} else {
-			fprintf(stderr, "Enable must be 0 or 1 was %s\n", 
+			fprintf(stderr, "Enable must be 0, 1, or 2 was %s\n", 
 				optarg);
 			retval = -1;
 		}
@@ -419,7 +430,7 @@ static int setopt(int count, char *vars[])
 			retval = -1;
 			break;
 		}
-		audit_request_both_lists(fd);
+		audit_request_rule_list(fd);
 		retval = -2;
 		break;
         case 'a':
@@ -427,37 +438,53 @@ static int setopt(int count, char *vars[])
 			fprintf(stderr, 
 				"Syscall auditing requested for task list\n");
 			retval = -1;
+		} else {
+			rc = audit_rule_setup(optarg, &add, &action);
+			if (rc > 1) {
+				fprintf(stderr, 
+					"Append rule - bad keyword %s\n",
+					optarg);
+				retval = -1;
+			} else if (rc == 1) {
+				fprintf(stderr, 
+				    "Append rule - possible is deprecated\n");
+				return -3; /* deprecated - eat it */
+			} else
+				retval = 1; /* success - please send */
 		}
-		else if (audit_rule_setup(optarg, &add, &action) != 0) {
-			fprintf(stderr, "Append rule - bad keyword %s\n", 
-				optarg);
-			retval = -1;
-		}
-		else
-			retval = 1; /* success - please send */
 		break;
         case 'A': 
 		if (strstr(optarg, "task") && audit_syscalladded) {
 			fprintf(stderr, 
 			   "Error: syscall auditing requested for task list\n");
 			retval = -1;
-		}
-		else if (audit_rule_setup(optarg, &add, &action) == 0) {
-			add |= AUDIT_FILTER_PREPEND;
-			retval = 1; /* success - please send */
-		}
-		else {
-			fprintf(stderr, "Add rule - bad keyword %s\n", optarg);
-			retval = -1;
+		} else {
+			rc = audit_rule_setup(optarg, &add, &action);
+			if (rc > 1) {
+				fprintf(stderr,
+				"Add rule - bad keyword %s\n", optarg);
+				retval = -1;
+			} else if (rc == 1) {
+				fprintf(stderr, 
+				    "Append rule - possible is deprecated\n");
+				return -3; /* deprecated - eat it */
+			} else {
+				add |= AUDIT_FILTER_PREPEND;
+				retval = 1; /* success - please send */
+			}
 		}
 		break;
         case 'd': 
-		if (audit_rule_setup(optarg, &del, &action) != 0) {
+		rc = audit_rule_setup(optarg, &del, &action);
+		if (rc > 1) {
 			fprintf(stderr, "Delete rule - bad keyword %s\n", 
 				optarg);
 			retval = -1;
-		}
-		else
+		} else if (rc == 1) {
+			fprintf(stderr, 
+			    "Delete rule - possible is deprecated\n");
+			return -3; /* deprecated - eat it */
+		} else
 			retval = 1; /* success - please send */
 		break;
         case 'S':
@@ -477,6 +504,10 @@ static int setopt(int count, char *vars[])
 			fprintf(stderr, 
 			  "Error: syscall auditing being added to user list\n");
 			return -1;
+		} else if (exclude) {
+			fprintf(stderr, 
+		    "Error: syscall auditing cannot be put on exclude list\n");
+			return -1;
 		} else {
 			if (!audit_elf) {
 				int machine;
@@ -496,7 +527,12 @@ static int setopt(int count, char *vars[])
 				audit_elf = elf;
 			}
 		}
-		switch (audit_rule_syscallbyname(&rule, optarg))
+		if (which == OLD) 
+			rc = audit_rule_syscallbyname(&rule, optarg);
+		else
+			rc = audit_rule_syscallbyname(
+				(struct audit_rule *)rule_new, optarg);
+		switch (rc)
 		{
 			case 0:
 				audit_syscalladded = 1;
@@ -525,8 +561,27 @@ static int setopt(int count, char *vars[])
 			fprintf(stderr, "List must be given before field\n");
 			retval = -1;
 			break;
-		}	
-		switch (audit_rule_fieldpair(&rule, optarg, flags))
+		}
+		if (strncmp(optarg, "msgtype=", 7) == 0) {
+			if (msgtype_cnt) {
+				fprintf(stderr, 
+				    "Only one msgtype may be given per rule\n");
+				retval = -1;
+				break;
+			} else
+				msgtype_cnt++;
+		}
+		if (which == OLD) {
+			char *ptr = strdup(optarg);
+			rc = audit_rule_fieldpair(&rule, ptr, flags);
+			if (rc == -10)
+				upgrade_rule(); /* need to upgrade... */
+			free(ptr);
+		}
+		if (which == NEW) 
+			rc = audit_rule_fieldpair_data(&rule_new,optarg,flags);
+//FIXME: make this a function
+		switch (rc)
 		{
 			case 0:
 				break;
@@ -581,55 +636,43 @@ static int setopt(int count, char *vars[])
 		 "msgtype field can only be used with exclude filter list\n");
 				retval = -1;
 				break;
+			case -10:
+				fprintf(stderr,
+					"Failed upgrading rule\n");
+				retval = -1;
+			case -11:
+				fprintf(stderr,
+					"String value too long\n");
+				retval = -1;
+				break;
+			case -12:
+				fprintf(stderr,
+			"Only msgtype field can be used with exclude filter\n");
+				retval = -1;
+				break;
+			case -13:
+				fprintf(stderr,
+			"Field (%s) only takes = or != operators\n", optarg);
+				retval = -1;
+				break;
+			case -14:
+				fprintf(stderr,
+				"Permission (%s) can only contain \'rwxa\n",
+					optarg);
+				retval = -1;
+				break;
 			default:
 				retval = -1;
 				break;
 		}
 		break;
         case 'm':
-//		if (audit_send_message(fd, AUDIT_USER, optarg) <= 0)
 		if (audit_log_user_message( fd, AUDIT_USER, optarg, NULL, 
 				NULL, NULL, 1) <=0)
 			retval = -1;
 		else
 			return -2;  // success - no reply for this
 		break;
-        case 't':
-		if (optarg && isdigit(optarg[0])) {
-			const char *ptr, *mstr;
-			int machine;
-			if (audit_elf) {
-				machine = audit_elf_to_machine(audit_elf);
-				if (machine < 0) {
-					fprintf(stderr, 
-					    "Error translating machine type");
-					return -1;
-				}
-			} else {
-				machine = audit_detect_machine();
-				if (machine < 0) {
-					fprintf(stderr, 
-					    "Error detecting machine type");
-					return -1;
-				}
-			}
-			mstr = audit_machine_to_name(machine);
-			if (mstr == NULL)
-				mstr = "Unknown";
-                        ptr = audit_syscall_to_name(strtol(optarg, NULL,0), machine);
-			if (ptr)
-				printf("arch:%s, syscall:%s\n", mstr, ptr);
-			else
-				printf("Translate unknown value: %s\n", optarg);
-			retval = -2; /* Success - no reply */
-			break;
-		}
-		else  
-			fprintf(stderr, 
-				"Translate needs a numeric value was %s\n",
-				 optarg);
-		retval = -1; /* No reply needed */
-		break;
 	case 'R':
 		fprintf(stderr, "Error - nested rule files not supported\n");
 		retval = -1;
@@ -641,63 +684,123 @@ static int setopt(int count, char *vars[])
 			retval = -1;
 			break;
 		}
-		{
-			// make sure both returns are accounted for
-			int rc1 = delete_all_rules();
-			retval = delete_all_watches();
-			if (rc1)
-				retval = rc1;
-			if (!retval)
-				retval = -2; // success no reply
+		retval = delete_all_rules(fd);
+		if (retval == 0) {
+			audit_request_rule_list(fd);
+			retval = -2;
 		}
-		audit_request_both_lists(fd);
 		break;
 	case 'w':
-		if (optarg) 
-			retval = audit_setup_watch_name(&watch, optarg, &ins);
-		else {
+		if (optarg) { 
+			add = AUDIT_FILTER_EXIT;
+			action = AUDIT_ALWAYS;
+			which = NEW;
+			audit_syscalladded = 1;
+			retval = audit_setup_watch_name(&rule_new, optarg);
+		} else {
 			fprintf(stderr, "watch option needs a path\n");	
 			retval = -1;
 		}
 		break;
 	case 'W':
-		if (optarg) 
-			retval = audit_setup_watch_name(&watch, optarg, &rem);
-		else {
+		if (optarg) { 
+			del = AUDIT_FILTER_EXIT;
+			action = AUDIT_ALWAYS;
+			which = NEW;
+			audit_syscalladded = 1;
+			retval = audit_setup_watch_name(&rule_new, optarg);
+		} else {
 			fprintf(stderr, "watch option needs a path\n");	
 			retval = -1;
 		}
 		break;
 	case 'k':
-		if (!ins && !rem) {
-			fprintf(stderr, 
-			"filterkey option needs a watch given prior to it\n");	
+		if (audit_syscalladded != 1 ||
+				(add==AUDIT_FILTER_UNSET &&
+					del==AUDIT_FILTER_UNSET)) {
+			fprintf(stderr,
+			"key option needs a watch or syscall given prior to it\n");
 			retval = -1;
-		}
-		else if (!optarg) {
-			fprintf(stderr, "filterkey option needs a filter\n");	
+		} else if (!optarg) {
+			fprintf(stderr, "key option needs a value\n");
 			retval = -1;
+		} else {
+			int flags = 0;
+			char *cmd=NULL;
+
+			/* Get the flag */
+			if (add != AUDIT_FILTER_UNSET)
+				flags = add & AUDIT_FILTER_MASK;
+			else if (del != AUDIT_FILTER_UNSET)
+				flags = del & AUDIT_FILTER_MASK;
+
+			/* Build the command */
+			asprintf(&cmd, "key=%s", optarg);
+			if (cmd) {
+				/* Add this to the rule */
+				int ret;
+				if (which == OLD)
+					upgrade_rule();
+				ret = audit_rule_fieldpair_data(&rule_new,
+					cmd, flags);
+				if (ret < 0)
+					retval = -1;
+				free(cmd);
+			} else {
+				fprintf(stderr, "Out of memory adding key\n");
+				retval = -1;
+			}
 		}
-		else 
-			retval =  audit_setup_filterkey(&watch, optarg);
 		break;
 	case 'p':
-		if (!ins && !rem) {
+		if (!add && !del) {
+			fprintf(stderr,
+			"permission option needs a watch given prior to it\n");
+			retval = -1;
+		} else if (!optarg) {
+			fprintf(stderr, "permission option needs a filter\n");
+			retval = -1;
+		} else {
+			if (which == OLD) {
+				fprintf(stderr,
+				"You must give a watch prior to perms\n");
+				retval = -1;
+			} else
+				retval = audit_setup_perms(rule_new, optarg);
+		}
+		break;
+        case 'q':
+		if (audit_syscalladded) {
 			fprintf(stderr, 
-			"permission option needs a watch given prior to it\n");	
+			   "Syscall auditing requested for make equivalent\n");
 			retval = -1;
+		} else {
+			char *mp, *sub;
+			retval = equiv_parse(optarg, &mp, &sub);
+			if (retval < 0) {
+				fprintf(stderr, 
+			   "Error parsing equivalent parts\n");
+				retval = -1;
+			} else {
+				retval = audit_make_equivalent(fd, mp, sub);
+				if (retval <= 0) {
+					retval = -1;
+				} else
+					return -2; // success - no reply needed
+			}
 		}
-		else if (!optarg) {
-			fprintf(stderr, "permission option needs a filter\n");	
+		break;
+        case 't':
+		retval = audit_trim_subtrees(fd);
+		if (retval <= 0)
 			retval = -1;
-		}
 		else
-			retval = audit_setup_perms(&watch, optarg);
+			return -2;  // success - no reply for this
 		break;
 	case 'v':
-			printf("auditctl version %s\n", VERSION);
-			retval = -2;
-			break;
+		printf("auditctl version %s\n", VERSION);
+		retval = -2;
+		break;
         default: 
 		usage();
 		retval = -1;
@@ -820,12 +923,16 @@ static int fileopt(const char *file)
 		rc = setopt(i, options);
 
 		/* handle reply or send rule */
-		if (handle_request(rc) == -1) {
-			fprintf(stderr, "There was an error in line %d of %s\n",
-				lineno, file);
-			if (!ignore) {
-				fclose(f);
-				return -1;
+		if (rc != -3) {
+			if (handle_request(rc) == -1) {
+				if (errno != ECONNREFUSED)
+					fprintf(stderr,
+					"There was an error in line %d of %s\n",
+					lineno, file);
+				if (!ignore) {
+					fclose(f);
+					return -1;
+				}
 			}
 		}
 		lineno++;
@@ -870,6 +977,8 @@ int main(int argc, char *argv[])
 		if (reset_vars())
 			return 1;
 		retval = setopt(argc, argv);
+		if (retval == -3)
+			return 0;
 	}
 	return handle_request(retval);
 }
@@ -897,21 +1006,35 @@ static int handle_request(int status)
 		if (add != AUDIT_FILTER_UNSET) {
 			// if !task add syscall any if not specified
 			if ((add & AUDIT_FILTER_MASK) != AUDIT_FILTER_TASK && 
-					audit_syscalladded != 1)
-				audit_rule_syscallbyname(&rule, "all");
-			rc = audit_add_rule(fd, &rule, add, action);
+					audit_syscalladded != 1) {
+				if (which == OLD)
+					audit_rule_syscallbyname(&rule, "all");
+				else
+					audit_rule_syscallbyname_data(
+							rule_new, "all");
+			}
+			if (which == OLD) {
+				rc = audit_add_rule(fd, &rule, add, action);
+			} else {
+				rc = audit_add_rule_data(fd, rule_new,
+								 add, action);
+			}
 		}
 		else if (del != AUDIT_FILTER_UNSET) {
 			if ((del & AUDIT_FILTER_MASK) != AUDIT_FILTER_TASK && 
-					audit_syscalladded != 1)
-				audit_rule_syscallbyname(&rule, "all");
-			rc = audit_delete_rule(fd, &rule, del, action);
-		}
-		else if (ins && !rem) 
-			rc = audit_insert_watch(fd, &watch);
-		else if (rem && !ins)
-			rc = audit_remove_watch(fd, &watch);
-		else {
+					audit_syscalladded != 1) {
+				if (which == OLD)
+					audit_rule_syscallbyname(&rule, "all");
+				else
+					audit_rule_syscallbyname_data(
+							rule_new, "all");
+			}
+			if (which == OLD)
+				rc = audit_delete_rule(fd, &rule, del, action);
+			else
+				rc = audit_delete_rule_data(fd, rule_new,
+								 del, action);
+		} else {
         		usage();
 	    		audit_close(fd);
 			exit(1);
@@ -964,30 +1087,6 @@ static void get_reply(void)
 	}
 }
 
-struct audit_watch *transport_to_watch(void *memblk)
-{
-	unsigned int offset;
-	struct watch_transport *r = (struct watch_transport *)memblk;
-	struct audit_watch *req = (struct audit_watch *)malloc(
-					sizeof(struct audit_watch));
-	if (req == NULL)
-		return NULL;
-
-	req->dev_major = r->dev_major;
-	req->dev_minor = r->dev_minor;
-	req->perms = r->perms;
-	req->valid = r->valid;
-	req->namelen = r->pathlen;
-	req->fklen =  r->fklen;
-
-	offset = sizeof(struct watch_transport);
-	req->filterkey = strndup(memblk + offset, req->fklen);
-	offset += req->fklen;
-	req->name = strndup(memblk + offset, req->namelen);
-
-	return req;
-}
-
 /*
  * This function interprets the reply and prints it to stdout. It returns
  * 0 if no more should be read and 1 to indicate that more messages of this
@@ -999,6 +1098,8 @@ static int audit_print_reply(struct audit_reply *rep)
 	int first;
 	int sparse;
 	int machine = audit_detect_machine();
+	size_t boffset;
+	int show_syscall;
 
 	audit_elf = 0; 
 	switch (rep->type) {
@@ -1007,8 +1108,6 @@ static int audit_print_reply(struct audit_reply *rep)
 		case NLMSG_DONE:
 			if (list_requested == 1)
 				printf("No rules\n");
-			else if (list_requested == 2)
-				printf("No watches\n");
 			return 0;
 		case NLMSG_ERROR: 
 		        printf("NLMSG_ERROR %d (%s)\n",
@@ -1024,47 +1123,112 @@ static int audit_print_reply(struct audit_reply *rep)
 			rep->status->backlog);
 			return 0;
 		case AUDIT_LIST:
+		case AUDIT_LIST_RULES:
 			list_requested = 0;
-			printf("AUDIT_LIST: %s,%s",
+			boffset = 0;
+			show_syscall = 1;
+			printf("%s: %s,%s", audit_msg_type_to_name(rep->type),
 				audit_flag_to_name((int)rep->rule->flags),
 				audit_action_to_name(rep->rule->action));
-			for (i = 0; i < rep->rule->field_count; i++) {
-				int field = rep->rule->fields[i];
-				int negated = 0;
 
-				if (field & AUDIT_NEGATE) {
-					field &= ~AUDIT_NEGATE;
-					negated  = 1;
+			for (i = 0; i < rep->rule->field_count; i++) {
+				const char *name;
+				int op;
+				int field = rep->rule->fields[i] &
+					~AUDIT_OPERATORS & ~AUDIT_NEGATE;
+				if (rep->type == AUDIT_LIST_RULES) {
+					op = rep->ruledata->fieldflags[i] &
+					(AUDIT_OPERATORS | AUDIT_NEGATE);
+				} else {
+					op = rep->rule->fields[i] &
+					    (AUDIT_OPERATORS | AUDIT_NEGATE);
 				}
                 
-				const char *name = audit_field_to_name(field);
+				name = audit_field_to_name(field);
 				if (name) {
 					if (strcmp(name, "arch") == 0) { 
 						audit_elf =rep->rule->values[i];
 						printf(" %s%s%u", name, 
-							negated ? "!=" : "=",
+							audit_operator_to_symbol(op),
 							(unsigned)rep->rule->values[i]);
 					}
-					else 
+					else if (strcmp(name, "msgtype") == 0) {
+						if (!audit_msg_type_to_name(rep->rule->values[i]))
+							printf(" %s%s%d", name,
+								audit_operator_to_symbol(op),
+								rep->rule->values[i]);
+						else {
+							printf(" %s%s%s", name,
+								audit_operator_to_symbol(op),
+								audit_msg_type_to_name(rep->rule->values[i]));
+						}
+					} else if ((field >= AUDIT_SUBJ_USER &&
+						  field <= AUDIT_OBJ_LEV_HIGH)
+						&& field != AUDIT_PPID &&
+					       rep->type == AUDIT_LIST_RULES) {
+						printf(" %s%s%.*s", name,
+						  audit_operator_to_symbol(op),
+						  rep->ruledata->values[i],
+						  &rep->ruledata->buf[boffset]);
+						boffset +=
+						    rep->ruledata->values[i];
+					} else if (field == AUDIT_WATCH) {
+						printf(" watch=%.*s", 
+						  rep->ruledata->values[i],
+						  &rep->ruledata->buf[boffset]);
+						boffset +=
+						    rep->ruledata->values[i];
+					} else if (field == AUDIT_FILTERKEY) {
+						printf(" key=%.*s",
+						rep->ruledata->values[i],
+						&rep->ruledata->buf[boffset]);
+						boffset +=
+						    rep->ruledata->values[i];
+					} else if (field == AUDIT_PERM) {
+						char perms[5];
+						int val=rep->rule->values[i];
+						perms[0] = 0;
+						if (val & AUDIT_PERM_READ)
+							strcat(perms, "r");
+						if (val & AUDIT_PERM_WRITE)
+							strcat(perms, "w");
+						if (val & AUDIT_PERM_EXEC)
+							strcat(perms, "x");
+						if (val & AUDIT_PERM_ATTR)
+							strcat(perms, "a");
+						printf(" perm=%s", perms);
+						show_syscall = 0;
+					} else {
 						printf(" %s%s%d", name, 
-							negated ? "!=" : "=",
+							audit_operator_to_symbol(op),
 							rep->rule->values[i]);
-				}
-				else 
+					}
+				} else { 
 					printf(" f%d%s%d", rep->rule->fields[i],
-						negated ? "!=" : "=",
+						audit_operator_to_symbol(op),
 						rep->rule->values[i]);
-				if (rep->rule->values[i])
+				}
+				/* Avoid printing value if the field type is 
+				 * known to return a string. */
+				if (rep->rule->values[i] && 
+						(field < AUDIT_SUBJ_USER ||
+						 field > AUDIT_SUBJ_CLR) &&
+						field != AUDIT_WATCH &&
+						field != AUDIT_FILTERKEY &&
+						field != AUDIT_PERM)
 					printf(" (0x%x)", rep->rule->values[i]);
 			}
-			if ((rep->rule->flags & AUDIT_FILTER_MASK) != 
-						AUDIT_FILTER_USER) {
+			if (show_syscall &&
+				((rep->rule->flags & AUDIT_FILTER_MASK) != 
+						AUDIT_FILTER_USER) &&
+				((rep->rule->flags & AUDIT_FILTER_MASK) !=
+						AUDIT_FILTER_TASK)) {
 				printf(" syscall=");
-				for (sparse = 0, i = 0; i < AUDIT_BITMASK_SIZE;
-					 i++)
-					if (rep->rule->mask[i] !=
-						 (unsigned int)~0)
+				for (sparse = 0, i = 0; 
+					i < (AUDIT_BITMASK_SIZE-1); i++) {
+					if (rep->rule->mask[i] != (uint32_t)~0)
 						sparse = 1;
+				}
 				if (!sparse) {
 					printf("all");
 				} else for (first = 1, i = 0;
@@ -1095,34 +1259,6 @@ static int audit_print_reply(struct audit_reply *rep)
 			}
 			printf("\n");
 			return 1; /* get more messages until NLMSG_DONE */
-		case AUDIT_WATCH_LIST:
-		{
-			char perms[5];
-			struct audit_watch *req=transport_to_watch(rep->watch);
-			list_requested = 0;
-			if (!req) {
-				fputs("No memory receiving watch list\n", 
-						stderr);
-				return 0;
-			}
-			perms[0] = 0;
-		        if (req->perms & WATCH_MAY_READ)
-				strcat(perms, "r");
-		        if (req->perms & WATCH_MAY_WRITE)
-				strcat(perms, "w");
-		        if (req->perms & WATCH_MAY_EXEC)
-				strcat(perms, "x");
-		        if (req->perms & WATCH_MAY_APPEND)
-				strcat(perms, "a");
-			printf("AUDIT_WATCH_LIST: dev=%u:%u, path=%s, "
-				"filterkey=%s, perms=%s, valid=%d\n",
-				req->dev_major, req->dev_minor, req->name,
-				req->filterkey, perms, req->valid);
-			free(req->name);
-			free(req->filterkey);
-			free(req);
-			return 1; /* get more messages until NLMSG_DONE */
-		}
 		default:
 			printf("Unknown: type=%d, len=%d\n", rep->type, 
 				rep->nlh->nlmsg_len);
@@ -1130,167 +1266,3 @@ static int audit_print_reply(struct audit_reply *rep)
 	}
 }
 
-/* Returns 0 for success and -1 for failure */
-static int delete_all_rules(void)
-{
-	int seq, i;
-	int timeout = 40; /* tenths of seconds */
-	struct audit_reply rep;
-	fd_set read_mask;
-
-	/* list the rules */
-	seq = audit_request_rules_list(fd);
-	if (seq <= 0) 
-		return -1;
-
-	FD_ZERO(&read_mask);
-	FD_SET(fd, &read_mask);
-
-	for (i = 0; i < timeout; i++) {
-		struct timeval t;
-		int rc;
-
-		t.tv_sec  = 0;
-		t.tv_usec = 100000; /* .1 second */
-		do {
-			rc = select(fd+1, &read_mask, NULL, NULL, &t);
-		} while (rc < 0 && errno == EINTR);
-		// We'll try to read just in case
-		rc = audit_get_reply(fd, &rep, GET_REPLY_NONBLOCKING, 0);
-		if (rc > 0) {
-			/* Reset timeout */
-			i = 0;
-
-			/* Don't make decisions based on wrong packet */
-			if (rep.nlh->nlmsg_seq != seq)
-				continue;
-
-			/* If we get done or error, break out */
-			if (rep.type == NLMSG_DONE)
-				break;
-
-			if (rep.type == NLMSG_ERROR && rep.error->error) {
-				fprintf(stderr, 
-					"Error receiving rules list (%s)\n", 
-					strerror(-rep.error->error));
-				return -1;
-			}
-
-			/* If its not what we are expecting, keep looping */
-			if (rep.type != AUDIT_LIST)
-				continue;
-
-			/* Found it, bounce it right back with delete */
-			rc = audit_send(fd, AUDIT_DEL, rep.rule, 
-					sizeof(struct audit_rule));
-			if (rc < 0) {
-				fprintf(stderr, "Error deleting rule (%s)\n",
-					strerror(-rc)); 
-				return -1;
-			}
-		}
-	}
-
-	return 0;
-}
-
-/* This function deletes all watches. It returns: -2 if watches are not 
- * supported, -1 if there was an error, and 0 for success */
-static int delete_all_watches(void)
-{
-	int i, seq;
-	int timeout = 40; /* tenths of seconds */
-	struct audit_reply rep;
-	llist entries;	/* list of watches */
-	const lnode *nptr;
-	fd_set read_mask;
-
-	/* list the rules */
-	seq = audit_request_watch_list(fd);
-	if (seq <= 0) {
-		// EINVAL means that it does have file audit support
-		if (seq == -EINVAL)
-			return -2;
-		else
-			return -1;
-	}
-
-	FD_ZERO(&read_mask);
-	FD_SET(fd, &read_mask);	
-	list_create(&entries);
-
-	for (i = 0; i < timeout; i++) {
-		struct timeval t;
-		int rc;
-
-		t.tv_sec  = 0;
-		t.tv_usec = 100000; /* .1 second */
-		do {
-			rc = select(fd+1, &read_mask, NULL, NULL, &t);
-		} while (rc < 0 && errno == EINTR);
-		// We'll try to read just in case
-		rc = audit_get_reply(fd, &rep, GET_REPLY_NONBLOCKING, 0);
-		if (rc > 0) {
-			lnode n;
-		
-			/* Reset timeout */
-			i = 0;
-
-			/* Don't make decisions based on wrong packet */
-			if (rep.nlh->nlmsg_seq != seq)
-				continue;
-
-			/* If we get done or error, break out */
-			if (rep.type == NLMSG_DONE )
-				break;
-
-			if (rep.type == NLMSG_ERROR && rep.error->error) {
-				/* If its the first one & EINVAL assume we
-				   are on a kernel without watch support */
-				if (rep.error->error == -EINVAL && 
-							entries.cnt == 0)
-					return -2;
-				fprintf(stderr,
-					"Error receiving watch list (%s)\n", 
-					strerror(-rep.error->error));
-				return -1;
-			}
-
-			/* If its not what we are expecting, keep looping */
-			if (rep.type != AUDIT_WATCH_LIST)
-				continue;
-
-			/* Add to linked list */
-			n.w = malloc(rep.len);
-			if (n.w == NULL) {
-				fprintf(stderr, "No memory\n");
-				return -1;
-			}
-			memcpy(n.w, rep.watch, rep.len);
-			n.size = rep.len;
-			list_append(&entries, &n);
-		}
-	}
-
-	/* loop through linked list and send */
-	if (entries.cnt) {
-		list_first(&entries);
-		nptr = list_get_cur(&entries);
-		do {
-			int rc = audit_send(fd, AUDIT_WATCH_REM, nptr->w, 
-					nptr->size);
-			if (rc < 0) {
-				fprintf(stderr, "Error deleting watch (%s)\n", 
-					strerror(-rc));
-				list_clear(&entries);
-				return -1;
-			}
-		} while ((nptr=list_next(&entries)));
-
-		/* Done with list - delete it */	
-		list_clear(&entries);
-	}
-
-	return 0;
-}
-
