@@ -1,5 +1,5 @@
 /* auditd-config.c -- 
- * Copyright 2004-06 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-2007 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -42,12 +42,14 @@ struct nv_pair
 {
 	const char *name;
 	const char *value;
+	const char *option;
 };
 
 struct kw_pair 
 {
 	const char *name;
-	int (*parser)(const char *, int, struct daemon_conf *);
+	int (*parser)(struct nv_pair *, int, struct daemon_conf *);
+	int max_options;
 };
 
 struct nv_list
@@ -59,58 +61,61 @@ struct nv_list
 static char *get_line(FILE *f, char *buf);
 static int nv_split(char *buf, struct nv_pair *nv);
 static const struct kw_pair *kw_lookup(const char *val);
-static int log_file_parser(const char *val, int line, 
+static int log_file_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int num_logs_parser(const char *val, int line, 
+static int num_logs_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int qos_parser(const char *val, int line,
+static int qos_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int dispatch_parser(const char *val, int line,
+static int dispatch_parser(struct nv_pair *nv, int line,
 		struct daemon_conf *config);
-static int max_log_size_parser(const char *val, int line, 
+static int max_log_size_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int max_log_size_action_parser(const char *val, int line, 
+static int max_log_size_action_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int log_format_parser(const char *val, int line, 
+static int log_format_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int flush_parser(const char *val, int line, struct daemon_conf *config);
-static int freq_parser(const char *val, int line, struct daemon_conf *config);
-static int space_left_parser(const char *val, int line, 
+static int flush_parser(struct nv_pair *nv, int line,
 		struct daemon_conf *config);
-static int space_action_parser(const char *val, int line, 
+static int freq_parser(struct nv_pair *nv, int line,
 		struct daemon_conf *config);
-static int action_mail_acct_parser(const char *val, int line, 
+static int space_left_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int admin_space_left_parser(const char *val, int line, 
+static int space_action_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int admin_space_left_action_parser(const char *val, int line, 
+static int action_mail_acct_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int disk_full_action_parser(const char *val, int line, 
+static int admin_space_left_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config);
-static int disk_error_action_parser(const char *val, int line, 
+static int admin_space_left_action_parser(struct nv_pair *nv, int line, 
+		struct daemon_conf *config);
+static int disk_full_action_parser(struct nv_pair *nv, int line, 
+		struct daemon_conf *config);
+static int disk_error_action_parser(struct nv_pair *nv, int line, 
+		struct daemon_conf *config);
+static int priority_boost_parser(struct nv_pair *nv, int line,
 		struct daemon_conf *config);
-static int priority_boost_parser(const char *val, int line, struct daemon_conf *config);
 static int sanity_check(struct daemon_conf *config);
 
 static const struct kw_pair keywords[] = 
 {
-  {"log_file",                 log_file_parser },
-  {"log_format",               log_format_parser },
-  {"flush",                    flush_parser },
-  {"freq",                     freq_parser },
-  {"num_logs",                 num_logs_parser },
-  {"dispatcher",               dispatch_parser },
-  {"disp_qos",                 qos_parser },
-  {"max_log_file",             max_log_size_parser },
-  {"max_log_file_action",      max_log_size_action_parser },
-  {"space_left",               space_left_parser },
-  {"space_left_action",        space_action_parser },
-  {"action_mail_acct",         action_mail_acct_parser },
-  {"admin_space_left",         admin_space_left_parser },
-  {"admin_space_left_action",  admin_space_left_action_parser },
-  {"disk_full_action",         disk_full_action_parser },
-  {"disk_error_action",        disk_error_action_parser },
-  {"priority_boost",           priority_boost_parser },
+  {"log_file",                 log_file_parser,			0 },
+  {"log_format",               log_format_parser,		0 },
+  {"flush",                    flush_parser,			0 },
+  {"freq",                     freq_parser,			0 },
+  {"num_logs",                 num_logs_parser,			0 },
+  {"dispatcher",               dispatch_parser,			0 },
+  {"disp_qos",                 qos_parser,			0 },
+  {"max_log_file",             max_log_size_parser,		0 },
+  {"max_log_file_action",      max_log_size_action_parser,	0 },
+  {"space_left",               space_left_parser,		0 },
+  {"space_left_action",        space_action_parser,		1 },
+  {"action_mail_acct",         action_mail_acct_parser,		0 },
+  {"admin_space_left",         admin_space_left_parser,		0 },
+  {"admin_space_left_action",  admin_space_left_action_parser,	1 },
+  {"disk_full_action",         disk_full_action_parser,		1 },
+  {"disk_error_action",        disk_error_action_parser,	1 },
+  {"priority_boost",           priority_boost_parser,		0 },
   { NULL,                      NULL }
 };
 
@@ -130,12 +135,12 @@ static const struct nv_list flush_techniques[] =
   { NULL,         0 }
 };
 
-// Ideas: e-mail, run command
 static const struct nv_list failure_actions[] =
 {
   {"ignore",  FA_IGNORE },
   {"syslog",  FA_SYSLOG },
   {"email",   FA_EMAIL },
+  {"exec",    FA_EXEC },
   {"suspend", FA_SUSPEND },
   {"single",  FA_SINGLE },
   {"halt",    FA_HALT },
@@ -161,6 +166,13 @@ static const struct nv_list qos_options[] =
 };
 
 const char *email_command = "/usr/lib/sendmail";
+static int allow_links = 0;
+
+
+void set_allow_links(int allow)
+{
+	allow_links = allow;
+}
 
 /*
  * Set everything to its default value
@@ -170,6 +182,7 @@ static void clear_config(struct daemon_conf *config)
 	config->qos = QOS_NON_BLOCKING;
 	config->sender_uid = 0;
 	config->sender_pid = 0;
+	config->sender_ctx = NULL;
 	config->log_file = strdup("/var/log/audit/audit.log");
 	config->log_format = LF_RAW;
 	config->priority_boost = 3;
@@ -181,17 +194,22 @@ static void clear_config(struct daemon_conf *config)
 	config->max_log_size_action = SZ_IGNORE;
 	config->space_left = 0L;
 	config->space_left_action = FA_IGNORE;
+        config->space_left_exe = NULL;
 	config->action_mail_acct = strdup("root");
 	config->admin_space_left= 0L;
 	config->admin_space_left_action = FA_IGNORE;
+        config->admin_space_left_exe = NULL;
 	config->disk_full_action = FA_IGNORE;
+        config->disk_full_exe = NULL;
 	config->disk_error_action = FA_SYSLOG;
+        config->disk_error_exe = NULL;
+
 }
 
 static log_test_t log_test = TEST_AUDITD;
 int load_config(struct daemon_conf *config, log_test_t lt)
 {
-	int fd, rc, lineno = 1;
+	int fd, rc, mode, lineno = 1;
 	struct stat st;
 	FILE *f;
 	char buf[128];
@@ -200,7 +218,10 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 	log_test = lt;
 
 	/* open the file */
-	rc = open(CONFIG_FILE, O_NOFOLLOW|O_RDONLY);
+	mode = O_RDONLY;
+	if (allow_links == 0)
+		mode |= O_NOFOLLOW;
+	rc = open(CONFIG_FILE, mode);
 	if (rc < 0) {
 		if (errno != ENOENT) {
 			audit_msg(LOG_ERR, "Error opening config file (%s)", 
@@ -295,8 +316,18 @@ int load_config(struct daemon_conf *config, log_test_t lt)
 			return 1;
 		}
 
+		/* Check number of options */
+		if (kw->max_options == 0 && nv.option != NULL) {
+			audit_msg(LOG_ERR, 
+				"Keyword \"%s\" has invalid option "
+				"\"%s\" in line %d of %s", 
+				nv.name, nv.option, lineno, CONFIG_FILE);
+			fclose(f);
+			return 1;
+		}
+
 		/* dispatch to keyword's local parser */
-		rc = kw->parser(nv.value, lineno, config);
+		rc = kw->parser(&nv, lineno, config);
 		if (rc != 0) {
 			fclose(f);
 			return 1; // local parser puts message out
@@ -330,6 +361,7 @@ static int nv_split(char *buf, struct nv_pair *nv)
 
 	nv->name = NULL;
 	nv->value = NULL;
+	nv->option = NULL;
 	ptr = strtok(buf, " ");
 	if (ptr == NULL)
 		return 0; /* If there's nothing, go to next line */
@@ -350,10 +382,16 @@ static int nv_split(char *buf, struct nv_pair *nv)
 		return 1;
 	nv->value = ptr;
 
-	/* Make sure there's nothing else */
+	/* See if there's an option */
 	ptr = strtok(NULL, " ");
-	if (ptr)
-		return 1;
+	if (ptr) {
+		nv->option = ptr;
+
+		/* Make sure there's nothing else */
+		ptr = strtok(NULL, " ");
+		if (ptr)
+			return 1;
+	}
 
 	/* Everything is OK */
 	return 0;
@@ -370,17 +408,18 @@ static const struct kw_pair *kw_lookup(const char *val)
 	return &keywords[i];
 }
  
-static int log_file_parser(const char *val, int line,struct daemon_conf *config)
+static int log_file_parser(struct nv_pair *nv, int line,
+	struct daemon_conf *config)
 {
 	char *dir = NULL, *tdir, *base;
 	DIR *d;
 	int fd, mode;
 	struct stat buf;
 
-	audit_msg(LOG_DEBUG, "log_file_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "log_file_parser called with: %s", nv->value);
 
 	/* split name into dir and basename. */
-	tdir = strdup(val);
+	tdir = strdup(nv->value);
 	if (tdir)
 		dir = dirname(tdir);
 	if (dir == NULL || strlen(dir) < 4) { //  '/var' is shortest dirname
@@ -391,7 +430,7 @@ static int log_file_parser(const char *val, int line,struct daemon_conf *config)
 		return 1;
 	}
 
-	base = basename((char *)val);
+	base = basename((char *)nv->value);
 	if (base == 0 || strlen(base) == 0) {
 		audit_msg(LOG_ERR, "The file name: %s is too short - line %d", 
 			base, line);
@@ -417,66 +456,66 @@ static int log_file_parser(const char *val, int line,struct daemon_conf *config)
 	else
 		mode = O_RDONLY;
 
-	fd = open(val, mode);
+	fd = open(nv->value, mode);
 	if (fd < 0) {
 		if (errno == ENOENT) {
-			fd = create_log_file(val);
+			fd = create_log_file(nv->value);
 			if (fd < 0) 
 				return 1;
 		} else {
-			audit_msg(LOG_ERR, "Unable to open %s (%s)", val, 
+			audit_msg(LOG_ERR, "Unable to open %s (%s)", nv->value, 
 					strerror(errno));
 			return 1;
 		}
 	}
 	if (fstat(fd, &buf) < 0) {
-		audit_msg(LOG_ERR, "Unable to stat %s (%s)",
-					val, strerror(errno));
+		audit_msg(LOG_ERR, "Unable to stat %s (%s)", 
+					nv->value, strerror(errno));
 		close(fd);
 		return 1;
 	}
 	close(fd);
 	if (!S_ISREG(buf.st_mode)) {
-		audit_msg(LOG_ERR, "%s is not a regular file", val);
+		audit_msg(LOG_ERR, "%s is not a regular file", nv->value);
 		return 1;
 	}
 	if (buf.st_uid != 0) {
-		audit_msg(LOG_ERR, "%s is not owned by root", val);
+		audit_msg(LOG_ERR, "%s is not owned by root", nv->value);
 		return 1;
 	}
-	if ((buf.st_mode & (S_IRUSR|S_IWUSR|S_IRGRP)) != 
-			   (S_IRUSR|S_IWUSR|S_IRGRP)) {
-		audit_msg(LOG_ERR, "%s permissions should be 0640", val);
+	if ((buf.st_mode & (S_IRWXU|S_IRWXG|S_IRWXO)) != 
+			  (S_IRUSR|S_IWUSR|S_IRGRP)) {
+		audit_msg(LOG_ERR, "%s permissions should be 0640", nv->value);
 		return 1;
 	}
 	free((void *)config->log_file);
-	config->log_file = strdup(val);
+	config->log_file = strdup(nv->value);
 	if (config->log_file == NULL)
 		return 1;
 	return 0;
 }
 
-static int num_logs_parser(const char *val, int line, 
+static int num_logs_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
-	const char *ptr = val;
+	const char *ptr = nv->value;
 	unsigned long i;
 
-	audit_msg(LOG_DEBUG, "num_logs_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "num_logs_parser called with: %s", nv->value);
 
 	/* check that all chars are numbers */
 	for (i=0; ptr[i]; i++) {
 		if (!isdigit(ptr[i])) {
 			audit_msg(LOG_ERR, 
 				"Value %s should only be numbers - line %d",
-				val, line);
+				nv->value, line);
 			return 1;
 		}
 	}
 
 	/* convert to unsigned long */
 	errno = 0;
-	i = strtoul(val, NULL, 10);
+	i = strtoul(nv->value, NULL, 10);
 	if (errno) {
 		audit_msg(LOG_ERR, 
 			"Error converting string to a number (%s) - line %d",
@@ -491,37 +530,37 @@ static int num_logs_parser(const char *val, int line,
 	return 0;
 }
 
-static int qos_parser(const char *val, int line,
+static int qos_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
 	int i;
 
-	audit_msg(LOG_DEBUG, "qos_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "qos_parser called with: %s", nv->value);
 	for (i=0; qos_options[i].name != NULL; i++) {
-		if (strcasecmp(val, qos_options[i].name) == 0) {
+		if (strcasecmp(nv->value, qos_options[i].name) == 0) {
 			config->qos = qos_options[i].option;
 			return 0;
 		}
 	}
-	audit_msg(LOG_ERR, "Option %s not found - line %d", val, line);
+	audit_msg(LOG_ERR, "Option %s not found - line %d", nv->value, line);
 	return 1;
 }
 
-
-static int dispatch_parser(const char *val, int line,struct daemon_conf *config)
+static int dispatch_parser(struct nv_pair *nv, int line,
+	struct daemon_conf *config)
 {
 	char *dir = NULL, *tdir, *base;
 	int fd;
 	struct stat buf;
 
-	audit_msg(LOG_DEBUG, "dispatch_parser called with: %s", val);
-	if (val == NULL) {
+	audit_msg(LOG_DEBUG, "dispatch_parser called with: %s", nv->value);
+	if (nv->value == NULL) {
 		config->dispatcher = NULL;
 		return 0;
 	}
 
 	/* split name into dir and basename. */
-	tdir = strdup(val);
+	tdir = strdup(nv->value);
 	if (tdir)
 		dir = dirname(tdir);
 	if (dir == NULL || strlen(dir) < 4) { //  '/var' is shortest dirname
@@ -533,7 +572,7 @@ static int dispatch_parser(const char *val, int line,struct daemon_conf *config)
 	}
 
 	free((void *)tdir);
-	base = basename((char *)val);
+	base = basename((char *)nv->value);
 	if (base == 0 || strlen(base) == 0) {
 		audit_msg(LOG_ERR, "The file name: %s is too short - line %d",
 			base, line);
@@ -541,60 +580,60 @@ static int dispatch_parser(const char *val, int line,struct daemon_conf *config)
 	}
 	/* if the file exists, see that its regular, owned by root,
 	 * and not world anything */
-	fd = open(val, O_RDONLY);
+	fd = open(nv->value, O_RDONLY);
 	if (fd < 0) {
-		audit_msg(LOG_ERR, "Unable to open %s (%s)", val,
+		audit_msg(LOG_ERR, "Unable to open %s (%s)", nv->value,
 			strerror(errno));
 		return 1;
 	}
 	if (fstat(fd, &buf) < 0) {
-		audit_msg(LOG_ERR, "Unable to stat %s (%s)", val,
+		audit_msg(LOG_ERR, "Unable to stat %s (%s)", nv->value,
 			strerror(errno));
 		close(fd);
 		return 1;
 	}
 	close(fd);
 	if (!S_ISREG(buf.st_mode)) {
-		audit_msg(LOG_ERR, "%s is not a regular file", val);
+		audit_msg(LOG_ERR, "%s is not a regular file", nv->value);
 		return 1;
 	}
 	if (buf.st_uid != 0) {
-		audit_msg(LOG_ERR, "%s is not owned by root", val);
+		audit_msg(LOG_ERR, "%s is not owned by root", nv->value);
 		return 1;
 	}
-	if ((buf.st_mode & (S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP)) !=
-			   (S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP)) {
-		audit_msg(LOG_ERR, "%s permissions should be 0750", val);
+	if ((buf.st_mode & (S_IRWXU|S_IRWXG|S_IWOTH)) !=
+			   (S_IRWXU|S_IRGRP|S_IXGRP)) {
+		audit_msg(LOG_ERR, "%s permissions should be 0750", nv->value);
 		return 1;
 	}
 	free((void *)config->dispatcher);
-	config->dispatcher = strdup(val);
+	config->dispatcher = strdup(nv->value);
 	if (config->dispatcher == NULL)
 		return 1;
 	return 0;
 }
 
-static int max_log_size_parser(const char *val, int line, 
+static int max_log_size_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
-	const char *ptr = val;
+	const char *ptr = nv->value;
 	unsigned long i;
 
-	audit_msg(LOG_DEBUG, "max_log_size_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "max_log_size_parser called with: %s", nv->value);
 
 	/* check that all chars are numbers */
 	for (i=0; ptr[i]; i++) {
 		if (!isdigit(ptr[i])) {
 			audit_msg(LOG_ERR, 
 				"Value %s should only be numbers - line %d",
-				val, line);
+				nv->value, line);
 			return 1;
 		}
 	}
 
 	/* convert to unsigned long */
 	errno = 0;
-	i = strtoul(val, NULL, 10);
+	i = strtoul(nv->value, NULL, 10);
 	if (errno) {
 		audit_msg(LOG_ERR, 
 			"Error converting string to a number (%s) - line %d",
@@ -605,73 +644,76 @@ static int max_log_size_parser(const char *val, int line,
 	return 0;
 }
 
-static int max_log_size_action_parser(const char *val, int line, 
+static int max_log_size_action_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
 	int i;
 
-	audit_msg(LOG_DEBUG, "max_log_size_action_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "max_log_size_action_parser called with: %s",
+		nv->value);
 	for (i=0; size_actions[i].name != NULL; i++) {
-		if (strcasecmp(val, size_actions[i].name) == 0) {
+		if (strcasecmp(nv->value, size_actions[i].name) == 0) {
 			config->max_log_size_action = size_actions[i].option;
 			return 0;
 		}
 	}
-	audit_msg(LOG_ERR, "Option %s not found - line %d", val, line);
+	audit_msg(LOG_ERR, "Option %s not found - line %d", nv->value, line);
 	return 1;
 }
 
-static int log_format_parser(const char *val, int line, 
+static int log_format_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
 	int i;
 
-	audit_msg(LOG_DEBUG, "log_format_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "log_format_parser called with: %s", nv->value);
 	for (i=0; log_formats[i].name != NULL; i++) {
-		if (strcasecmp(val, log_formats[i].name) == 0) {
+		if (strcasecmp(nv->value, log_formats[i].name) == 0) {
 			config->log_format = log_formats[i].option;
 			return 0;
 		}
 	}
-	audit_msg(LOG_ERR, "Option %s not found - line %d", val, line);
+	audit_msg(LOG_ERR, "Option %s not found - line %d", nv->value, line);
 	return 1;
 }
 
-static int flush_parser(const char *val, int line, struct daemon_conf *config)
+static int flush_parser(struct nv_pair *nv, int line,
+		struct daemon_conf *config)
 {
 	int i;
 
-	audit_msg(LOG_DEBUG, "flush_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "flush_parser called with: %s", nv->value);
 	for (i=0; flush_techniques[i].name != NULL; i++) {
-		if (strcasecmp(val, flush_techniques[i].name) == 0) {
+		if (strcasecmp(nv->value, flush_techniques[i].name) == 0) {
 			config->flush = flush_techniques[i].option;
 			return 0;
 		}
 	}
-	audit_msg(LOG_ERR, "Option %s not found - line %d", val, line);
+	audit_msg(LOG_ERR, "Option %s not found - line %d", nv->value, line);
 	return 1;
 }
 
-static int freq_parser(const char *val, int line, struct daemon_conf *config)
+static int freq_parser(struct nv_pair *nv, int line,
+		struct daemon_conf *config)
 {
-	const char *ptr = val;
+	const char *ptr = nv->value;
 	int i;
 
-	audit_msg(LOG_DEBUG, "freq_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "freq_parser called with: %s", nv->value);
 
 	/* check that all chars are numbers */
 	for (i=0; ptr[i]; i++) {
 		if (!isdigit(ptr[i])) {
 			audit_msg(LOG_ERR, 
 				"Value %s should only be numbers - line %d",
-				val, line);
+				nv->value, line);
 			return 1;
 		}
 	}
 
 	/* convert to unsigned int */
 	errno = 0;
-	i = strtoul(val, NULL, 10);
+	i = strtoul(nv->value, NULL, 10);
 	if (errno) {
 		audit_msg(LOG_ERR, 
 			"Error converting string to a number (%s) - line %d",
@@ -682,34 +724,34 @@ static int freq_parser(const char *val, int line, struct daemon_conf *config)
 	if (i > INT_MAX) {
 		audit_msg(LOG_ERR, 
 			"Error - converted number (%s) is too large - line %d",
-			val, line);
+			nv->value, line);
 		return 1;
 	}
 	config->freq = (unsigned int)i;
 	return 0;
 }
 
-static int space_left_parser(const char *val, int line, 
+static int space_left_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
-	const char *ptr = val;
+	const char *ptr = nv->value;
 	unsigned long i;
 
-	audit_msg(LOG_DEBUG, "space_left_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "space_left_parser called with: %s", nv->value);
 
 	/* check that all chars are numbers */
 	for (i=0; ptr[i]; i++) {
 		if (!isdigit(ptr[i])) {
 			audit_msg(LOG_ERR, 
 				"Value %s should only be numbers - line %d",
-				val, line);
+				nv->value, line);
 			return 1;
 		}
 	}
 
 	/* convert to unsigned long */
 	errno = 0;
-	i = strtoul(val, NULL, 10);
+	i = strtoul(nv->value, NULL, 10);
 	if (errno) {
 		audit_msg(LOG_ERR, 
 			"Error converting string to a number (%s) - line %d",
@@ -720,14 +762,44 @@ static int space_left_parser(const char *val, int line,
 	return 0;
 }
 
-static int space_action_parser(const char *val, int line, 
+static int check_exe_name(const char *val)
+{
+	struct stat buf;
+
+	if (*val != '/') {
+		audit_msg(LOG_ERR, "Absolute path needed for %s", val);
+		return -1;
+	}
+
+	if (stat(val, &buf) < 0) {
+		audit_msg(LOG_ERR, "Unable to stat %s (%s)", val,
+			strerror(errno));
+		return -1;
+	}
+	if (!S_ISREG(buf.st_mode)) {
+		audit_msg(LOG_ERR, "%s is not a regular file", val);
+		return -1;
+	}
+	if (buf.st_uid != 0) {
+		audit_msg(LOG_ERR, "%s is not owned by root", val);
+		return -1;
+	}
+	if ((buf.st_mode & (S_IRWXU|S_IRWXG|S_IWOTH)) !=
+			   (S_IRWXU|S_IRGRP|S_IXGRP)) {
+		audit_msg(LOG_ERR, "%s permissions should be 0750", val);
+		return -1;
+	}
+	return 0;
+}
+
+static int space_action_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
 	int i;
 
-	audit_msg(LOG_DEBUG, "space_action_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "space_action_parser called with: %s", nv->value);
 	for (i=0; failure_actions[i].name != NULL; i++) {
-		if (strcasecmp(val, failure_actions[i].name) == 0) {
+		if (strcasecmp(nv->value, failure_actions[i].name) == 0) {
 			if (failure_actions[i].option == FA_EMAIL) {
 				if (access(email_command, X_OK)) {
 					audit_msg(LOG_ERR,
@@ -737,9 +809,18 @@ static int space_action_parser(const char *val, int line,
 			}
 			config->space_left_action = failure_actions[i].option;
 			return 0;
+		} else if (i == FA_EXEC) {
+			if (strncasecmp(failure_actions[i].name,
+						 nv->value, 4) == 0){
+				if (check_exe_name(nv->option))
+					return 1;
+				config->space_left_exe = strdup(nv->option);
+				config->space_left_action = FA_EXEC;
+				return 0;
+			}
 		}
 	}
-	audit_msg(LOG_ERR, "Option %s not found - line %d", val, line);
+	audit_msg(LOG_ERR, "Option %s not found - line %d", nv->value, line);
 	return 1;
 }
 
@@ -784,13 +865,14 @@ int validate_email(const char *acct)
 	return 0;
 }
 
-static int action_mail_acct_parser(const char *val, int line, 
+static int action_mail_acct_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
 	char *tmail;
 	
-	audit_msg(LOG_DEBUG, "action_mail_acct_parser called with: %s", val);
-	tmail = strdup(val);
+	audit_msg(LOG_DEBUG, "action_mail_acct_parser called with: %s",
+							nv->value);
+	tmail = strdup(nv->value);
 	if (tmail == NULL)
 		return 1;
 
@@ -806,27 +888,28 @@ static int action_mail_acct_parser(const char *val, int line,
 	return 0;
 }
 
-static int admin_space_left_parser(const char *val, int line, 
+static int admin_space_left_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
-	const char *ptr = val;
+	const char *ptr = nv->value;
 	unsigned long i;
 
-	audit_msg(LOG_DEBUG, "admin_space_left_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "admin_space_left_parser called with: %s",
+							nv->value);
 
 	/* check that all chars are numbers */
 	for (i=0; ptr[i]; i++) {
 		if (!isdigit(ptr[i])) {
 			audit_msg(LOG_ERR, 
 				"Value %s should only be numbers - line %d",
-				val, line);
+				nv->value, line);
 			return 1;
 		}
 	}
 
 	/* convert to unsigned long */
 	errno = 0;
-	i = strtoul(val, NULL, 10);
+	i = strtoul(nv->value, NULL, 10);
 	if (errno) {
 		audit_msg(LOG_ERR, 
 			"Error converting string to a number (%s) - line %d",
@@ -837,14 +920,15 @@ static int admin_space_left_parser(const char *val, int line,
 	return 0;
 }
 
-static int admin_space_left_action_parser(const char *val, int line, 
+static int admin_space_left_action_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
 	int i;
 
-	audit_msg(LOG_DEBUG, "admin_space_left_action_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "admin_space_left_action_parser called with: %s",
+								nv->value);
 	for (i=0; failure_actions[i].name != NULL; i++) {
-		if (strcasecmp(val, failure_actions[i].name) == 0) {
+		if (strcasecmp(nv->value, failure_actions[i].name) == 0) {
 			if (failure_actions[i].option == FA_EMAIL) {
 				if (access(email_command, X_OK)) {
 					audit_msg(LOG_ERR,
@@ -855,20 +939,31 @@ static int admin_space_left_action_parser(const char *val, int line,
 			config->admin_space_left_action =
 						failure_actions[i].option;
 			return 0;
+		} else if (i == FA_EXEC) {
+			if (strncasecmp(failure_actions[i].name,
+							nv->value, 4) == 0){
+				if (check_exe_name(nv->option))
+					return 1;
+				config->admin_space_left_exe = 
+							strdup(nv->option);
+				config->admin_space_left_action = FA_EXEC;
+				return 0;
+			}
 		}
 	}
-	audit_msg(LOG_ERR, "Option %s not found - line %d", val, line);
+	audit_msg(LOG_ERR, "Option %s not found - line %d", nv->value, line);
 	return 1;
 }
 
-static int disk_full_action_parser(const char *val, int line, 
+static int disk_full_action_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
 	int i;
 
-	audit_msg(LOG_DEBUG, "disk_full_action_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "disk_full_action_parser called with: %s",
+								nv->value);
 	for (i=0; failure_actions[i].name != NULL; i++) {
-		if (strcasecmp(val, failure_actions[i].name) == 0) {
+		if (strcasecmp(nv->value, failure_actions[i].name) == 0) {
 			if (failure_actions[i].option != FA_EMAIL) {
 				config->disk_full_action =
 						failure_actions[i].option;
@@ -876,23 +971,33 @@ static int disk_full_action_parser(const char *val, int line,
 			} else {
 				audit_msg(LOG_ERR, 
 			"Illegal option %s for disk_full_action - line %d",
-					 val, line);
+					nv->value, line);
 				return 1;
 			}
+		} else if (i == FA_EXEC) {
+			if (strncasecmp(failure_actions[i].name, 
+							nv->value, 4) == 0){
+				if (check_exe_name(nv->option))
+					return 1;
+				config->disk_full_exe = strdup(nv->option);
+				config->disk_full_action = FA_EXEC;
+				return 0;
+			}
 		}
 	}
-	audit_msg(LOG_ERR, "Option %s not found - line %d", val, line);
+	audit_msg(LOG_ERR, "Option %s not found - line %d", nv->value, line);
 	return 1;
 }
 
-static int disk_error_action_parser(const char *val, int line, 
+static int disk_error_action_parser(struct nv_pair *nv, int line, 
 		struct daemon_conf *config)
 {
 	int i;
 
-	audit_msg(LOG_DEBUG, "disk_error_action_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "disk_error_action_parser called with: %s",
+								nv->value);
 	for (i=0; failure_actions[i].name != NULL; i++) {
-		if (strcasecmp(val, failure_actions[i].name) == 0) {
+		if (strcasecmp(nv->value, failure_actions[i].name) == 0) {
 			if (failure_actions[i].option != FA_EMAIL) {
 				config->disk_error_action = 
 						failure_actions[i].option;
@@ -900,35 +1005,46 @@ static int disk_error_action_parser(const char *val, int line,
 			} else {
 				audit_msg(LOG_ERR, 
 			"Illegal option %s for disk_error_action - line %d",
-					 val, line);
+					nv->value, line);
 				return 1;
 			}
+		} else if (i == FA_EXEC) {
+			if (strncasecmp(failure_actions[i].name,
+							nv->value, 4) == 0){
+				if (check_exe_name(nv->option))
+					return 1;
+				config->disk_error_exe = strdup(nv->option);
+				config->disk_error_action = FA_EXEC;
+				return 0;
+			}
 		}
 	}
-	audit_msg(LOG_ERR, "Option %s not found - line %d", val, line);
+	audit_msg(LOG_ERR, "Option %s not found - line %d", nv->value, line);
 	return 1;
 }
 
-static int priority_boost_parser(const char *val, int line, struct daemon_conf *config)
+static int priority_boost_parser(struct nv_pair *nv, int line,
+	struct daemon_conf *config)
 {
-	const char *ptr = val;
+	const char *ptr = nv->value;
 	int i;
 
-	audit_msg(LOG_DEBUG, "priority_boost_parser called with: %s", val);
+	audit_msg(LOG_DEBUG, "priority_boost_parser called with: %s",
+								nv->value);
 
 	/* check that all chars are numbers */
 	for (i=0; ptr[i]; i++) {
 		if (!isdigit(ptr[i])) {
 			audit_msg(LOG_ERR, 
 				"Value %s should only be numbers - line %d",
-				val, line);
+				nv->value, line);
 			return 1;
 		}
 	}
 
 	/* convert to unsigned int */
 	errno = 0;
-	i = strtoul(val, NULL, 10);
+	i = strtoul(nv->value, NULL, 10);
 	if (errno) {
 		audit_msg(LOG_ERR, 
 			"Error converting string to a number (%s) - line %d",
@@ -939,7 +1055,7 @@ static int priority_boost_parser(const char *val, int line, struct daemon_conf *
 	if (i > INT_MAX) {
 		audit_msg(LOG_ERR, 
 			"Error - converted number (%s) is too large - line %d",
-			val, line);
+			nv->value, line);
 		return 1;
 	}
 	config->priority_boost = (unsigned int)i;
@@ -998,8 +1114,13 @@ int create_log_file(const char *val)
 
 void free_config(struct daemon_conf *config)
 {
-	free((void*)config->log_file);
-	free((void*)config->dispatcher);
+	free((void *)config->sender_ctx);
+	free((void *)config->log_file);
+	free((void *)config->dispatcher);
 	free((void *)config->action_mail_acct);
+	free((void *)config->space_left_exe);
+        free((void *)config->admin_space_left_exe);
+        free((void *)config->disk_full_exe);
+        free((void *)config->disk_error_exe);
 }
 
