@@ -57,16 +57,16 @@ static void handle_event(struct auditd_consumer_data *data);
 static void write_to_log(const char *buf, struct auditd_consumer_data *data);
 static void check_log_file_size(int lfd, struct auditd_consumer_data *data);
 static void check_space_left(int lfd, struct daemon_conf *config);
-static void do_space_left_action(failure_action_t action, const char *mail_acct,
-	int admin);
+static void do_space_left_action(struct daemon_conf *config, int admin);
 static void do_disk_full_action(struct daemon_conf *config);
 static void do_disk_error_action(const char *func, struct daemon_conf *config);
 static void rotate_logs_now(struct auditd_consumer_data *data);
 static void rotate_logs(struct auditd_consumer_data *data, 
 		unsigned int num_logs);
 static void shift_logs(struct auditd_consumer_data *data);
-static int open_audit_log(struct auditd_consumer_data *data);
+static int  open_audit_log(struct auditd_consumer_data *data);
 static void change_runlevel(const char *level);
+static void safe_exec(const char *exe);
 static char *format_raw(const struct audit_reply *rep);
 static void reconfigure(struct auditd_consumer_data *data);
 
@@ -261,15 +261,16 @@ static void handle_event(struct auditd_consumer_data *data)
 					     do_disk_full_action(data->config);
 		        	        } else
 					     //EIO is only likely failure mode
-					     do_disk_error_action("flush",
+					     do_disk_error_action("flush", 
 						data->config);
 				}
 
 				/* EIO is only likely failure mode */
-				if ((data->config->daemonize == D_BACKGROUND)&&
-						(fsync(data->log_fd) != 0))
+				if ((data->config->daemonize == D_BACKGROUND)&& 
+						(fsync(data->log_fd) != 0)) {
 				     do_disk_error_action("fsync",
 					data->config);
+				}
 			}
 		}
 	}
@@ -376,19 +377,15 @@ static void check_space_left(int lfd, struct daemon_conf *config)
 		        blocks = config->space_left * (MEGABYTE/block_size);
         		if (blocks > buf.f_bavail) {
 				if (fs_space_warning == 0) {
+					do_space_left_action(config, 0);
 					fs_space_warning = 1;
-					do_space_left_action(
-						config->space_left_action,
-						config->action_mail_acct, 0);
 				}
 			}
 		        blocks=config->admin_space_left * (MEGABYTE/block_size);
         		if (blocks > buf.f_bavail) {
 				if (fs_admin_space_warning == 0) {
+					do_space_left_action(config, 1);
 					fs_admin_space_warning = 1;
-					do_space_left_action(
-					    config->admin_space_left_action,
-					    config->action_mail_acct, 1);
 				}
 			}
 	
@@ -400,9 +397,14 @@ static void check_space_left(int lfd, struct daemon_conf *config)
 
 extern int sendmail(const char *subject, const char *content, 
 	const char *mail_acct);
-static void do_space_left_action(failure_action_t action, 
-	const char *mail_acct, int admin)
+static void do_space_left_action(struct daemon_conf *config, int admin)
 {
+	int action;
+	if (admin)
+		action = config->admin_space_left_action;
+	else
+		action = config->space_left_action;
+
 	switch (action)
 	{
 		case FA_IGNORE:
@@ -415,17 +417,23 @@ static void do_space_left_action(failure_action_t action,
 			if (admin == 0) {
 				sendmail("Audit Disk Space Alert", 
 				"The audit daemon is low on disk space for logging! Please take action\nto ensure no loss of service.",
-					mail_acct);
+					config->action_mail_acct);
 				audit_msg(LOG_ALERT, 
 			    "Audit daemon is low on disk space for logging");
 			} else {
 				sendmail("Audit Admin Space Alert", 
 				"The audit daemon is very low on disk space for logging! Immediate action\nis required to ensure no loss of service.",
-					mail_acct);
+					config->action_mail_acct);
 				audit_msg(LOG_ALERT, 
 			  "Audit daemon is very low on disk space for logging");
 			}
 			break;
+		case FA_EXEC:
+			if (admin)
+				safe_exec(config->admin_space_left_exe);
+			else
+				safe_exec(config->space_left_exe);
+			break;
 		case FA_SUSPEND:
 			audit_msg(LOG_ALERT,
 			    "Audit daemon is suspending logging due to low disk space.");
@@ -456,6 +464,9 @@ static void do_disk_full_action(struct daemon_conf *config)
 		case FA_IGNORE:
 		case FA_SYSLOG: /* Message is syslogged above */
 			break;
+		case FA_EXEC:
+			safe_exec(config->disk_full_exe);
+			break;
 		case FA_SUSPEND:
 			audit_msg(LOG_ALERT,
 			    "Audit daemon is suspending logging due to no space left on logging partition.");
@@ -491,6 +502,9 @@ static void do_disk_error_action(const char * func, struct daemon_conf *config)
 		case FA_IGNORE:
 		case FA_SYSLOG: /* Message is syslogged above */
 			break;
+		case FA_EXEC:
+			safe_exec(config->disk_error_exe);
+			break;
 		case FA_SUSPEND:
 			audit_msg(LOG_ALERT,
 			    "Audit daemon is suspending logging due to previously mentioned write error");
@@ -645,6 +659,7 @@ static void shift_logs(struct auditd_consumer_data *data)
 	}
 	last_log = num_logs;
 	rotate_logs(data, num_logs+1);
+	free(name);
 }
 
 /*
@@ -760,6 +775,27 @@ static void change_runlevel(const char *level)
 	exit(1);
 }
 
+static void safe_exec(const char *exe)
+{
+	char *argv[2];
+	int pid;
+
+	pid = fork();
+	if (pid < 0) {
+		audit_msg(LOG_ALERT, 
+			"Audit daemon failed to fork doing safe_exec");
+		return;
+	}
+	if (pid)	/* Parent */
+		return;
+	/* Child */
+	argv[0] = (char *)exe;
+	argv[1] = NULL;
+	execve(exe, argv, NULL);
+	audit_msg(LOG_ALERT, "Audit daemon failed to exec %s", exe);
+	exit(1);
+}
+
 /*
 * This function will take an audit structure and return a
 * text buffer that's unformatted for writing to disk. If there
@@ -809,22 +845,26 @@ static void reconfigure(struct auditd_consumer_data *data)
 	struct daemon_conf *oconf = data->config;
 	uid_t uid = nconf->sender_uid;
 	pid_t pid = nconf->sender_pid;
+	const char *ctx = nconf->sender_ctx;
 	struct timeval tv;
-	char txt[128];
+	char txt[MAX_AUDIT_MESSAGE_LENGTH];
 	char date[40];
 	unsigned int seq_num;
 	int need_size_check = 0, need_reopen = 0, need_space_check = 0;
 
-	snprintf(txt, sizeof(txt), "config change requested by pid=%d auid=%u",
-		pid, uid);
+	snprintf(txt, sizeof(txt),
+		"config change requested by pid=%d auid=%u subj=%s",
+		pid, uid, ctx);
 	audit_msg(LOG_NOTICE, txt);
 
 	/* Do the reconfiguring. These are done in a specific
 	 * order from least invasive to most invasive. We will
 	 * start with general system parameters. */
 
-	// start with disk full action.
+	// start with disk error action.
 	oconf->disk_error_action = nconf->disk_error_action;
+	free(oconf->disk_error_exe);
+	oconf->disk_error_exe = nconf->disk_error_exe;
 
 	// numlogs is next
 	oconf->num_logs = nconf->num_logs;
@@ -870,13 +910,13 @@ static void reconfigure(struct auditd_consumer_data *data)
 				// Likely errors: Socketpairs or exec perms
 				do_disk_error_action("reconfig", data->config);
 			}
-		}
+		} 
 		// have one, but none after this
 		else if (nconf->dispatcher == NULL) {
 			shutdown_dispatcher();
 			free((char *)oconf->dispatcher);
 			oconf->dispatcher = NULL;
-		}
+		} 
 		// they are different apps
 		else if (strcmp(oconf->dispatcher, nconf->dispatcher)) {
 			shutdown_dispatcher();
@@ -966,6 +1006,18 @@ static void reconfigure(struct auditd_consumer_data *data)
 		need_space_check = 1;
 	}
 
+	// space left exe
+	if (oconf->space_left_exe || nconf->space_left_exe) {
+		if (nconf->space_left_exe == NULL)
+			; /* do nothing if new one is blank */
+		else if (oconf->space_left_exe == NULL && nconf->space_left_exe)
+			need_space_check = 1;
+		else if (strcmp(oconf->space_left_exe, nconf->space_left_exe))
+			need_space_check = 1;
+		free(oconf->space_left_exe);
+		oconf->space_left_exe = nconf->space_left_exe;
+	}
+
 	// admin space left
 	if (oconf->admin_space_left != nconf->admin_space_left) {
 		oconf->admin_space_left = nconf->admin_space_left;
@@ -978,12 +1030,37 @@ static void reconfigure(struct auditd_consumer_data *data)
 		need_space_check = 1;
 	}
 
+	// admin space left exe
+	if (oconf->admin_space_left_exe || nconf->admin_space_left_exe) {
+		if (nconf->admin_space_left_exe == NULL)
+			; /* do nothing if new one is blank */
+		else if (oconf->admin_space_left_exe == NULL &&
+					 nconf->admin_space_left_exe)
+			need_space_check = 1;
+		else if (strcmp(oconf->admin_space_left_exe,
+					nconf->admin_space_left_exe))
+			need_space_check = 1;
+		free(oconf->admin_space_left_exe);
+		oconf->admin_space_left_exe = nconf->admin_space_left_exe;
+	}
 	// disk full action
 	if (oconf->disk_full_action != nconf->disk_full_action) {
 		oconf->disk_full_action = nconf->disk_full_action;
 		need_space_check = 1;
 	}
 
+	// disk full exe
+	if (oconf->disk_full_exe || nconf->disk_full_exe) {
+		if (nconf->disk_full_exe == NULL)
+			; /* do nothing if new one is blank */
+		else if (oconf->disk_full_exe == NULL && nconf->disk_full_exe)
+			need_space_check = 1;
+		else if (strcmp(oconf->disk_full_exe, nconf->disk_full_exe))
+			need_space_check = 1;
+		free(oconf->disk_full_exe);
+		oconf->disk_full_exe = nconf->disk_full_exe;
+	}
+
 	if (need_space_check) {
 		/* note save suspended flag, then do space_left. If suspended
 		 * is still 0, then copy saved suspended back. This avoids
@@ -1012,8 +1089,8 @@ static void reconfigure(struct auditd_consumer_data *data)
         }
 
 	data->head->reply.len = snprintf(txt, sizeof(txt), 
-		"%s config changed, auid=%u pid=%d res=success", date, 
-		uid, pid);
+		"%s config changed, auid=%u pid=%d subj=%s res=success", date, 
+		uid, pid, ctx );
 	audit_msg(LOG_NOTICE, txt);
 	data->head->reply.type = AUDIT_DAEMON_CONFIG;
 	data->head->reply.message = strdup(txt);
@@ -1022,5 +1099,6 @@ static void reconfigure(struct auditd_consumer_data *data)
 		audit_msg(LOG_ERR, "Cannot allocate config message");
 		// FIXME: Should call some error handler
 	}
+	free((char *)ctx);
 }
 
