@@ -1,5 +1,5 @@
 /* auditd.c -- 
- * Copyright 2004-2006 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2004-06 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,8 @@
 #include "auditd-dispatch.h"
 #include "private.h"
 
-#define DMSG_SIZE 116
+#define DEFAULT_BUF_SZ	384
+#define DMSG_SIZE (DEFAULT_BUF_SZ + 48) 
 #define SUCCESS 0
 #define FAILURE 1
 
@@ -57,6 +58,7 @@ static int fd = -1;
 static struct daemon_conf config;
 static const char *pidfile = "/var/run/auditd.pid";
 static int init_pipe[2];
+static int do_fork = 1;
 
 /* Local function prototypes */
 static void close_down(void);
@@ -69,7 +71,7 @@ static int get_reply(int fd, struct audit_reply *rep, int seq);
  */
 static void usage(void)
 {
-	puts("Usage: auditd [ -f ]");
+	puts("Usage: auditd [ -f -l -n ]");
 	exit(2);
 }
 
@@ -212,16 +214,21 @@ static int become_daemon(void)
 	pid_t pid;
 	int status;
 
-	if (pipe(init_pipe) || fcntl(init_pipe[0], F_SETFD, FD_CLOEXEC) ||
+	if (do_fork) {
+		if (pipe(init_pipe) || 
+				fcntl(init_pipe[0], F_SETFD, FD_CLOEXEC) ||
 				fcntl(init_pipe[0], F_SETFD, FD_CLOEXEC))
-		return -1;
+			return -1;
+		pid = fork();
+	} else
+		pid = 0;
 
-	pid = fork();
 	switch (pid)
 	{
 		case 0:
 			/* No longer need this...   */
-			close(init_pipe[0]);
+			if (do_fork) 
+				close(init_pipe[0]);
 
 			/* Open stdin,out,err to /dev/null */
 			fd = open("/dev/null", O_RDWR);
@@ -266,7 +273,7 @@ static void tell_parent(int status)
 {
 	int rc;
 
-	if (config.daemonize != D_BACKGROUND)
+	if (config.daemonize != D_BACKGROUND || do_fork == 0)
 		return;
 	do {
 		rc = write(init_pipe[1], &status, sizeof(status));
@@ -279,21 +286,28 @@ int main(int argc, char *argv[])
 	fd_set read_mask;
 	struct auditd_reply_list *rep = NULL;
 	struct rlimit limit;
-	int hup_info_requested = 0;
+	int hup_info_requested = 0, usr1_info_requested = 0;
 	int i;
 
 	/* Get params && set mode */
-	if (argc > 2)
-		usage();
-	if (argc == 2) {
-		if (strcmp(argv[1], "-f") == 0) {
-			config.daemonize = D_FOREGROUND;
-			set_aumessage_mode(MSG_STDERR, DBG_YES);
-		} else
-			usage();
+	config.daemonize = D_BACKGROUND;
+	if (argc > 1) {
+		for (i=1; i<argc; i++) {
+			if (strcmp(argv[i], "-f") == 0) 
+				config.daemonize = D_FOREGROUND;
+			else if (strcmp(argv[i], "-l") == 0)
+				set_allow_links(1);
+			else if (strcmp(argv[i], "-n") == 0)
+				do_fork = 0;
+			else
+				usage();
+		}
 	}
+
+	// Make paramemters take effect
+	if (config.daemonize == D_FOREGROUND)
+		set_aumessage_mode(MSG_STDERR, DBG_YES);
 	else {
-		config.daemonize = D_BACKGROUND;
 		set_aumessage_mode(MSG_SYSLOG, DBG_NO);
 		(void) umask( umask( 077 ) | 022 );
 	}
@@ -384,13 +398,15 @@ int main(int argc, char *argv[])
 
 	/* Write message to log that we are alive */
 	{
-		char start[80];
+		char start[DEFAULT_BUF_SZ];
 		const char *fmt = audit_lookup_format((int)config.log_format);
 		if (fmt == NULL)
 			fmt = "UNKNOWN";
+//FIXME add SUBJ
 		snprintf(start, sizeof(start), 
-		    "auditd start, ver=%s, format=%s, auid=%u res=success",
-		     VERSION, fmt, audit_getloginuid());
+		    "auditd start, ver=%s, format=%s, "
+		    "auid=%u pid=%d res=success",
+		     VERSION, fmt, audit_getloginuid(), getpid());
 		if (send_audit_event(AUDIT_DAEMON_START, start)) {
         		audit_msg(LOG_ERR, "Cannot send start message");
 			if (pidfile)
@@ -409,11 +425,12 @@ int main(int argc, char *argv[])
 
 	/* Tell the kernel we are alive */
 	if (audit_set_pid(fd, getpid(), WAIT_YES) < 0) {
-		char emsg[64];
+		char emsg[DEFAULT_BUF_SZ];
 		snprintf(emsg, sizeof(emsg),
-			"auditd error halt, auid=%u res=failed",
-			audit_getloginuid());
+			"auditd error halt, auid=%u pid=%d res=failed",
+			audit_getloginuid(), getpid());
 		stop = 1;
+//FIXME add subj
 		send_audit_event(AUDIT_DAEMON_ABORT, emsg);
 		audit_msg(LOG_ERR, "Unable to set audit pid, exiting");
 		close_down();
@@ -429,11 +446,12 @@ int main(int argc, char *argv[])
 
 	/* Enable auditing just in case it was off */
 	if (audit_set_enabled(fd, 1) < 0) {
-		char emsg[64];
+		char emsg[DEFAULT_BUF_SZ];
 		snprintf(emsg, sizeof(emsg),
-			"auditd error halt, auid=%u res=failed",
-			audit_getloginuid());
+			"auditd error halt, auid=%u pid=%d res=failed",
+			audit_getloginuid(), getpid());
 		stop = 1;
+//FIXME add subj
 		send_audit_event(AUDIT_DAEMON_ABORT, emsg);
 		audit_msg(LOG_ERR, "Unable to enable auditing, exiting");
 		close_down();
@@ -446,7 +464,8 @@ int main(int argc, char *argv[])
 		VERSION);
 
 	/* Parent should be gone by now...   */
-	close(init_pipe[1]);
+	if (do_fork)
+		close(init_pipe[1]);
 
 	for (;;) {
 		struct timeval tv;
@@ -454,11 +473,12 @@ int main(int argc, char *argv[])
        
 		if (rep == NULL) { 
 			if ((rep = malloc(sizeof(*rep))) == NULL) {
-				char emsg[64];
+				char emsg[DEFAULT_BUF_SZ];
 				snprintf(emsg, sizeof(emsg), 
-				    "auditd error halt, auid=%u res=failed",
-				    audit_getloginuid());
+			    "auditd error halt, auid=%u pid=%d res=failed",
+				    audit_getloginuid(), getpid());
 				stop = 1;
+//FIXME add subj
 				send_audit_event(AUDIT_DAEMON_ABORT, emsg);
 				audit_msg(LOG_ERR, 
 					"Cannot allocate audit reply, exiting");
@@ -490,6 +510,7 @@ int main(int argc, char *argv[])
 					case NLMSG_ERROR:
 					case AUDIT_GET: /* Or these */
 					case AUDIT_LIST:
+					case AUDIT_LIST_RULES:
 					case AUDIT_FIRST_DAEMON...AUDIT_LAST_DAEMON:
 						break;
 					case AUDIT_SIGNAL_INFO:
@@ -500,16 +521,35 @@ int main(int argc, char *argv[])
 							send_audit_event(
 							AUDIT_DAEMON_CONFIG, 
 				"auditd error getting hup info - no change,"
-				" sending auid=? pid=? res=failed");
-						    } else
-							rep = 0;
+				" sending auid=? pid=? subj=? res=failed");
+						    } else {
+							free(rep);
+							rep = NULL;
+						    }
 						    hup_info_requested = 0;
+						} else if(usr1_info_requested){
+							char usr1[MAX_AUDIT_MESSAGE_LENGTH];
+							if (rep->reply.len == 24) {
+								snprintf(usr1, 
+								sizeof(usr1),
+					"auditd sending auid=? pid=? subj=?");
+							} else {
+								snprintf(usr1, 
+								sizeof(usr1),
+					"auditd sending auid=%u pid=%d subj=%s",
+						rep->reply.signal_info->uid, 
+						rep->reply.signal_info->pid,
+						rep->reply.signal_info->ctx);
+							}
+							send_audit_event(
+							AUDIT_DAEMON_ROTATE, 
+							usr1);
 						}
 						break;
 					default:
 						dispatch_event(&rep->reply);
 						enqueue_event(rep);
-						rep = 0;
+						rep = NULL;
 						break;
 				}
 			}
@@ -520,15 +560,19 @@ int main(int argc, char *argv[])
 			rc = audit_request_signal_info(fd);
 			if (rc < 0)
 			    send_audit_event(AUDIT_DAEMON_CONFIG, 
-				"auditd error getting hup info - no change, sending auid=? pid=? res=failed");
+				"auditd error getting hup info - no change, sending auid=? pid=? subj=? res=failed");
 			else
 				hup_info_requested = 1;
 		}
 		if (rot) {
+			int rc;
 			rot = 0;
-			// FIXME: need to get sending uid & pid
-			send_audit_event(AUDIT_DAEMON_ROTATE, 
-				"auditd sending auid=? pid=? res=success");
+			rc = audit_request_signal_info(fd);
+			if (rc < 0)
+			    send_audit_event(AUDIT_DAEMON_ROTATE, 
+				"auditd error getting usr1 info - no change, sending auid=? pid=? subj=? res=failed");
+			else
+				usr1_info_requested = 1;
 		}
 		if (stop) {
 			/* Write message to log that we are going down */
@@ -540,17 +584,18 @@ int main(int argc, char *argv[])
 
 				rc = get_reply(fd, &trep, rc);
 				if (rc > 0) {
-					char txt[68];
+					char txt[MAX_AUDIT_MESSAGE_LENGTH];
 					snprintf(txt, sizeof(txt),
-	    	   "auditd normal halt, sending auid=%u pid=%d res=success",
+    	   "auditd normal halt, sending auid=%u pid=%d subj=%s res=success",
 						trep.signal_info->uid,
-						trep.signal_info->pid); 
+						trep.signal_info->pid, 
+						trep.signal_info->ctx); 
 					send_audit_event(AUDIT_DAEMON_END, txt);
 				} 
 			} 
 			if (rc <= 0)
 			    send_audit_event(AUDIT_DAEMON_END, 
-		"auditd normal halt, sending auid=? pid=? res=success");
+		"auditd normal halt, sending auid=? pid=? subj=? res=success");
 			free(rep);
 			shutdown_dispatcher();
 			break;
