@@ -29,6 +29,7 @@
 #include "aureport-options.h"
 #include "ausearch-parse.h"
 #include "ausearch-string.h"
+#include "ausearch-lookup.h"
 #include "aureport-scan.h"
 
 static void do_summary_total(llist *l);
@@ -41,13 +42,16 @@ summary_data sd;
 void reset_counters(void)
 {
 	sd.changes = 0UL;
+	sd.crypto = 0UL;
 	sd.acct_changes = 0UL;
 	sd.good_logins = 0UL;
 	sd.bad_logins = 0UL;
+	sd.good_auth = 0UL;
+	sd.bad_auth = 0UL;
 	sd.events = 0UL;
 	sd.avcs = 0UL;
+	sd.mac = 0UL;
 	sd.failed_syscalls = 0UL;
-	sd.fs_watches = 0UL;
 	sd.anomalies = 0UL;
 	sd.responses = 0UL;
 	slist_create(&sd.users);
@@ -59,20 +63,25 @@ void reset_counters(void)
 	ilist_create(&sd.pids);
 	ilist_create(&sd.sys_list);
 	ilist_create(&sd.anom_list);
+	ilist_create(&sd.mac_list);
 	ilist_create(&sd.resp_list);
+	ilist_create(&sd.crypto_list);
 }
 
 /* This function inits the counters */
 void destroy_counters(void)
 {
 	sd.changes = 0UL;
+	sd.crypto = 0UL;
 	sd.acct_changes = 0UL;
 	sd.good_logins = 0UL;
 	sd.bad_logins = 0UL;
+	sd.good_auth = 0UL;
+	sd.bad_auth = 0UL;
 	sd.events = 0UL;
 	sd.avcs = 0UL;
+	sd.mac = 0UL;
 	sd.failed_syscalls = 0UL;
-	sd.fs_watches = 0UL;
 	sd.anomalies = 0UL;
 	sd.responses = 0UL;
 	slist_clear(&sd.users);
@@ -84,9 +93,90 @@ void destroy_counters(void)
 	ilist_clear(&sd.pids);
 	ilist_clear(&sd.sys_list);
 	ilist_clear(&sd.anom_list);
+	ilist_create(&sd.mac_list);
 	ilist_clear(&sd.resp_list);
+	ilist_create(&sd.crypto_list);
 }
 
+/* This function will return 0 on no match and 1 on match */
+int classify_success(const llist *l)
+{
+//printf("%d,succ=%d:%d\n", l->head->type, event_failed, l->s.success);
+	// If match only failed... 
+	if (event_failed == F_FAILED)
+		return l->s.success == S_FAILED ? 1 : 0;
+	// If match only success... 
+	if (event_failed == F_SUCCESS)
+		return l->s.success == S_SUCCESS ? 1 : 0;
+	// Otherwise...we don't care so pretend it matched
+	return 1;
+}
+
+/* This function will return 0 on no match and 1 on match */
+int classify_conf(const llist *l)
+{
+	int rc = 1;
+
+	switch (l->head->type)
+	{
+		case AUDIT_CONFIG_CHANGE:
+		case AUDIT_USYS_CONFIG:
+			break;
+		case AUDIT_ADD_USER:
+			if (event_conf_act == C_DEL)
+				rc = 0;
+			break;
+		case AUDIT_DEL_USER:
+			if (event_conf_act == C_ADD)
+				rc = 0;
+			break;
+		case AUDIT_ADD_GROUP:
+			if (event_conf_act == C_DEL)
+				rc = 0;
+			break;
+		case AUDIT_DEL_GROUP:
+			if (event_conf_act == C_ADD)
+				rc = 0;
+			break;
+		case AUDIT_MAC_CIPSOV4_ADD:
+			if (event_conf_act == C_DEL)
+				rc = 0;
+			break;
+		case AUDIT_MAC_CIPSOV4_DEL:
+			if (event_conf_act == C_ADD)
+				rc = 0;
+			break;
+		case AUDIT_MAC_MAP_ADD:
+			if (event_conf_act == C_DEL)
+				rc = 0;
+			break;
+		case AUDIT_MAC_MAP_DEL:
+			if (event_conf_act == C_ADD)
+				rc = 0;
+			break;
+		case AUDIT_MAC_IPSEC_ADDSA:
+			if (event_conf_act == C_DEL)
+				rc = 0;
+			break;
+		case AUDIT_MAC_IPSEC_DELSA:
+			if (event_conf_act == C_ADD)
+				rc = 0;
+			break;
+		case AUDIT_MAC_IPSEC_ADDSPD:
+			if (event_conf_act == C_DEL)
+				rc = 0;
+			break;
+		case AUDIT_MAC_IPSEC_DELSPD:
+			if (event_conf_act == C_ADD)
+				rc = 0;
+			break;
+		default:
+			break;
+	}
+//printf("conf=%d:%d\n", l->head->type, rc);
+	return rc;
+}
+ 
 /*
  * This function performs that matching of search params with the record.
  * It returns 1 on a match, and 0 if no match.
@@ -96,20 +186,11 @@ int scan(llist *l)
 	// Are we within time range?
 	if (start_time == 0 || l->e.sec >= start_time) {
 		if (end_time == 0 || l->e.sec <= end_time) {
-			if (event_id == -1 || event_id == l->e.serial) {
-
-				// OK - do the heavier checking
-				if (extract_search_items(l) == 0) {
-					// If match only failed... 
-					if (event_failed == F_FAILED &&
-							l->s.success != 0)
-						return 0;
-					// If match only success... 
-					if (event_failed == F_SUCCESS &&
-							l->s.success == 0)
-						return 0;
+			// OK - do the heavier checking
+			if (extract_search_items(l) == 0) {
+				if (classify_success(l) && classify_conf(l))
 					return 1;
-				}
+				return 0;
 			}
 		}
 	}
@@ -118,7 +199,7 @@ int scan(llist *l)
 
 int per_event_processing(llist *l)
 {
-	int rc = 0;
+	int rc;
 
 	switch (report_detail)
 	{
@@ -130,6 +211,7 @@ int per_event_processing(llist *l)
 			break;
 		case D_SPECIFIC:
 		default:
+			rc = 0;
 			break;
 	} 
 	return rc;
@@ -146,39 +228,87 @@ static int per_event_summary(llist *l)
 			rc = 1;
 			break;
 		case RPT_AVC:
-			list_first(l);
-			if (list_find_msg(l, AUDIT_AVC) && l->s.tcontext) 
-				slist_add_if_uniq(&sd.avc_objs, l->s.tcontext);
-			else {
-				list_first(l);
-				if (list_find_msg(l, AUDIT_USER_AVC) && 
-							l->s.tcontext) {
-					slist_add_if_uniq(&sd.avc_objs,
-							l->s.tcontext);
+			if (list_find_msg(l, AUDIT_AVC)) {
+				if (alist_find_avc(l->s.avc)) {
+					do { 
+						slist_add_if_uniq(&sd.avc_objs,
+						      l->s.avc->cur->tcontext);
+					} while (alist_next_avc(l->s.avc));
+				}
+			} else {
+				if (list_find_msg(l, AUDIT_USER_AVC)) {
+					if (alist_find_avc(l->s.avc)) { 
+						do {
+							slist_add_if_uniq(
+								&sd.avc_objs,
+						    l->s.avc->cur->tcontext);
+						} while (alist_next_avc(
+								l->s.avc));
+					}
+				}
+			}
+			break;
+		case RPT_MAC:
+			if (list_find_msg_range(l, AUDIT_MAC_POLICY_LOAD,
+						AUDIT_MAC_MAP_DEL)) {
+				ilist_add_if_uniq(&sd.mac_list, 
+							l->head->type, 0);
+			} else {
+				if (list_find_msg_range(l, 
+					AUDIT_FIRST_USER_LSPP_MSG,
+						AUDIT_LAST_USER_LSPP_MSG)) {
+					ilist_add_if_uniq(&sd.mac_list, 
+							l->head->type, 0);
 				}
 			}
 			break;
 		case RPT_CONFIG:
 			UNIMPLEMENTED;
 			break;
+		case RPT_AUTH:
+			if (list_find_msg(l, AUDIT_USER_AUTH)) {
+				if (l->s.loginuid == -1 && l->s.acct != NULL)
+					slist_add_if_uniq(&sd.users, l->s.acct);
+				else {
+					char name[64];
+
+					slist_add_if_uniq(&sd.users,
+						aulookup_uid(l->s.loginuid,
+							name,
+							sizeof(name))
+						);
+				}
+			} else if (list_find_msg(l, AUDIT_USER_ACCT)) {
+				// Only count the failures
+				if (l->s.success == S_FAILED) {
+					if (l->s.loginuid == -1 && 
+						l->s.acct != NULL)
+					slist_add_if_uniq(&sd.users, l->s.acct);
+					else {
+						char name[64];
+	
+						slist_add_if_uniq(&sd.users,
+							aulookup_uid(
+								l->s.loginuid,
+								name,
+								sizeof(name))
+							);
+					}
+				}
+			}
+			break;
 		case RPT_LOGIN:
-			list_first(l);
 			if (list_find_msg(l, AUDIT_USER_LOGIN)) {
 				if (l->s.loginuid == -1 && l->s.acct != NULL)
 					slist_add_if_uniq(&sd.users, l->s.acct);
 				else {
-					struct passwd *pw;
+					char name[64];
 
-					pw = getpwuid(l->s.loginuid);
-					if (pw) {
-						slist_add_if_uniq(&sd.users,
-							pw->pw_name);
-					} else {
-						char tmp[32];
-						snprintf(tmp, sizeof(tmp), "%d",
-							l->s.loginuid);
-						slist_add_if_uniq(&sd.users, tmp);
-					}
+					slist_add_if_uniq(&sd.users,
+						aulookup_uid(l->s.loginuid,
+							name,
+							sizeof(name))
+						);
 				}
 			}
 			break;
@@ -235,22 +365,6 @@ static int per_event_summary(llist *l)
 			if (l->s.exe)
 				slist_add_if_uniq(&sd.exes, l->s.exe);
 			break;
-		case RPT_WATCH:
-			list_first(l);
-			if (list_find_msg(l, AUDIT_FS_WATCH)) {
-				const snode *sn;
-				slist *sptr = l->s.filename;
-
-				slist_first(sptr);
-				sn=slist_get_cur(sptr);
-				while (sn) {
-					if (sn->str)
-						slist_add_if_uniq(&sd.files,
-								sn->str);
-					sn=slist_next(sptr);
-				}
-			} 
-			break;
 		case RPT_ANOMALY:
 			if (list_find_msg_range(l, AUDIT_FIRST_ANOM_MSG,
 							AUDIT_LAST_ANOM_MSG)) {
@@ -272,6 +386,20 @@ static int per_event_summary(llist *l)
 							l->head->type, 0);
 			}
 			break;
+		case RPT_CRYPTO:
+			if (list_find_msg_range(l, AUDIT_FIRST_KERN_CRYPTO_MSG,
+						AUDIT_LAST_KERN_CRYPTO_MSG)) {
+				ilist_add_if_uniq(&sd.crypto_list, 
+							l->head->type, 0);
+			} else {
+				if (list_find_msg_range(l, 
+					AUDIT_FIRST_CRYPTO_MSG,
+						AUDIT_LAST_CRYPTO_MSG)) {
+					ilist_add_if_uniq(&sd.crypto_list, 
+							l->head->type, 0);
+				}
+			}
+			break;
 		default:
 			break;
 	}
@@ -285,32 +413,64 @@ static int per_event_detailed(llist *l)
 	switch (report_type)
 	{
 		case RPT_AVC:
-			list_first(l);
 			if (list_find_msg(l, AUDIT_AVC)) {
 				print_per_event_item(l);
 				rc = 1;
 			}
 			break;
+		case RPT_MAC:
+			if (report_detail == D_DETAILED) {
+				if (list_find_msg_range(l, 
+						AUDIT_MAC_POLICY_LOAD,
+						AUDIT_MAC_IPSEC_DELSPD)) {
+					print_per_event_item(l);
+					rc = 1;
+				} else {
+					if (list_find_msg_range(l, 
+						AUDIT_FIRST_USER_LSPP_MSG,
+						AUDIT_LAST_USER_LSPP_MSG)) {
+						print_per_event_item(l);
+						rc = 1;
+					}
+				}
+			}
+			break;
 		case RPT_CONFIG:
-			list_first(l);
 			if (list_find_msg(l, AUDIT_CONFIG_CHANGE)) {
 				print_per_event_item(l);
 				rc = 1;
 			} else if (list_find_msg(l, AUDIT_DAEMON_CONFIG)) {
 				print_per_event_item(l);
 				rc = 1;
+			} else if (list_find_msg(l, AUDIT_USYS_CONFIG)) {
+				print_per_event_item(l);
+				rc = 1;
+			} else if (list_find_msg_range(l,
+					AUDIT_MAC_POLICY_LOAD,
+					AUDIT_MAC_IPSEC_DELSPD)) {
+						print_per_event_item(l);
+						rc = 1;
+			}
+			break;
+		case RPT_AUTH:
+			if (list_find_msg(l, AUDIT_USER_AUTH)) {
+				print_per_event_item(l);
+				rc = 1;
+			} else if (list_find_msg(l, AUDIT_USER_ACCT)) {
+				// Only count the failed acct
+				if (l->s.success == S_FAILED) {
+					print_per_event_item(l);
+					rc = 1;
+				}
 			}
 			break;
 		case RPT_LOGIN:
-			list_first(l);
 			if (list_find_msg(l, AUDIT_USER_LOGIN)) {
 				print_per_event_item(l);
 				rc = 1;
 			}
 			break;
 		case RPT_ACCT_MOD:
-			// FIXME: consider adding/deleting users & groups
-			list_first(l);
 			if (list_find_msg(l, AUDIT_USER_CHAUTHTOK)) {
 				print_per_event_item(l);
 				rc = 1;
@@ -321,6 +481,11 @@ static int per_event_detailed(llist *l)
 			} else if (list_find_msg(l, AUDIT_CHGRP_ID)) {
 				print_per_event_item(l);
 				rc = 1;
+			} else if (list_find_msg_range(l,
+					AUDIT_ROLE_ASSIGN,
+					AUDIT_ROLE_REMOVE)) {
+				print_per_event_item(l);
+				rc = 1;
 			}
 			break;
 		case RPT_EVENT:
@@ -409,17 +574,6 @@ static int per_event_detailed(llist *l)
 				UNIMPLEMENTED;
 			}
 			break;
-		case RPT_WATCH:
-			list_first(l);
-			if (report_detail == D_DETAILED) {
-				if (list_find_msg(l, AUDIT_FS_WATCH)) {
-					print_per_event_item(l);
-					rc = 1;
-				}
-			} else { //  specific watch report
-				UNIMPLEMENTED;
-			}
-			break;
 		case RPT_ANOMALY:
 			if (report_detail == D_DETAILED) {
 				if (list_find_msg_range(l, 
@@ -451,6 +605,25 @@ static int per_event_detailed(llist *l)
 				UNIMPLEMENTED;
 			}
 			break;
+		case RPT_CRYPTO:
+			if (report_detail == D_DETAILED) {
+				if (list_find_msg_range(l, 
+						AUDIT_FIRST_KERN_CRYPTO_MSG,
+						AUDIT_LAST_KERN_CRYPTO_MSG)) {
+					print_per_event_item(l);
+					rc = 1;
+				} else {
+					if (list_find_msg_range(l, 
+						AUDIT_FIRST_CRYPTO_MSG,
+						AUDIT_LAST_CRYPTO_MSG)) {
+						print_per_event_item(l);
+						rc = 1;
+					}
+				}
+			} else { // FIXME: specific crypto report
+				UNIMPLEMENTED;
+			}
+			break;
 		default:
 			break;
 	}
@@ -459,40 +632,64 @@ static int per_event_detailed(llist *l)
 
 static void do_summary_total(llist *l)
 {
-	char tmp[32];
-
 	// add events
 	sd.events++;
 
 	// add config changes
-	list_first(l);
 	if (list_find_msg(l, AUDIT_CONFIG_CHANGE))
 		sd.changes++;
-	list_first(l);
 	if (list_find_msg(l, AUDIT_DAEMON_CONFIG)) 
 		sd.changes++;
+	if (list_find_msg(l, AUDIT_USYS_CONFIG)) 
+		sd.changes++;
+	list_first(l);
+	if (list_find_msg_range(l, AUDIT_MAC_POLICY_LOAD,
+					AUDIT_MAC_IPSEC_DELSPD))
+		sd.changes++;
 
 	// add acct changes
-	list_first(l);
 	if (list_find_msg(l, AUDIT_USER_CHAUTHTOK))
 		sd.acct_changes++;
-	list_first(l);
 	if (list_find_msg_range(l, AUDIT_ADD_USER, AUDIT_DEL_GROUP))
 		sd.acct_changes++;
-	list_first(l);
 	if (list_find_msg(l, AUDIT_CHGRP_ID))
 		sd.acct_changes++;
-	// add logins
 	list_first(l);
+	if (list_find_msg_range(l, AUDIT_ROLE_ASSIGN, AUDIT_ROLE_REMOVE))
+		sd.acct_changes++;
+
+	// Crypto
+	list_first(l);
+	if (list_find_msg_range(l, AUDIT_FIRST_KERN_CRYPTO_MSG,
+					AUDIT_LAST_KERN_CRYPTO_MSG))
+		sd.crypto++;
+	if (list_find_msg_range(l, AUDIT_FIRST_CRYPTO_MSG, 
+					AUDIT_LAST_CRYPTO_MSG))
+		sd.crypto++;
+
+	// add logins
 	if (list_find_msg(l, AUDIT_USER_LOGIN)) {
-		if (l->s.success == 1)
+		if (l->s.success == S_SUCCESS)
 			sd.good_logins++;
-		else if (l->s.success == 0)
+		else if (l->s.success == S_FAILED)
 			sd.bad_logins++;
 	}
 
+	// add use of auth
+	if (list_find_msg(l, AUDIT_USER_AUTH)) {
+		if (l->s.success == S_SUCCESS)
+			sd.good_auth++;
+		else if (l->s.success == S_FAILED)
+			sd.bad_auth++;
+	} else if (list_find_msg(l, AUDIT_USER_ACCT)) {
+		// Only count the failures
+		if (l->s.success == S_FAILED)
+			sd.bad_auth++;
+	}
+
 	// add users
 	if (l->s.loginuid != -1) {
+		char tmp[32];
 		snprintf(tmp, sizeof(tmp), "%d", l->s.loginuid);
 		slist_add_if_uniq(&sd.users, tmp);
 	}
@@ -524,24 +721,24 @@ static void do_summary_total(llist *l)
 	}
 
 	// add avcs
-	list_first(l);
-	if (list_find_msg(l, AUDIT_AVC)) { 
+	if (list_find_msg(l, AUDIT_AVC)) 
 		sd.avcs++;
-	} else { 
-		list_first(l);
-		if (list_find_msg(l, AUDIT_USER_AVC))
+	else if (list_find_msg(l, AUDIT_USER_AVC))
 			sd.avcs++;
-	}
+
+	// MAC
+	list_first(l);
+	if (list_find_msg_range(l, AUDIT_MAC_POLICY_LOAD,
+					AUDIT_MAC_IPSEC_DELSPD))
+		sd.mac++;
+	if (list_find_msg_range(l, AUDIT_FIRST_USER_LSPP_MSG, 
+					AUDIT_LAST_USER_LSPP_MSG))
+		sd.mac++;
 
 	// add failed syscalls
-	if (l->s.success == 0 && l->s.syscall > 0)
+	if (l->s.success == S_FAILED && l->s.syscall > 0)
 		sd.failed_syscalls++;
 
-	// add watches
-	list_first(l);
-	if (list_find_msg(l, AUDIT_FS_WATCH))
-		sd.fs_watches++;
-
 	// add pids
 	if (l->s.pid != -1) {
 		ilist_add_if_uniq(&sd.pids, l->s.pid, 0);
