@@ -1,6 +1,6 @@
 /*
 * ausearch-llist.c - Minimal linked list library
-* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005-2006 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -35,8 +35,9 @@ void list_create(llist *l)
 	l->e.serial = 0L;      
 	l->s.gid = -1;          
 	l->s.egid = -1;         
+	l->s.ppid = -1;            
 	l->s.pid = -1;            
-	l->s.success = -1;
+	l->s.success = S_UNSET;
 	l->s.uid = -1;            
 	l->s.euid = -1;           
 	l->s.loginuid = -1;
@@ -45,12 +46,9 @@ void list_create(llist *l)
 	l->s.terminal = NULL;
 	l->s.cwd = NULL;
 	l->s.exe = NULL;
+	l->s.key = NULL;
 	l->s.comm = NULL;
-	l->s.scontext = NULL;
-	l->s.tcontext = NULL;
-	l->s.avc_result = AVC_UNSET;
-	l->s.avc_perm = NULL;
-	l->s.avc_class = NULL;
+	l->s.avc = NULL;
 	l->s.acct = NULL;
 	l->s.arch = 0;
 	l->s.syscall = 0;
@@ -106,7 +104,7 @@ void list_append(llist *l, lnode *node)
 	newnode->next = NULL;
 
 	// if we are at top, fix this up
-	if (l->head == 0)
+	if (l->head == NULL)
 		l->head = newnode;
 	else	// Otherwise add pointer to newnode
 		l->cur->next = newnode;
@@ -143,8 +141,7 @@ void list_clear(llist* l)
 	current = l->head;
 	while (current) {
 		nextnode=current->next;
-		if (current->message)
-			free(current->message);
+		free(current->message);
 		free(current);
 		current=nextnode;
 	}
@@ -156,8 +153,9 @@ void list_clear(llist* l)
 	l->e.serial = 0L;      
 	l->s.gid = -1;
 	l->s.egid = -1;
+	l->s.ppid = -1;
 	l->s.pid = -1;
-	l->s.success = -1;
+	l->s.success = S_UNSET;
 	l->s.uid = -1;
 	l->s.euid = -1;
 	l->s.loginuid = -1;
@@ -182,26 +180,18 @@ void list_clear(llist* l)
 		free(l->s.exe);
 		l->s.exe = NULL;
 	}
+	if (l->s.key) {
+		free(l->s.key);
+		l->s.key = NULL;
+	}
 	if (l->s.comm) {
 		free(l->s.comm);
 		l->s.comm = NULL;
 	}
-	if (l->s.scontext) {
-		free(l->s.scontext);
-		l->s.scontext = NULL;
-	}
-	if (l->s.tcontext) {
-		free(l->s.tcontext);
-		l->s.tcontext = NULL;
-	}
-	l->s.avc_result = AVC_UNSET;
-	if (l->s.avc_perm) {
-		free(l->s.avc_perm);
-		l->s.avc_perm = NULL;
-	}
-	if (l->s.avc_class) {
-		free(l->s.avc_class);
-		l->s.avc_class = NULL;
+	if (l->s.avc) {
+		alist_clear(l->s.avc);
+		free(l->s.avc);
+		l->s.avc = NULL;
 	}
 	if (l->s.acct) {
 		free(l->s.acct);
