@@ -1,6 +1,6 @@
 /*
 * ausearch-llist.h - Header file for ausearch-llist.c
-* Copyright (c) 2005-06 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005-2006 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This software may be freely redistributed and/or modified under the
@@ -27,8 +27,9 @@
 #include "config.h"
 #include <sys/types.h>
 #include "ausearch-string.h"
+#include "ausearch-avc.h"
+#include "ausearch-common.h"
 
-typedef enum { AVC_UNSET, AVC_DENIED, AVC_GRANTED } avc_t;
 
 typedef struct
 {
@@ -39,26 +40,24 @@ typedef struct
 
 typedef struct
 {
+  pid_t ppid;           // parent process ID
   pid_t pid;            // process ID
   uid_t uid;            // user ID
   uid_t euid;           // effective user ID
   uid_t loginuid;       // login user ID
   gid_t gid;            // group ID
   gid_t egid;           // effective group ID
-  int success;          // success flag, 1 = yes, 0 = no, -1 = unset
+  success_t success;    // success flag, 1 = yes, 0 = no, -1 = unset
   int arch;             // arch
   int syscall;          // syscall
   char *hostname;       // hostname
-  slist *filename;      // filename
+  slist *filename;      // filename list
   char *cwd;            // current working dir
   char *exe;            // executable
+  char *key;            // key field
   char *terminal;       // terminal
   char *comm;           // comm name
-  char *scontext;       // se linux subject context
-  char *tcontext;       // se linux object context
-  avc_t avc_result;     // se linux avc denied/granted
-  char *avc_perm;      // se linux avc permission mentioned
-  char *avc_class;     // se linux class mentioned
+  alist *avc;		// avcs for the event
   char *acct;		// account used when uid is invalid
 } search_items;
 
