@@ -1,6 +1,6 @@
 /*
 * ausearch-match.c - Extract interesting fields and check for match
-* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005-06 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -56,6 +56,9 @@ int match(llist *l)
 					return 0;
 				if (group_match(l) == 0)
 					return 0;
+				if ((event_ppid != -1) && 
+						(event_ppid != l->s.ppid))
+					return 0;
 				if ((event_pid != -1) && 
 						(event_pid != l->s.pid))
 					return 0;
@@ -63,7 +66,7 @@ int match(llist *l)
 						(event_syscall != l->s.syscall))
 					return 0;
 
-				if ((event_success != -1) &&
+				if ((event_success != S_UNSET) &&
 						(event_success != l->s.success))
 					return 0;
 				// event_type requires looking at each item
@@ -144,32 +147,14 @@ int match(llist *l)
 					if (strmatch(event_comm, 
 						l->s.comm) == 0)
 						return 0; 
-				}
+				}				
 				if (event_key) {
-					if (l->s.filename == NULL)
+					if (l->s.key == NULL)
 						return 0;
-					else {
-						int found = 0;
-						const snode *sn;
-						slist *sptr = l->s.filename;
-
-						slist_first(sptr);
-						sn=slist_get_cur(sptr);
-						do {
-							if (sn->str == NULL)
-								return 0;
-							if (strmatch(
-								event_key,
-								sn->key)) {
-								found = 1;
-								break;
-							}
-						} while ((sn=slist_next(sptr)));
-						if (!found)
-							return 0;
-					}
-				}
-
+					if (strmatch(event_key, 
+						l->s.key) == 0)
+						return 0; 
+				}				
 				if (context_match(l) == 0)
 					return 0;
 				return 1;
@@ -252,37 +237,49 @@ static int group_match(llist *l)
  */
 static int context_match(llist *l)
 {
-	if (event_subject && event_object && 
-			strcmp(event_subject, event_object) == 0) {
-		int found = 0;
-		if (l->s.scontext == NULL && 
-				l->s.tcontext == NULL)
-			return 0;
-		if (l->s.scontext && 
-			strmatch(event_subject, 
-				l->s.scontext) )
-			found = 1;
-		else if (l->s.tcontext &&
-			strmatch(event_subject, 
-				l->s.tcontext) )
-			found = 1;
-			if (found == 0)
+	if (event_se) { /* This does the "or" check if -se test */
+		if (event_subject) {
+			if (l->s.avc && alist_find_subj(l->s.avc)) {
+				do {
+					if (strmatch(event_subject, 
+						l->s.avc->cur->scontext))
+						return 1;
+				} while(alist_next_subj(l->s.avc));
+			}
+		} 
+		if (event_object) {
+			if (l->s.avc && alist_find_obj(l->s.avc)) {
+				do {
+					if (strmatch(event_object, 
+						l->s.avc->cur->tcontext))
+						return 1;
+				} while(alist_next_obj(l->s.avc));
+			}
+		}
+		return 0;
+	} else {
+		if (event_subject) {
+			if (l->s.avc == NULL)
 				return 0;
-		return 1;
-	}
-	if (event_subject) {
-		if (l->s.scontext == NULL)
-			return 0;
-		if (strmatch(event_subject, 
-				l->s.scontext) == 0)
-			return 0;
-	}
-	if (event_object) {
-		if (l->s.tcontext == NULL)
-			return 0;
-		if (strmatch(event_object, 
-				l->s.tcontext) == 0)
-			return 0;
+			if (alist_find_subj(l->s.avc)) {
+				do {
+					if (strmatch(event_subject, 
+						l->s.avc->cur->scontext) == 0)
+						return 0;
+				} while(alist_next_subj(l->s.avc));
+			}
+		} 
+		if (event_object) {
+			if (l->s.avc == NULL)
+				return 0;
+			if (alist_find_obj(l->s.avc)) {
+				do {
+					if (strmatch(event_object, 
+						l->s.avc->cur->tcontext) == 0)
+						return 0;
+				} while(alist_next_obj(l->s.avc));
+			}
+		}
 	}
 	return 1;
 }
