@@ -17,6 +17,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Authors:
+ *     Debora Velarde <dvelarde@us.ibm.com>
  *     Steve Grubb <sgrubb@redhat.com>
  */
 
@@ -40,11 +41,12 @@ char *user_file = NULL;
 unsigned int event_id = -1;
 gid_t event_gid = -1, event_egid = -1;
 ilist *event_type = NULL;
-pid_t event_pid = -1;
-int event_success = -1, event_exact_match = 0;
+pid_t event_pid = -1, event_ppid = -1;
+success_t event_success = S_UNSET;
+int event_exact_match = 0;
 uid_t event_uid = -1, event_euid = -1, event_loginuid = -1;
 int event_syscall = -1;
-int event_ua = 0, event_ga = 0;
+int event_ua = 0, event_ga = 0, event_se = 0;
 const char *event_key = NULL;
 const char *event_filename = NULL;
 const char *event_exe = NULL;
@@ -53,7 +55,7 @@ const char *event_hostname = NULL;
 const char *event_terminal = NULL;
 const char *event_subject = NULL;
 const char *event_object = NULL;
-report_t report_format = RPT_RAW;
+report_t report_format = RPT_DEFAULT;
 
 struct nv_pair {
     int        value;
@@ -62,41 +64,72 @@ struct nv_pair {
 
 
 enum { S_EVENT, S_COMM, S_FILENAME, S_ALL_GID, S_EFF_GID, S_GID, S_HELP,
-S_HOSTNAME, S_INTERP, S_INFILE, S_MESSAGE_TYPE, S_PID, S_SYSCALL, S_SUCCESS,
+S_HOSTNAME, S_INTERP, S_INFILE, S_MESSAGE_TYPE, S_PID, S_SYSCALL, S_OSUCCESS,
 S_TIME_END, S_TIME_START, S_TERMINAL, S_ALL_UID, S_EFF_UID, S_UID, S_LOGINID,
 S_VERSION, S_EXACT_MATCH, S_EXECUTABLE, S_CONTEXT, S_SUBJECT, S_OBJECT,
-S_KEY };
+S_PPID, S_KEY, S_RAW };
 
 static struct nv_pair optiontab[] = {
 	{ S_EVENT, "-a" },
+	{ S_EVENT, "--event" },
 	{ S_COMM, "-c" },
+	{ S_COMM, "--comm" },
 	{ S_FILENAME, "-f" },
+	{ S_FILENAME, "--file" },
 	{ S_ALL_GID, "-ga" },
+	{ S_ALL_GID, "--gid-all" },
 	{ S_EFF_GID, "-ge" },
+	{ S_EFF_GID, "--gid-effective" },
 	{ S_GID, "-gi" },
+	{ S_GID, "--gid" },
 	{ S_HELP, "-h" },
 	{ S_HELP, "--help" },
 	{ S_HOSTNAME, "-hn" },
+	{ S_HOSTNAME, "--host" },
 	{ S_INTERP, "-i" },
+	{ S_INTERP, "--interpret" },
 	{ S_INFILE, "-if" },
+	{ S_INFILE, "--input" },
 	{ S_KEY, "-k" },
+	{ S_KEY, "--key" },
 	{ S_MESSAGE_TYPE, "-m" },
+	{ S_MESSAGE_TYPE, "--message" },
 	{ S_OBJECT, "-o" },
+	{ S_OBJECT, "--object" },
 	{ S_PID, "-p" },
+	{ S_PID, "--pid" },
+	{ S_PPID, "-pp" },
+	{ S_PPID, "--ppid" },
+	{ S_RAW, "-r" },
+	{ S_RAW, "--raw" },
 	{ S_SYSCALL, "-sc" },
+	{ S_SYSCALL, "--syscall" },
 	{ S_CONTEXT, "-se" },
+	{ S_CONTEXT, "--context" },
 	{ S_SUBJECT, "-su" },
-	{ S_SUCCESS, "-sv" },
+	{ S_SUBJECT, "--subject" },
+	{ S_OSUCCESS, "-sv" },
+	{ S_OSUCCESS, "--success" },
 	{ S_TIME_END, "-te"},
+	{ S_TIME_END, "--end"},
 	{ S_TIME_START, "-ts" },
+	{ S_TIME_START, "--start" },
 	{ S_TERMINAL, "-tm" },
+	{ S_TERMINAL, "--terminal" },
 	{ S_ALL_UID, "-ua" },
+	{ S_ALL_UID, "--uid-all" },
 	{ S_EFF_UID, "-ue" },
+	{ S_EFF_UID, "--uid-effective" },
 	{ S_UID, "-ui" },
+	{ S_UID, "--uid" },
 	{ S_LOGINID, "-ul" },
+	{ S_LOGINID, "--loginuid" },
 	{ S_VERSION, "-v" },
+	{ S_VERSION, "--version" },
 	{ S_EXACT_MATCH, "-w" },
-	{ S_EXECUTABLE, "-x" }
+	{ S_EXACT_MATCH, "--word" },
+	{ S_EXECUTABLE, "-x" },
+	{ S_EXECUTABLE, "--executable" }
 };
 #define OPTION_NAMES (sizeof(optiontab)/sizeof(optiontab[0]))
 
@@ -114,34 +147,36 @@ static int audit_lookup_option(const char *name)
 static void usage(void)
 {
 	printf("usage: ausearch [options]\n"
-	"\t-a  <Audit event id>\t\tsearch based on audit event id\n"
-	"\t-c  <Comm name>\t\t\tsearch based on command line name\n"
-	"\t-f  <File name>\t\t\tsearch based on file name\n"
-	"\t-ga <all Group id>\t\tsearch based on All group ids\n"
-	"\t-ge <effective Group id>\tsearch based on Effective group id\n"
-	"\t-gi <Group Id>\t\t\tsearch based on group id\n"
-	"\t-h\t\t\t\thelp\n"
-	"\t-hn <Host Name>\t\t\tsearch based on host name\n"
-	"\t-i\t\t\t\tInterpret results to be human readable\n"
-	"\t-if <Input File name>\t\tuse this file instead of current logs\n"
-	"\t-k  <key string>\t\tsearch based on key field\n"
-	"\t-m  <Message type>\t\tsearch based on message type\n"
-	"\t-o  <SE Linux Object context>\tsearch based on context of object\n"
-	"\t-p  <Process id>\t\tsearch based on process id\n"
-	"\t-sc <SysCall name>\t\tsearch based on syscall name or number\n"
-	"\t-se <SE Linux context>\t\tsearch based on either subject or object\n"
-	"\t-su <SE Linux context>\t\tsearch based on context of the Subject\n"
-	"\t-sv <Success Value>\t\tsearch based on syscall or event\n\t\t\t\t\tsuccess value\n"
-	"\t-te [end date] [end time]\tending date & time for search\n"
-	"\t-ts [start date] [start time]\tstarting data & time for search\n"
-	"\t-tm <TerMinal>\t\t\tsearch based on terminal\n"
-	"\t-ua <all User id>\t\tsearch based on All user id's\n"
-	"\t-ue <effective User id>\t\tsearch based on Effective user id\n"
-	"\t-ui <User Id>\t\t\tsearch based on user id\n"
-	"\t-ul <login id>\t\t\tsearch based on the User's Login id\n"
-	"\t-v\t\t\t\tversion\n"
-	"\t-w\t\t\t\tstring matches are whole word\n"
-	"\t-x  <executable name>\t\tsearch based on excutable name\n"
+	"\t-a,--event <Audit event id>\tsearch based on audit event id\n"
+	"\t-c,--comm  <Comm name>\t\tsearch based on command line name\n"
+	"\t-f,--file  <File name>\t\tsearch based on file name\n"
+	"\t-ga,--gid-all <all Group id>\tsearch based on All group ids\n"
+	"\t-ge,--gid-effective <effective Group id>  search based on Effective\n\t\t\t\t\tgroup id\n"
+	"\t-gi,--gid <Group Id>\t\tsearch based on group id\n"
+	"\t-h,--help\t\t\thelp\n"
+	"\t-hn,--host <Host Name>\t\tsearch based on host name\n"
+	"\t-i,--interpret\t\t\tInterpret results to be human readable\n"
+	"\t-if,--input <Input File name>\tuse this file instead of current logs\n"
+	"\t-k,--key  <key string>\t\tsearch based on key field\n"
+	"\t-m,--message  <Message type>\tsearch based on message type\n"
+	"\t-o,--object  <SE Linux Object context> search based on context of object\n"
+	"\t-p,--pid  <Process id>\t\tsearch based on process id\n"
+	"\t-pp,--ppid <Parent Process id>\tsearch based on parent process id\n"
+	"\t-r,--raw\t\t\toutput is completely unformatted\n"
+	"\t-sc,--syscall <SysCall name>\tsearch based on syscall name or number\n"
+	"\t-se,--context <SE Linux context> search based on either subject or\n\t\t\t\t\t object\n"
+	"\t-su,--subject <SE Linux context> search based on context of the Subject\n"
+	"\t-sv,--success <Success Value>\tsearch based on syscall or event\n\t\t\t\t\tsuccess value\n"
+	"\t-te,--end [end date] [end time]\tending date & time for search\n"
+	"\t-ts,--start [start date] [start time]\tstarting data & time for search\n"
+	"\t-tm,--terminal <TerMinal>\tsearch based on terminal\n"
+	"\t-ua,--uid-all <all User id>\tsearch based on All user id's\n"
+	"\t-ue,--uid-effective <effective User id>  search based on Effective\n\t\t\t\t\tuser id\n"
+	"\t-ui,--uid <User Id>\t\tsearch based on user id\n"
+	"\t-ul,--loginuid <login id>\tsearch based on the User's Login id\n"
+	"\t-v,--version\t\t\tversion\n"
+	"\t-w,--word\t\t\tstring matches are whole word\n"
+	"\t-x,--executable  <executable name>  search based on excutable name\n"
 	);
 }
 
@@ -225,7 +260,8 @@ int check_params(int count, char *vars[])
 		case S_EVENT:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -a\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -248,7 +284,8 @@ int check_params(int count, char *vars[])
 		case S_COMM:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -c\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			} else {
@@ -261,7 +298,8 @@ int check_params(int count, char *vars[])
 		case S_FILENAME:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -f\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 			} else {
 				event_filename = strdup(optarg);
@@ -273,7 +311,8 @@ int check_params(int count, char *vars[])
 		case S_KEY:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -k\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 			} else {
 				event_key = strdup(optarg);
@@ -285,7 +324,8 @@ int check_params(int count, char *vars[])
 		case S_ALL_GID:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -ga\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -318,7 +358,8 @@ int check_params(int count, char *vars[])
 		case S_EFF_GID:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -ge\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -349,7 +390,8 @@ int check_params(int count, char *vars[])
 		case S_GID:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -gi\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -384,7 +426,8 @@ int check_params(int count, char *vars[])
 		case S_HOSTNAME:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -hn\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 			} else {
 				event_hostname = strdup(optarg);
@@ -394,17 +437,26 @@ int check_params(int count, char *vars[])
 			}
 			break;
 		case S_INTERP:
-			report_format = RPT_INTERP;
+			if (report_format == RPT_DEFAULT)
+				report_format = RPT_INTERP;
+			else {
+				fprintf(stderr, 
+					"Conflicting output format %s\n",
+					vars[c]);
+        	                retval = -1;
+			}
 			if (optarg) {
 				fprintf(stderr, 
-					"Argument is NOT required for -i\n");
+					"Argument is NOT required for %s\n",
+					vars[c]);
         	                retval = -1;
 			}
 			break;
 		case S_INFILE:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -if\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 			} else {
 				user_file = strdup(optarg);
@@ -416,7 +468,8 @@ int check_params(int count, char *vars[])
 		case S_MESSAGE_TYPE:
 	                if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -m\n");
+					"Argument is required for %s\n",
+					vars[c]);
         	                retval = -1;
 	                } else {
 				if (strcasecmp(optarg, "ALL") != 0) {
@@ -442,7 +495,8 @@ int check_params(int count, char *vars[])
 		case S_OBJECT:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -o\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			} else {
@@ -452,10 +506,32 @@ int check_params(int count, char *vars[])
 				c++;
 			}
 			break;
+		case S_PPID:
+			if (!optarg) {
+				fprintf(stderr, 
+					"Argument is required for %s\n",
+					vars[c]);
+				retval = -1;
+				break;
+			}
+			if (isdigit(optarg[0])) {
+				errno = 0;
+				event_ppid = strtol(optarg,NULL,10);
+				if (errno)
+					retval = -1;
+				c++;
+			} else {
+				fprintf(stderr, 
+			"Parent process id must be a numeric value, was %s\n",
+					optarg);
+				retval = -1;
+			}
+			break;
 		case S_PID:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -p\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -472,10 +548,25 @@ int check_params(int count, char *vars[])
 				retval = -1;
 			}
 			break;
+		case S_RAW:
+			if (report_format == RPT_DEFAULT)
+				report_format = RPT_RAW;
+			else {
+				fprintf(stderr, 
+					"Conflicting output format --raw\n");
+        	                retval = -1;
+			}
+			if (optarg) {
+				fprintf(stderr, 
+					"Argument is NOT required for --raw\n");
+        	                retval = -1;
+			}
+			break;
 		case S_SYSCALL:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -sc\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -511,7 +602,8 @@ int check_params(int count, char *vars[])
 		case S_CONTEXT:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -se\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			} else {
@@ -521,13 +613,15 @@ int check_params(int count, char *vars[])
 				event_object = strdup(optarg);
         	                if (event_object == NULL)
                 	                retval = -1;
+				event_se = 1;
 				c++;
 			}
 			break;
 		case S_SUBJECT:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -su\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			} else {
@@ -537,19 +631,20 @@ int check_params(int count, char *vars[])
 				c++;
 			}
 			break;
-		case S_SUCCESS:
+		case S_OSUCCESS:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -sv\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
                 	if ( (strstr(optarg, "yes")!=NULL) || 
 					(strstr(optarg, "no")!=NULL) ) {
                         	if (strcmp(optarg, "yes") == 0)
-					event_success = 1;
+					event_success = S_SUCCESS;
 				else
-					event_success = 0;
+					event_success = S_FAILED;
                 	} else {
                         	fprintf(stderr, 
 					"Success must be 'yes' or 'no'.\n");
@@ -582,12 +677,24 @@ int check_params(int count, char *vars[])
 							case T_NOW:
 							    set_tm_now(&d);
 								break;
+							case T_RECENT:
+							    set_tm_recent(&d);
+								break;
 							case T_TODAY:
 							    set_tm_today(&d);
 								break;
 							case T_YESTERDAY:
 							   set_tm_yesterday(&d);
 								break;
+							case T_THIS_WEEK:
+							   set_tm_this_week(&d);
+								break;
+							case T_THIS_MONTH:
+							  set_tm_this_month(&d);
+								break;
+							case T_THIS_YEAR:
+							   set_tm_this_year(&d);
+								break;
 						}
 						end_time = mktime(&d); 
 					} else if ( (strchr(optarg, ':')) == NULL) {
@@ -606,7 +713,8 @@ int check_params(int count, char *vars[])
 				break;
 			} 
 			fprintf(stderr, 
-				"-te requires either date and/or time\n");
+				"%s requires either date and/or time\n",
+				vars[c]);
 			retval = -1;
 			break;
 		case S_TIME_START:
@@ -634,12 +742,24 @@ int check_params(int count, char *vars[])
 							case T_NOW:
 							    set_tm_now(&d);
 								break;
+							case T_RECENT:
+							    set_tm_recent(&d);
+								break;
 							case T_TODAY:
 							    set_tm_today(&d);
 								break;
 							case T_YESTERDAY:
 							   set_tm_yesterday(&d);
 								break;
+							case T_THIS_WEEK:
+							   set_tm_this_week(&d);
+								break;
+							case T_THIS_MONTH:
+							  set_tm_this_month(&d);
+								break;
+							case T_THIS_YEAR:
+							   set_tm_this_year(&d);
+								break;
 						}
 						start_time = mktime(&d); 
 					}
@@ -659,13 +779,15 @@ int check_params(int count, char *vars[])
 				break;
 			}
 			fprintf(stderr, 
-				"-ts requires either date and/or time\n");
+				"%s requires either date and/or time\n",
+				vars[c]);
 			retval = -1;
 			break;
 		case S_TERMINAL:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -tm\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
                 	} else { 
 				event_terminal = strdup(optarg);
@@ -677,7 +799,8 @@ int check_params(int count, char *vars[])
 		case S_UID:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -ui\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -708,7 +831,8 @@ int check_params(int count, char *vars[])
 		case S_EFF_UID:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -ue\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -739,7 +863,8 @@ int check_params(int count, char *vars[])
 		case S_ALL_UID:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -ua\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -773,7 +898,8 @@ int check_params(int count, char *vars[])
 		case S_LOGINID:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -ul\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 				break;
 			}
@@ -811,7 +937,8 @@ int check_params(int count, char *vars[])
 		case S_EXECUTABLE:
 			if (!optarg) {
 				fprintf(stderr, 
-					"Argument is required for -x\n");
+					"Argument is required for %s\n",
+					vars[c]);
 				retval = -1;
 	               	} else { 
 				event_exe = strdup(optarg);
