@@ -27,29 +27,17 @@
 #include <time.h>
 #include <sys/types.h>
 #include "ausearch-int.h"
+#include "ausearch-common.h"
 
 /* Global variables that describe what search is to be performed */
-extern time_t start_time, end_time;
-extern unsigned int event_id;
-extern gid_t event_gid, event_egid;
 extern ilist *event_type;
-extern pid_t event_pid;
-extern int event_success;
-extern int event_exact_match;
-extern uid_t event_uid, event_euid, event_loginuid;
 extern const char *event_key;
-extern const char *event_filename;
-extern const char *event_exe;
-extern const char *event_comm;
-extern const char *event_hostname;
-extern const char *event_terminal;
 extern const char *event_subject;
 extern const char *event_object;
-extern int event_syscall;
-extern int event_ua, event_ga;
+extern int event_se;
+extern pid_t event_ppid;
 
 /* Data type to govern output format */
-typedef enum { RPT_RAW, RPT_INTERP, RPT_PRETTY } report_t;
 extern report_t report_format;
 
 /* Function to process commandline options */
