@@ -34,14 +34,16 @@
 #include "ausearch-options.h"
 #include "ausearch-parse.h"
 
+#define NAME_OFFSET 36
 
 static int parse_syscall(lnode *n, search_items *s);
 static int parse_dir(const lnode *n, search_items *s);
+static int common_path_parser(search_items *s, char *path);
+static int avc_parse_path(const lnode *n, search_items *s);
 static int parse_path(const lnode *n, search_items *s);
 static int parse_user(const lnode *n, search_items *s);
 static int parse_login(const lnode *n, search_items *s);
 static int parse_daemon(const lnode *n, search_items *s);
-static int parse_fs_watch(const lnode *n, search_items *s);
 static int parse_sockaddr(const lnode *n, search_items *s);
 static int parse_avc(const lnode *n, search_items *s);
 static int parse_kernel_anom(const lnode *n, search_items *s);
@@ -109,6 +111,18 @@ char *unescape(char *buf)
 	return str;
 }
 
+static int audit_avc_init(search_items *s)
+{
+	if (s->avc == NULL) {
+		//create
+		s->avc = malloc(sizeof(alist));
+		if (s->avc == NULL)
+			return -1;
+		alist_create(s->avc);
+	}
+	return 0;
+}
+
 /*
  * This function will take the list and extract the searchable fields from it.
  * It returns 0 on success and 1 on failure.
@@ -133,15 +147,15 @@ int extract_search_items(llist *l)
 				return -2;
 			*term = 0;
 			n->type = audit_name_to_msg_type(ptr);
+			*term = ' ';
 			if (n->type < 0) {
-//				fprintf(stderr, "Error converting type:%s\n", 
-//					ptr);
-//				return 3;
+/*				fprintf(stderr, "Error converting type:%s\n", 
+					ptr);
+				return 3; */
+
 				// If we don't know, don't extract.
-				*term = ' ';
 				return 0;
 			}
-			*term = ' ';
 
 			switch (n->type) {
 			case AUDIT_SYSCALL:
@@ -150,12 +164,12 @@ int extract_search_items(llist *l)
 			case AUDIT_CWD:
 				ret = parse_dir(n, s);
 				break;
+			case AUDIT_AVC_PATH:
+				ret = avc_parse_path(n, s);
+				break;
 			case AUDIT_PATH:
 				ret = parse_path(n, s);
 				break;
-			case AUDIT_FS_WATCH:
-				ret = parse_fs_watch(n, s);
-				break;
 			case AUDIT_USER:
 			case AUDIT_FIRST_USER_MSG...AUDIT_LAST_USER_MSG:
 			case AUDIT_FIRST_USER_MSG2...AUDIT_LAST_USER_MSG2:
@@ -184,18 +198,18 @@ int extract_search_items(llist *l)
 			   AUDIT_FIRST_KERN_ANOM_MSG...AUDIT_LAST_KERN_ANOM_MSG:
 				ret = parse_kernel_anom(n, s);
 				break;
-			case AUDIT_MAC_POLICY_LOAD...AUDIT_MAC_CONFIG_CHANGE:
+			case AUDIT_MAC_POLICY_LOAD...AUDIT_MAC_IPSEC_DELSPD:
 				ret = parse_simple_message(n, s);
 				break;
 			case AUDIT_KERNEL:
 			case AUDIT_IPC:
 			case AUDIT_SELINUX_ERR:
-			case AUDIT_AVC_PATH:
 				// Nothing to parse
 				break;
 			default:
 				break;
 			}
+			// if (ret) printf("type:%d ret:%d\n", n->type, ret);
 		} while ((n=list_next(l)) && ret==0);
 	}
 	return ret;
@@ -206,7 +220,7 @@ static int parse_syscall(lnode *n, search_items *s)
 	char *ptr, *str, *term;
 
 	term = n->message;
-	if (report_format != RPT_RAW) {
+	if (report_format > RPT_DEFAULT) {
 		// get arch
 		str = strstr(term, "arch=");
 		if (str == NULL) 
@@ -238,124 +252,152 @@ static int parse_syscall(lnode *n, search_items *s)
 	// get success
 	*term = ' ';
 	str = strstr(term, "success=");
-	if (str == NULL)
-		return 7;
-	ptr = str + 8;
-	term = strchr(ptr, ' ');
-	if (term == NULL)
-		return 8;
-	*term = 0;
-	if (strcmp(ptr, "yes") == 0)
-		s->success = 1;
-	else
-		s->success = 0;
+	if (str) { // exit_group does not set success !?!
+		ptr = str + 8;
+		term = strchr(ptr, ' ');
+		if (term == NULL)
+			return 7;
+		*term = 0;
+		if (strcmp(ptr, "yes") == 0)
+			s->success = S_SUCCESS;
+		else
+			s->success = S_FAILED;
+		*term = ' ';
+	}
 	// get a0
-	*term = ' ';
 	str = strstr(term, "a0=");
 	if (str == NULL)
-		return 9;
+		return 8;
 	ptr = str + 3;
 	term = strchr(ptr, ' ');
 	if (term == NULL)
-		return 10;
+		return 9;
 	*term = 0;
 	errno = 0;
 	// 64 bit dump on 32 bit machine looks bad here - need long long
 	n->a0 = strtoull(ptr, NULL, 16); // Hex
 	if (errno)
-		return 11;
-	// pid
+		return 10;
+	// ppid
 	*term = ' ';
+	str = strstr(term, "ppid=");
+	if (str != NULL) { // ppid is an optional field
+		ptr = str + 5;
+		term = strchr(ptr, ' ');
+		if (term == NULL)
+			return 11;
+		*term = 0;
+		errno = 0;
+		s->ppid = strtoul(ptr, NULL, 10);
+		if (errno)
+			return 12;
+		*term = ' ';
+	}
+	// pid
 	str = strstr(term, "pid=");
 	if (str == NULL)
-		return 12;
+		return 13;
 	ptr = str + 4;
 	term = strchr(ptr, ' ');
 	if (term == NULL)
-		return 13;
+		return 14;
 	*term = 0;
 	errno = 0;
 	s->pid = strtoul(ptr, NULL, 10);
 	if (errno)
-		return 14;
+		return 15;
 	// loginuid
 	*term = ' ';
 	str = strstr(term, "auid=");
 	if (str == NULL) {
 		str = strstr(term, "loginuid=");
 		if (str == NULL)
-			return 15;
+			return 16;
 		ptr = str + 9;
 	} else
 		ptr = str + 5;
 	term = strchr(ptr, ' ');
 	if (term == NULL)
-		return 16;
+		return 17;
 	*term = 0;
 	errno = 0;
 	s->loginuid = strtoul(ptr, NULL, 10);
 	if (errno)
-		return 17;
+		return 18;
 	// uid
 	*term = ' ';
 	str = strstr(term, "uid=");
 	if (str == NULL)
-		return 18;
+		return 19;
 	ptr = str + 4;
 	term = strchr(ptr, ' ');
 	if (term == NULL)
-		return 19;
+		return 20;
 	*term = 0;
 	errno = 0;
 	s->uid = strtoul(ptr, NULL, 10);
 	if (errno)
-		return 20;
+		return 21;
 	// gid
 	*term = ' ';
 	str = strstr(term, "gid=");
 	if (str == NULL)
-		return 21;
+		return 22;
 	ptr = str + 4;
 	term = strchr(ptr, ' ');
 	if (term == NULL)
-		return 22;
+		return 23;
 	*term = 0;
 	errno = 0;
 	s->gid = strtoul(ptr, NULL, 10);
 	if (errno)
-		return 23;
+		return 24;
 	// euid
 	*term = ' ';
 	str = strstr(term, "euid=");
 	if (str == NULL)
-		return 24;
+		return 25;
 	ptr = str + 5;
 	term = strchr(ptr, ' ');
 	if (term == NULL)
-		return 25;
+		return 26;
 	*term = 0;
 	errno = 0;
 	s->euid = strtoul(ptr, NULL, 10);
 	if (errno)
-		return 26;
+		return 27;
 	// egid
 	*term = ' ';
 	str = strstr(term, "egid=");
 	if (str == NULL)
-		return 27;
+		return 28;
 	ptr = str + 5;
 	term = strchr(ptr, ' ');
 	if (term == NULL)
-		return 28;
+		return 29;
 	*term = 0;
 	errno = 0;
 	s->egid = strtoul(ptr, NULL, 10);
 	if (errno)
-		return 29;
+		return 30;
 	*term = ' ';
+
+	if (event_terminal) {
+		// dont do this search unless needed
+		str = strstr(term, "tty=");
+		if (str) {
+			str += 4;
+			term = strchr(str, ' ');
+			if (term == NULL)
+				return 31;
+			*term = 0;
+			s->terminal = strdup(str);
+			*term = ' ';
+		}
+	}
 	if (event_comm) {
 		// dont do this search unless needed
-		str = strstr(n->message, "comm=");
+		str = strstr(term, "comm=");
 		if (str) {
 			/* Make the syscall one override */
 			if (s->comm)
@@ -365,13 +407,14 @@ static int parse_syscall(lnode *n, search_items *s)
 				str++;
 				term = strchr(str, '"');
 				if (term == NULL)
-					return 30;
+					return 32;
 				*term = 0;
 				s->comm = strdup(str);
 				*term = '"';
 			} else 
 				s->comm = unescape(str);
-		}
+		} else
+			return 33;
 	}
 	if (event_exe) {
 		// dont do this search unless needed
@@ -382,12 +425,49 @@ static int parse_syscall(lnode *n, search_items *s)
 				str++;
 				term = strchr(str, '"');
 				if (term == NULL)
-					return 31;
+					return 34;
 				*term = 0;
 				s->exe = strdup(str);
 				*term = '"';
 			} else 
 				s->exe = unescape(str);
+		} else
+			return 35;
+	}
+	if (event_subject) {
+		// scontext
+		str = strstr(term, "subj=");
+		if (str != NULL) {
+			str += 5;
+			term = strchr(str, ' ');
+			if (term == NULL)
+				return 36;
+			*term = 0;
+			if (audit_avc_init(s) == 0) {
+				anode an;
+
+				anode_init(&an);
+				an.scontext = strdup(str);
+				alist_append(s->avc, &an);
+				*term = ' ';
+			} else
+				return 37;
+		}
+	}
+	if (event_key) {
+		str = strstr(term, "key=");
+		if (str) {
+			str += 4;
+			if (*str == '"') {
+				str++;
+				term = strchr(str, '"');
+				if (term == NULL)
+					return 38;
+				*term = 0;
+				s->key = strdup(str);
+				*term = '"';
+			} else 
+				s->key = unescape(str);
 		}
 	}
 	return 0;
@@ -399,7 +479,7 @@ static int parse_dir(const lnode *n, search_items *s)
 
 	if (event_filename) {
 	// dont do this search unless needed
-		str = strstr(n->message, " cwd=");
+		str = strstr(n->message+NAME_OFFSET, " cwd=");
 		if (str) {
 			str += 5;
 			if (*str == '"') {
@@ -409,90 +489,134 @@ static int parse_dir(const lnode *n, search_items *s)
 				if (term == NULL)
 					return 1;
 				*term = 0;
-				if (!s->cwd)
+				if (!s->cwd) 
 					s->cwd = strdup(str);
 				*term = '"';
-			} else if (!s->cwd)
+			} else if (!s->cwd) 
 				s->cwd = unescape(str);
 		}
 	}
 	return 0;
 }
-static int parse_path(const lnode *n, search_items *s)
+
+static int common_path_parser(search_items *s, char *path)
 {
-	char *str, *term;
+	char *term;
+
+	if (*path == '"') {
+		/* string is normal */
+		path++;
+		term = strchr(path, '"');
+		if (term == NULL)
+			return 1;
+		if (!s->filename) {
+			//create
+			s->filename = malloc(sizeof(slist));
+			if (s->filename == NULL)
+				return 2;
+			slist_create(s->filename);
+		}
+		*term = 0;
+		if (s->filename) {
+			// append
+			snode sn;
+			sn.str = strdup(path);
+			sn.key = NULL;
+			sn.hits = 1;
+			// Attempt to rebuild path if relative
+			if ((sn.str[0] == '.') && ((sn.str[1] == '.') ||
+				(sn.str[1] == '/')) && s->cwd) {
+				char *tmp = malloc(PATH_MAX);
+				if (tmp == NULL)
+					return 3;
+				snprintf(tmp, PATH_MAX,
+					"%s/%s", s->cwd, sn.str);
+				free(sn.str);
+				sn.str = tmp;
+			}
+			slist_append(s->filename, &sn);
+		}
+		*term = '"';
+	} else { 
+		if (!s->filename) {
+			//create
+			s->filename = malloc(sizeof(slist));
+			if (s->filename == NULL)
+				return 4;
+			slist_create(s->filename);
+		}
+		if (s->filename) {
+			// append
+			snode sn;
+			sn.str = unescape(path);
+			sn.key = NULL;
+			sn.hits = 1;
+			// Attempt to rebuild path if relative
+			if ((sn.str[0] == '.') && ((sn.str[1] == '.') ||
+				(sn.str[1] == '/')) && s->cwd) {
+				char *tmp = malloc(PATH_MAX);
+				if (tmp == NULL)
+					return 5;
+				snprintf(tmp, PATH_MAX, "%s/%s", 
+					s->cwd, sn.str);
+				free(sn.str);
+				sn.str = tmp;
+			}
+			slist_append(s->filename, &sn);
+		}
+	}
+	return 0;
+}
+
+static int avc_parse_path(const lnode *n, search_items *s)
+{
+	char *str;
 
 	if (event_filename) {
 	// dont do this search unless needed
-		str = strstr(n->message, " name=");
+		str = strstr(n->message, " path=");
 		if (str) {
 			str += 6;
-			if (*str == '"') {
-				/* string is normal */
-				str++;
-				term = strchr(str, '"');
-				if (term == NULL)
-					return 1;
-				if (!s->filename) {
-					//create
-					s->filename = malloc(sizeof(slist));
-					if (s->filename == NULL)
-						return 2;
-					slist_create(s->filename);
-				}
+			return common_path_parser(s, str);
+		}
+		return 1;
+	}
+	return 0;
+}
+
+static int parse_path(const lnode *n, search_items *s)
+{
+	// We add 32 to message because we do not nee to look at
+	// anything before that. Its only time and type.
+	char *str, *term = n->message+NAME_OFFSET;
+
+	if (event_filename) {
+		// dont do this search unless needed
+		str = strstr(term, " name=");
+		if (str) {
+			str += 6;
+			if (common_path_parser(s, str))
+				return 1;
+		}
+	}
+	if (event_object) {
+		// tcontext
+		str = strstr(term, "obj=");
+		if (str != NULL) {
+			str += 4;
+			term = strchr(str, ' ');
+			if (term)
 				*term = 0;
-				if (s->filename) {
-					// append
-					snode sn;
-					sn.str = strdup(str);
-					sn.key = NULL;
-					sn.hits = 1;
-					// Attempt to rebuild path if relative
-					if ((sn.str[0] == '.') &&
-						((sn.str[1] == '.') ||
-						(sn.str[1] == '/')) && s->cwd) {
-						char *tmp = malloc(PATH_MAX);
-						if (tmp == NULL)
-							return 3;
-						snprintf(tmp, PATH_MAX,
-							"%s/%s", s->cwd,
-							sn.str);
-						free(sn.str);
-						sn.str = tmp;
-					}
-					slist_append(s->filename, &sn);
-				}
-				*term = '"';
-			} else { 
-				if (!s->filename) {
-					//create
-					s->filename = malloc(sizeof(slist));
-					if (s->filename == NULL)
-						return 4;
-					slist_create(s->filename);
-				}
-				if (s->filename) {
-					// append
-					snode sn;
-					sn.str = unescape(str);
-					sn.key = NULL;
-					sn.hits = 1;
-					// Attempt to rebuild path if relative
-					if ((sn.str[0] == '.') &&
-						((sn.str[1] == '.') ||
-						(sn.str[1] == '/')) && s->cwd) {
-						char *tmp = malloc(PATH_MAX);
-						if (tmp == NULL)
-							return 5;
-						snprintf(tmp, PATH_MAX,
-							"%s/%s", s->cwd,
-							sn.str);
-						free(sn.str);
-						sn.str = tmp;
-					}
-					slist_append(s->filename, &sn);
-				}
-			}
+			if (audit_avc_init(s) == 0) {
+				anode an;
+
+				anode_init(&an);
+				an.tcontext = strdup(str);
+				alist_append(s->avc, &an);
+				if (term)
+					*term = ' ';
+			} else
+				return 6;
 		}
 	}
 	return 0;
@@ -549,6 +673,26 @@ static int parse_user(const lnode *n, search_items *s)
 	if (errno)
 		return 9;
 	*term = ' ';
+	if (event_subject) {
+		// scontext
+		str = strstr(term, "subj=");
+		if (str != NULL) {
+			str += 5;
+			term = strchr(str, ' ');
+			if (term == NULL)
+				return 10;
+			*term = 0;
+			if (audit_avc_init(s) == 0) {
+				anode an;
+
+				anode_init(&an);
+				an.scontext = strdup(str);
+				alist_append(s->avc, &an);
+				*term = ' ';
+			} else
+				return 11;
+		}
+	}
 	// get uid - something has uid after auid ??
 	str = strstr(term, "uid=");
 	if (str != NULL) {
@@ -557,29 +701,29 @@ static int parse_user(const lnode *n, search_items *s)
 		while (isdigit(*term))
 			term++;
 		if (term == ptr)
-			return 10;
+			return 12;
 
 		saved = *term;
 		*term = 0;
 		errno = 0;
 		s->uid = strtoul(ptr, NULL, 10);
 		if (errno)
-			return 11;
+			return 13;
 		*term = saved;
 	} else {
 		str = strstr(term, "acct=");
 		if (str != NULL) {
 			ptr = str + 5;
-			term = ptr;
-			while (*term != ':' && *term != ' ')
-				term++;
-			if (term == ptr)
-				return 12;
 
-			saved = *term;
-			*term = 0;
-			s->acct = strdup(ptr);
-			*term = saved;
+			term = ptr + 1;
+			if (*ptr == '"') {
+				while (term != '"')
+					term++;
+				*term = 0;
+				s->acct = strdup(str);
+				*term = '"';
+			} else 
+				s->acct = unescape(str);
 		}
 	}
 	mptr = term + 1;
@@ -592,7 +736,7 @@ static int parse_user(const lnode *n, search_items *s)
 			str += 9;
 			term = strchr(str, ',');
 			if (term == NULL)
-				return 13;
+				return 15;
 			*term = 0;
 			s->hostname = strdup(str);
 			*term = ',';
@@ -606,7 +750,7 @@ static int parse_user(const lnode *n, search_items *s)
 					str += 5;
 					term = strchr(str, ',');
 					if (term == NULL)
-						return 14;
+						return 16;
 					*term = 0;
 					free(s->hostname);
 					s->hostname = strdup(str);
@@ -622,7 +766,7 @@ static int parse_user(const lnode *n, search_items *s)
 			str += 9;
 			term = strchr(str, ' ');
 			if (term == NULL)
-				return 15;
+				return 17;
 			*term = 0;
 			s->terminal = strdup(str);
 			*term = ' ';
@@ -637,7 +781,7 @@ static int parse_user(const lnode *n, search_items *s)
 				str++;
 				term = strchr(str, '"');
 				if (term == NULL)
-					return 16;
+					return 18;
 				*term = 0;
 				s->exe = strdup(str);
 				*term = '"';
@@ -652,23 +796,23 @@ static int parse_user(const lnode *n, search_items *s)
 		ptr = str + 4;
 		term = strchr(ptr, '\'');
 		if (term == NULL)
-			return 17;
+			return 19;
 		*term = 0;
 		if (strncmp(ptr, "failed", 6) == 0)
-			s->success = 0;
+			s->success = S_FAILED;
 		else
-			s->success = 1;
+			s->success = S_SUCCESS;
 		*term = '\'';
 	} else if ((str = strstr(mptr, "result="))) {
 		ptr = str + 7;
 		term = strchr(ptr, ')');
 		if (term == NULL)
-			return 18;
+			return 20;
 		*term = 0;
 		if (strcasecmp(ptr, "success") == 0)
-			s->success = 1;
+			s->success = S_SUCCESS;
 		else
-			s->success = 0;
+			s->success = S_FAILED;
 		*term = ')';
 	}
 	return 0;
@@ -724,6 +868,10 @@ static int parse_login(const lnode *n, search_items *s)
 		return 8;
 	if (term)
 		*term = ' ';
+
+	// Login means the kernel changed loginuid, its always
+	// successful at this point.
+	s->success = S_SUCCESS;
 	return 0;
 }
 
@@ -742,14 +890,8 @@ static int parse_daemon(const lnode *n, search_items *s)
 		return 2;
 	ptr = str + 5;
 	term = strchr(ptr, ' ');
-	if (term == NULL) {
-		term = strchr(ptr, ',');
-		if (term == NULL) { 
-			term = ptr;
-			while (*term)
-				term++;
-		}
-	}
+	if (term == NULL)
+		return 3;
 	saved = *term;
 	*term = 0;
 	errno = 0;
@@ -758,6 +900,43 @@ static int parse_daemon(const lnode *n, search_items *s)
 		return 4;
 	*term = saved;
 
+	// pid
+	str = strstr(term, "pid=");
+	if (str == NULL)
+		return 5;
+	ptr = str + 4;
+	term = strchr(ptr, ' ');
+	if (term == NULL) 
+		return 6;
+	saved = *term;
+	*term = 0;
+	errno = 0;
+	s->pid = strtoul(ptr, NULL, 10);
+	if (errno)
+		return 7;
+	*term = saved;
+
+	if (event_subject) {
+		// scontext
+		str = strstr(term, "subj=");
+		if (str != NULL) {
+			str += 5;
+			term = strchr(str, ' ');
+			if (term == NULL)
+				return 8;
+			*term = 0;
+			if (audit_avc_init(s) == 0) {
+				anode an;
+
+				anode_init(&an);
+				an.scontext = strdup(str);
+				alist_append(s->avc, &an);
+				*term = ' ';
+			} else
+				return 9;
+		}
+	}
+
 	// success
 	str = strstr(mptr, "res=");
 	if (str) {
@@ -765,101 +944,16 @@ static int parse_daemon(const lnode *n, search_items *s)
 		while (isalpha(*term))
 			term++;
 		if (term == ptr)
-			return 5;
+			return 10;
 		saved = *term;
 		*term = 0;
 		if (strncmp(ptr, "failed", 6) == 0)
-			s->success = 0;
+			s->success = S_FAILED;
 		else
-			s->success = 1;
+			s->success = S_SUCCESS;
 		*term = saved;
 	}
 
-	// pid
-	str = strstr(mptr, "pid=");
-	if (str == NULL) 
-		return 6;
-	ptr = str + 4;
-	term = strchr(ptr, ' ');
-	if (term)
-		*term = 0;
-	errno = 0;
-	s->pid = strtoul(ptr, NULL, 10);
-	if (errno)
-		return 7;
-	if (term) 
-		*term = ' ';
-
-	return 0;
-}
-
-static int parse_fs_watch(const lnode *n, search_items *s)
-{
-	char *str, *term;
-
-	if (event_filename) {
-	// dont do this search unless needed
-		term = n->message;
-		str = strstr(term, "watch=");
-		if (str) {
-			str += 6;
-			if (*str == '"') {
-				str++;
-				term = strchr(str, '"');
-				if (term == NULL)
-					return 1;
-				if (!s->filename) {
-					//create
-					s->filename = malloc(sizeof(slist));
-					if (s->filename == NULL)
-						return 2;
-					slist_create(s->filename);
-				}
-				*term = 0;
-				if (s->filename) {
-					// append
-					snode sn;
-					sn.str = strdup(str);
-					sn.key = NULL;
-					sn.hits = 1;
-					slist_append(s->filename, &sn);
-				}
-				*term = '"';
-			} else { 
-				if (!s->filename) {
-					//create
-					s->filename = malloc(sizeof(slist));
-					if (s->filename == NULL)
-						return 3;
-					slist_create(s->filename);
-				}
-				if (s->filename) {
-					// append
-					snode sn;
-					sn.str = unescape(str);
-					sn.key = NULL;
-					sn.hits = 1;
-					slist_append(s->filename, &sn);
-				}
-				term = str;
-			}
-		}
-		if (event_key) {
-			str = strstr(term, "filterkey=");
-			if (str && s->filename) {
-				str += 10;
-				term = str;
-				while (*term != ' ')
-					term++;
-				if (term != str) {
-					char saved = *term;
-					*term = 0;
-					slist_add_key(s->filename, str);
-					*term = saved;
-				}
-			}
-		}
-	}
 	return 0;
 }
 
@@ -960,11 +1054,13 @@ static int parse_sockaddr(const lnode *n, search_items *s)
 static int parse_avc(const lnode *n, search_items *s)
 {
 	char *str, *term;
+	anode an;
+	int rc=0;
 
 	term = n->message;
+	anode_init(&an);
 
-	// get the avc message info. Think about flag to skip this.
-	// its likely only needed for anything except ausearch.
+	// get the avc message info.
 	str = strstr(term, "avc: ");
 	if (str) {
 		str += 5;
@@ -972,12 +1068,14 @@ static int parse_avc(const lnode *n, search_items *s)
 		if (term == NULL)
 			return 1;
 		*term = 0;
+		// FIXME. Do not override syscall pass/fail if its already
+		// set. Syscall pass/fail is the authoritative value.
 		if (strstr(str, "denied")) {
-			s->success = 0; 
-			s->avc_result = AVC_DENIED;
+			s->success = S_FAILED; 
+			an.avc_result = AVC_DENIED;
 		} else {
-			s->success = 1;
-			s->avc_result = AVC_GRANTED;
+			s->success = S_SUCCESS;
+			an.avc_result = AVC_GRANTED;
 		}
 		*term = '{';
 
@@ -991,22 +1089,25 @@ static int parse_avc(const lnode *n, search_items *s)
 		while (*(term-1) == ' ')
 			term--;
 		*term = 0;
-		if (s->avc_perm == NULL)
-			s->avc_perm = strdup(str);
+		an.avc_perm = strdup(str);
 		*term = ' ';
 	}
 
 	if (event_comm && s->comm == NULL) {
 	// dont do this search unless needed
 		str = strstr(term, "comm=");
-		if (str == NULL)
-			return 3;
+		if (str == NULL) {
+			rc = 3;
+			goto err;
+		}
 		str += 5;
 		if (*str == '"') {
 			str++;
 			term = strchr(str, '"');
-			if (term == NULL)
-				return 4;
+			if (term == NULL) {
+				rc = 4;
+				goto err;
+			}
 			*term = 0;
 			s->comm = strdup(str);
 			*term = '"';
@@ -1015,50 +1116,63 @@ static int parse_avc(const lnode *n, search_items *s)
 			term = str + 6;
 		}
 	}
-	if (event_subject && s->scontext == NULL) {
+	if (event_subject) {
 		// scontext
 		str = strstr(term, "scontext=");
 		if (str != NULL) {
 			str += 9;
 			term = strchr(str, ' ');
-			if (term == NULL)
-				return 5;
+			if (term == NULL) {
+				rc = 5;
+				goto err;
+			}
 			*term = 0;
-			s->scontext = strdup(str);
+			an.scontext = strdup(str);
 			*term = ' ';
 		}
 	}
 
-	if (event_object && s->tcontext == NULL) {
+	if (event_object) {
 		// tcontext
 		str = strstr(term, "tcontext=");
 		if (str != NULL) {
 			str += 9;
 			term = strchr(str, ' ');
-			if (term == NULL)
-				return 6;
+			if (term == NULL) {
+				rc = 6;
+				goto err;
+			}
 			*term = 0;
-			s->tcontext = strdup(str);
+			an.tcontext = strdup(str);
 			*term = ' ';
 		}
 	}
 
 	// Now get the class...its at the end, so we do things different
 	str = strstr(term, "tclass=");
-	if (str == NULL)
-		return 7;
+	if (str == NULL) {
+		rc = 7;
+		goto err;
+	}
 	str += 7;
 	term = strchr(str, ' ');
 	if (term)
 		*term = 0;
-	/* FIXME: We sometimes get more than one avc_class. This might
-	   need to be on a list */
-	if (s->avc_class)
-		free(s->avc_class);
-	s->avc_class = strdup(str);
+	an.avc_class = strdup(str);
 	if (term)
 		*term = ' ';
+
+	if (audit_avc_init(s) == 0) {
+		alist_append(s->avc, &an);
+	} else {
+		rc = 8;
+		goto err;
+	}
+
 	return 0;
+err:
+	anode_clear(&an);
+	return rc;
 }
 
 static int parse_kernel_anom(const lnode *n, search_items *s)
@@ -1079,6 +1193,93 @@ static int parse_kernel_anom(const lnode *n, search_items *s)
 		return 2;
 	if (term)
 		*term = ' ';
+	else
+		term = n->message;
+
+	// get uid
+	str = strstr(term, "uid=");
+	if (str) {
+		ptr = str + 4;
+		term = strchr(ptr, ' ');
+		if (term == NULL)
+			return 3;
+		*term = 0;
+		errno = 0;
+		s->uid = strtoul(ptr, NULL, 10);
+		if (errno)
+			return 4;
+		*term = ' ';
+	}
+
+	// get gid
+	str = strstr(term, "gid=");
+	if (str) {
+		ptr = str + 4;
+		term = strchr(ptr, ' ');
+		if (term == NULL)
+			return 5;
+		*term = 0;
+		errno = 0;
+		s->gid = strtoul(ptr, NULL, 10);
+		if (errno)
+			return 6;
+		*term = ' ';
+	}
+
+	if (event_subject) {
+		// scontext
+		str = strstr(term, "subj=");
+		if (str) {
+			str += 5;
+			term = strchr(str, ' ');
+			if (term == NULL)
+				return 7;
+			*term = 0;
+			if (audit_avc_init(s) == 0) {
+				anode an;
+
+				anode_init(&an);
+				an.scontext = strdup(str);
+				alist_append(s->avc, &an);
+				*term = ' ';
+			} else
+				return 8;
+		}
+	}
+
+	// get pid
+	str = strstr(term, "pid=");
+	if (str) {
+		ptr = str + 4;
+		term = strchr(ptr, ' ');
+		if (term == NULL)
+			return 9;
+		*term = 0;
+		errno = 0;
+		s->pid = strtoul(ptr, NULL, 10);
+		if (errno)
+			return 10;
+		*term = ' ';
+	}
+
+	if (event_comm) {
+		// dont do this search unless needed
+		str = strstr(term, "comm=");
+		if (str) {
+			str += 5;
+			if (*str == '"') {
+				str++;
+				term = strchr(str, '"');
+				if (term == NULL)
+					return 11;
+				*term = 0;
+				s->comm = strdup(str);
+				*term = '"';
+			} else 
+				s->comm = unescape(str);
+		} 
+	}
+
 	return 0;
 }
 
@@ -1102,5 +1303,54 @@ static int parse_simple_message(const lnode *n, search_items *s)
 		return 2;
 	if (term)
 		*term = ' ';
+	else
+		term = ptr;
+
+	// Now get subj label
+	if (event_subject) {
+		// scontext
+		str = strstr(term, "subj=");
+		if (str != NULL) {
+			str += 5;
+			term = strchr(str, ' ');
+			if (term)
+				*term = 0;
+			if (audit_avc_init(s) == 0) {
+				anode an;
+
+				anode_init(&an);
+				an.scontext = strdup(str);
+				alist_append(s->avc, &an);
+				if (term)
+					*term = ' ';
+				else	// Set it back to something sane
+					term = str;
+			} else
+				return 3;
+		}
+	}
+
+	// defaulting this to 1 for these messages. The kernel generally
+	// does not log the res since it can be nothing but success. 
+	// But it can still be overriden below if res= is found in the event
+	if (n->type == AUDIT_CONFIG_CHANGE) 
+		s->success = S_SUCCESS;
+
+	// and results (usually last)
+	str = strstr(term, "res=");
+	if (str != NULL) {
+		ptr = str + 4;
+		term = strchr(ptr, ' ');
+		if (term)
+			*term = 0;
+		errno = 0;
+		s->success = strtoul(ptr, NULL, 10);
+		if (errno)
+			return 4;
+		if (term)
+			*term = ' ';
+	}
+
 	return 0;
 }
+
