@@ -1,6 +1,6 @@
 /*
 * ausearch-report.c - Format and output events
-* Copyright (c) 2005-06 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005-07 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -49,19 +49,14 @@ struct nv_pair {
 
 /* This is the list of field types that we can interpret */
 enum { T_UID, T_GID, T_SYSCALL, T_ARCH, T_EXIT, T_ESCAPED, T_PERM, T_MODE, 
-T_SOCKADDR, T_FLAGS, T_PROMISC };
-
-/* These are all related to the fs watch code */
-#define WATCH_MAY_EXEC          1
-#define WATCH_MAY_WRITE         2
-#define WATCH_MAY_READ          4
-#define WATCH_MAY_APPEND        8
+T_SOCKADDR, T_FLAGS, T_PROMISC, T_CAPABILITY, T_SIGNAL };
 
 /* Function in ausearch-match for unescaping filenames */
 extern char *unescape(char *buf);
 
 /* Local functions */
 static void output_raw(llist *l);
+static void output_default(llist *l);
 static void output_interpreted(llist *l);
 static void output_interpreted_node(const lnode *n);
 static void interpret(char *name, char *val, int comma);
@@ -79,6 +74,9 @@ void output_record(llist *l)
 		case RPT_RAW:
 			output_raw(l);
 			break;
+		case RPT_DEFAULT:
+			output_default(l);
+			break;
 		case RPT_INTERP:
 			output_interpreted(l);
 			break;
@@ -90,11 +88,27 @@ void output_record(llist *l)
 	}
 }
 
+/* This function will output the record as is */
+static void output_raw(llist *l)
+{
+	const lnode *n;
+
+	list_first(l);
+	n = list_get_cur(l);
+	if (!n) {
+		fprintf(stderr, "Error - no elements in record.");
+		return;
+	}
+	do {
+		printf("%s\n", n->message);
+	} while ((n=list_next(l)));
+}
+
 /*
  * This function will take the linked list and format it for output. No
  * interpretation is performed. The output order is lifo for everything.
  */
-static void output_raw(llist *l)
+static void output_default(llist *l)
 {
 	const lnode *n;
 
@@ -154,39 +168,39 @@ static void output_interpreted_node(const lnode *n)
 		return;
 	} else {
 		time_t t;
-		int milli;
+		int milli,num = n->type;
 		unsigned long serial;
 		struct tm *btm;
 		char tmp[32];
+		const char *bptr;
 
 		*ptr++ = 0;
-		if (n->type == -1) {
+		if (num == -1) {
 			// see if we are older and wiser now.
-			const char *bptr = strchr(str, '[');
+			bptr = strchr(str, '[');
 			if (bptr && bptr < ptr) {
 				char *eptr;
 				bptr++;
 				eptr = strchr(bptr, ']');
 				if (eptr) {
-					int num;
 					*eptr = 0;
 					errno = 0;
 					num = strtoul(bptr, NULL, 10);
 					*eptr = ']';
-					if (errno == 0) {
-						bptr =
-						    audit_msg_type_to_name(num);
-						if (bptr) {
-							printf(
-							  "type=%s msg=audit(",
-								 bptr);
-							goto no_print;
-						}
-					}
+					if (errno) 
+						num = -1;
 				}
 			}
 		}
+
 		// print everything up to it.
+		if (num >= 0) {
+			bptr = audit_msg_type_to_name(num);
+			if (bptr) {
+				printf("type=%s msg=audit(", bptr);
+				goto no_print;
+			}
+		} 
 		printf("%s(", str);
 no_print:
 
@@ -309,6 +323,7 @@ static struct nv_pair typetab[] = {
 	{T_ESCAPED, "name"},
 	{T_ESCAPED, "watch"},
 	{T_ESCAPED, "cwd"},
+	{T_ESCAPED, "cmd"},
 	{T_PERM, "perm"},
 	{T_PERM, "perm_mask"},
 	{T_MODE, "mode"},
@@ -316,6 +331,8 @@ static struct nv_pair typetab[] = {
 	{T_FLAGS, "flags"},
 	{T_PROMISC, "prom"},
 	{T_PROMISC, "old_prom"},
+	{T_CAPABILITY, "capability"},
+	{T_SIGNAL, "sig"},
 };
 #define TYPE_NAMES (sizeof(typetab)/sizeof(typetab[0]))
 
@@ -385,8 +402,9 @@ static void print_syscall(const char *val)
 	const char *sys;
 	int ival;
 
-	if (machine < 0) {
+	if (machine <= 0) 
 		machine = audit_detect_machine();
+	if (machine < 0) {
 		printf("%s ", val);
 		return;
 	}
@@ -442,6 +460,16 @@ static void print_escaped(char *val)
 			return;
 		*term = 0;
 		printf("%s ", val);
+// FIXME: working here...was trying to detect (null) and handle that differently
+// The other 2 should have " around the file names.
+/*	} else if (*val == '(') {
+		char *term;
+		val++;
+		term = strchr(val, ' ');
+		if (term == NULL)
+			return;
+		*term = 0;
+		printf("%s ", val); */
 	} else {
 		str = unescape(val);
 		printf("%s ", str);
@@ -464,29 +492,29 @@ static void print_perm(const char *val)
 	if (ival == 0)
 		ival = 0x0F;
 
-	if (ival & WATCH_MAY_READ) {
+	if (ival & AUDIT_PERM_READ) {
 		printf("read");
 		printed = 1;
 	}
-	if (ival & WATCH_MAY_WRITE) {
+	if (ival & AUDIT_PERM_WRITE) {
 		if (printed)
 			printf(",write");
 		else
 			printf("write");
 		printed = 1;
 	}
-	if (ival & WATCH_MAY_EXEC) {
+	if (ival & AUDIT_PERM_EXEC) {
 		if (printed)
 			printf(",exec");
 		else
 			printf("exec");
 		printed = 1;
 	}
-	if (ival & WATCH_MAY_APPEND) {
+	if (ival & AUDIT_PERM_ATTR) {
 		if (printed)
-			printf(",append");
+			printf(",attr");
 		else
-			printf("append");
+			printf("attr");
 	}
 	printf(" ");
 }
@@ -738,6 +766,77 @@ static void print_promiscuous(const char *val)
 		printf("yes ");
 }
 
+/*
+ * This table maps file system flags to their text name
+ */
+static struct nv_pair captab[] = {
+        {0, "chown"},
+        {1, "dac_override"},
+        {2, "dac_read_search"},
+        {3, "fowner"},
+        {4, "fsetid"},
+        {5, "kill"},
+        {6, "setgid"},
+        {7, "setuid"},
+        {8, "setpcap"},
+        {9, "linux_immutable"},
+        {10, "net_bind_service"},
+        {11, "net_broadcast"},
+        {12, "net_admin"},
+        {13, "net_raw"},
+        {14, "ipc_lock"},
+        {15, "ipc_owner"},
+        {16, "sys_module"},
+        {17, "sys_rawio"},
+        {18, "sys_chroot"},
+        {19, "sys_ptrace"},
+        {20, "sys_pacct"},
+        {21, "sys_admin"},
+        {22, "sys_boot"},
+        {23, "sys_nice"},
+        {24, "sys_resource"},
+        {25, "sys_time"},
+        {26, "sys_tty_config"},
+        {27, "mknod"},
+        {28, "lease"},
+        {29, "audit_write"},
+        {30, "audit_control"},
+};
+#define CAP_NAMES (sizeof(captab)/sizeof(captab[0]))
+
+static void print_capabilities(char *val)
+{
+	int cap, i;
+
+	errno = 0;
+	cap = strtoul(val, NULL, 10);
+	if (errno) {
+		printf("conversion error(%s) ", val);
+		return;
+	}
+
+        for (i = 0; i < CAP_NAMES; i++) {
+                if (captab[i].value == cap) {
+                        printf("%s ", captab[i].name);
+			return;
+		}
+
+	}
+}
+
+static void print_signals(char *val)
+{
+	int i;
+
+	errno = 0;
+	i = strtoul(val, NULL, 10);
+	if (errno) {
+		printf("conversion error(%s)", val);
+		return;
+	}
+	printf("%s", strsignal(i));
+}
+
 static void interpret(char *name, char *val, int comma)
 {
 	int type;
@@ -780,6 +879,12 @@ static void interpret(char *name, char *val, int comma)
 		case T_PROMISC:
 			print_promiscuous(val);
 			break;
+		case T_CAPABILITY:
+			print_capabilities(val);
+			break;
+		case T_SIGNAL:
+			print_signals(val);
+			break;
 		default:
 			printf("%s%c", val, comma ? ',' : ' ');
 	}
