@@ -1,6 +1,6 @@
 /*
 * ausearch-string.c - Minimal linked list library for strings
-* Copyright (c) 2005-06 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved. 
 *
 * This software may be freely redistributed and/or modified under the
@@ -98,13 +98,13 @@ void slist_append(slist *l, snode *node)
 	l->cnt++;
 }
 
-void slist_add_key(slist *l, const char *key)
+/* void slist_add_key(slist *l, const char *key)
 {
 	if (l && l->cur) {
 		free(l->cur->key);
 		l->cur->key = strdup(key);
 	}
-}
+} */
 
 int slist_find_item(slist *l, unsigned int i)
 {
@@ -144,6 +144,7 @@ void slist_clear(slist* l)
 	l->cnt = 0;
 }
 
+/* This function dominates the timing of aureport. Needs to be more efficient */
 int slist_add_if_uniq(slist *l, const char *str)
 {
 	snode sn;
