@@ -34,8 +34,12 @@ struct nv_pair {
 
 static struct nv_pair timetab[] = {
         { T_NOW, "now" },
+        { T_RECENT, "recent" },
         { T_TODAY, "today" },
         { T_YESTERDAY, "yesterday" },
+        { T_THIS_WEEK, "this-week" },
+        { T_THIS_MONTH, "this-month" },
+        { T_THIS_YEAR, "this-year" },
 };
 #define TIME_NAMES (sizeof(timetab)/sizeof(timetab[0]))
 
@@ -43,10 +47,13 @@ int lookup_time(const char *name)
 {
         int i;
 
-        for (i = 0; i < TIME_NAMES; i++)
-                if (!strcmp(timetab[i].name, name))
+        for (i = 0; i < TIME_NAMES; i++) {
+                if (strcmp(timetab[i].name, name) == 0) {
                         return timetab[i].value;
+		}
+	}
         return -1;
+
 }
 
 void clear_tm(struct tm *t)
@@ -88,6 +95,49 @@ void set_tm_yesterday(struct tm *d)
 	replace_date(d, tv);
 }
 
+void set_tm_recent(struct tm *d)
+{
+        time_t t = time(NULL) - (time_t)(10*60); /* 10 minutes ago */
+        struct tm *tv = localtime(&t);
+	replace_time(d, tv);
+	replace_date(d, tv);
+}
+
+void set_tm_this_week(struct tm *d)
+{
+        time_t t = time(NULL);
+        struct tm *tv = localtime(&t);
+        d->tm_sec = 1;          /* seconds */
+        d->tm_min = 0;          /* minutes */
+        d->tm_hour = 0;         /* hours */
+	t -= (time_t)(tv->tm_wday*24*60*60);
+        tv = localtime(&t);
+	replace_date(d, tv);
+}
+
+void set_tm_this_month(struct tm *d)
+{
+        time_t t = time(NULL);
+        struct tm *tv = localtime(&t);
+        d->tm_sec = 1;          /* seconds */
+        d->tm_min = 0;          /* minutes */
+        d->tm_hour = 0;         /* hours */
+	replace_date(d, tv);
+        d->tm_mday = 1;         /* override day of month */
+}
+
+void set_tm_this_year(struct tm *d)
+{
+        time_t t = time(NULL);
+        struct tm *tv = localtime(&t);
+        d->tm_sec = 1;          /* seconds */
+        d->tm_min = 0;          /* minutes */
+        d->tm_hour = 0;         /* hours */
+	replace_date(d, tv);
+        d->tm_mday = 1;         /* override day of month */
+        d->tm_mon = 0;          /* override month */
+}
+
 /* Combine date & time into 1 struct. Results in date. */
 void add_tm(struct tm *d, struct tm *t)
 {
@@ -151,18 +201,37 @@ static int lookup_and_set_time(const char *da, struct tm *d)
 			case T_NOW:
 				set_tm_now(d);
 				break;
+			case T_RECENT:
+				set_tm_recent(d);
+				break;
 			case T_TODAY:
 				set_tm_today(d);
 				break;
 			case T_YESTERDAY:
 				set_tm_yesterday(d);
 				break;
+			case T_THIS_WEEK:
+				set_tm_this_week(d);
+				break;
+			case T_THIS_MONTH:
+				set_tm_this_month(d);
+				break;
+			case T_THIS_YEAR:
+				set_tm_this_year(d);
+				break;
 		}
 		return 0;
 	} else
 		return -1;
 }
 
+/* static void print_time(struct tm *d)
+{
+	char outstr[200];
+	strftime(outstr, sizeof(outstr), "%c", d);
+	printf("%s\n", outstr);
+} */
+
 int ausearch_time_start(const char *da, const char *ti)
 {
 /* If da == NULL, use current date */
@@ -189,6 +258,7 @@ int ausearch_time_start(const char *da, const char *ti)
 			}
 		}
 	}
+
 	if (ti != NULL) {
 		char tmp_t[9];
 
@@ -211,6 +281,7 @@ int ausearch_time_start(const char *da, const char *ti)
 		}
 	} else
 		clear_tm(&t);
+
 	add_tm(&d, &t);
 	if (d.tm_year < 104) {
 		fprintf(stderr, "Error - year is %d\n", d.tm_year+1900);
