@@ -1,6 +1,6 @@
 /*
  * ausearch.c - main file for ausearch utility 
- * Copyright 2005-06 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2005-07 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -42,7 +42,10 @@
 
 static FILE *log_fd = NULL;
 static int found = 0;
+static int pipe_mode = 0;
 static int process_logs(void);
+static int process_log_fd(void);
+static int process_stdin(void);
 static int process_file(char *filename);
 static int get_record(llist *);
 static void extract_timestamp(const char *b, event *e);
@@ -53,6 +56,23 @@ extern char *user_file;
 extern int match(llist *l);
 extern void output_record(llist *l);
 
+static int input_is_pipe(void)
+{
+	struct stat st;
+
+	if (fstat(0, &st) == 0) {
+		if (S_ISFIFO(st.st_mode)) {
+			if (user_file) {
+				fprintf(stderr,
+				    "-if option conflicts with stdin method");
+				exit(1);
+			}
+			pipe_mode = 1;
+		}
+	}
+	return pipe_mode;
+}
+
 int main(int argc, char *argv[])
 {
 	struct rlimit limit;
@@ -72,7 +92,9 @@ int main(int argc, char *argv[])
 	set_aumessage_mode(MSG_STDERR, DBG_NO);
 	(void) umask( umask( 077 ) | 027 );
 
-	if (user_file)
+	if (input_is_pipe())
+		rc = process_stdin();
+	else if (user_file)
 		rc = process_file(user_file);
 	else
 		rc = process_logs();
@@ -82,7 +104,8 @@ int main(int argc, char *argv[])
 	if (rc)
 		return rc;
 	if (!found) {
-		fprintf(stderr, "<no matches>\n");
+		if (report_format != RPT_RAW)
+			fprintf(stderr, "<no matches>\n");
 		return 1;
 	}
 	return 0;
@@ -148,18 +171,11 @@ static int process_logs(void)
 	return 0;
 }
 
-static int process_file(char *filename)
+static int process_log_fd(void)
 {
 	llist entries; // entries in a record
 	int ret;
 
-	log_fd = fopen(filename, "r");
-	if (log_fd == NULL) {
-		fprintf(stderr, "Error opening %s (%s)\n", filename, 
-			strerror(errno));
-		return 1;
-	}
-
 	/* For each record in file */
 	list_create(&entries);
 	do {
@@ -177,6 +193,25 @@ static int process_file(char *filename)
 	return 0;
 }
 
+static int process_stdin(void)
+{
+	log_fd = stdin;
+
+	return process_log_fd();
+}
+
+static int process_file(char *filename)
+{
+	log_fd = fopen(filename, "r");
+	if (log_fd == NULL) {
+		fprintf(stderr, "Error opening %s (%s)\n", filename, 
+			strerror(errno));
+		return 1;
+	}
+
+	return process_log_fd();
+}
+
 /*
  * This function returns a malloc'd buffer of the next record in the audit
  * logs. It returns 0 on success, 1 on eof, -1 on error. 
@@ -205,6 +240,9 @@ static int get_record(llist *l)
 				if (!buff)
 					return -1;
 			}
+			// FIXME: In pipe mode, if there is a waiting buffer
+			// and 5 seconds has elapsed, go ahead and process
+			// the buffer - nothings coming that's related.
 			rc = fgets_unlocked(buff, MAX_AUDIT_MESSAGE_LENGTH,
 					log_fd);
 		}
@@ -213,7 +251,7 @@ static int get_record(llist *l)
 			event e;
 			char *ptr;
 
-			ptr = strchr(buff, 0x0a);
+			ptr = strrchr(buff, 0x0a);
 			if (ptr)
 				*ptr = 0;
 			n.message=strdup(buff);
@@ -251,13 +289,16 @@ static void extract_timestamp(const char *b, event *e)
 {
 	char *ptr, *tmp;
 
-	tmp = strdupa(b);
+	tmp = strndupa(b, 80);
 	ptr = strtok(tmp, " ");
 	if (ptr) {
 		// at this point we have type=
 		ptr = strtok(NULL, " ");
 		if (ptr) {
-			ptr = strchr(ptr, '(');
+			if (*(ptr+9) == '(')
+				ptr+=9;
+			else
+				ptr = strchr(ptr, '(');
 			if (ptr) {
 			// now we should be pointed at the timestamp
 				char *eptr;
@@ -283,7 +324,7 @@ static int str2event(char *s, event *e)
 	char *ptr;
 
 	errno = 0;
-	ptr = strchr(s, ':');
+	ptr = strchr(s+10, ':');
 	if (ptr) {
 		e->serial = strtoul(ptr+1, NULL, 10);
 		*ptr = 0;
